# apiInAsm.pl - assembly-language specifics for apigen
#
# Copyright 2003, 2004 Wind River Systems, Inc.
#
# modification history
# --------------------
# 01e,05oct04,wsl  fix format of error messages
# 01d,30sep04,wsl  add line numbers to error messages, SPR 93816
# 01c,14sep04,wsl  fix SPR 97660, extra spaces after routine separator comment
#                  cause loss of routine
# 01b,21mar03,wsl  add langSynopsisRequired variable
# 01a,06mar03,wsl  written
#
# DESCRIPTION
# This script sets up the variables and routines required for handling input
# files in assembly language.
#
# The following variables must be defined by any "apiIn" file:
# \is
# \i $langLibStart
# Pattern that starts a library header
# (line-oriented - see apiInC.pl for an example).
# Used to find the beginning of real text in libraries.
#
# \i $langCommentEnd
# String that ends a comment
# (string-oriented - like '\*\/' or "").
# Used to eliminate the comment end from title line.
#
# \i $langHeaderEnd
# Pattern that ends a comment header
# (line-oriented - like '\*\/$' or '\s*([^#].*)?$').
# Used to find the end of all comment headers.
#
# \i $langCommentPrefix
# String that begins lines in continuing comment
# (string-oriented - like '\*' or '#').
# Used to remove markup at the start of each routine line.
#
# \i $langRoutineStart
# Pattern that starts a routine's comments
# (text-oriented - like '\/\*{11,}\n' or '#{11,}\n').
# Used to split the file into routines.
#
# \i $::langIncludesRequired
# Flag indicating that a missing INCLUDES section is an error.
#
# \i $::langSynopsisRequired
# Flag indication that a missing SYNOPSIS section is an error.
#
# \i $::langReturnsRequired
# Flag indicating that a missing RETURNS section is an error.
#
# \i $::langErrnoRequired
# Flag indicating that a missing ERRNO/ERRORS section is an error.
#
# \i $::blankWhenNoRoutines
# Logical flag, when non-zero, no ROUTINES section is printed when there
# are no documented routines.  When zero, the ROUTINES section says
# "No user-callable routines".
# \ie
#
# The following routines must be defined by any "apiIn" file:
# \is
# \i apiLangInit()
# Set the required global variables.
# \i apiPreprocess (@textLines)
# Do any processing necessary before removing any comment characters
# \i apiSynopsisGet($text)
# Return the synopsis extracted from the code text.
# \ie
#
# NOROUTINES
#

# specify namespace

package inAsm;

###############################################################################
#
# apiLangInit - initialize for processing of assembly-language files
#
# This routine just sets the values of some variables.  These variables are
# documented in the header for the routine apiExtract().
#

sub apiLangInit
    {
    $v = $::v;

    $extract::langLibStart      =
         '\s*([A-Z][-,_A-Z0-9\s]*($|:)|\\\\h\s.*|\\\\TITLE.*|\.TH.*|\.SH.*)';
    $extract::langCommentEnd    = '\*\/';
    $extract::langHeaderEnd     = "\\*\\/\\s*\$";
    $extract::langCommentPrefix = '\*';
    $extract::langRoutineStart  = '\/\*{11,}\s*\n';

    $::langIncludesRequired = 1;
    $::langSynopsisRequired = 1;
    $::langReturnsRequired  = 1;
    $::langErrnoRequired    = 1;

    $::blankWhenNoRoutines = 0;
    }


###############################################################################
#
# apiPreprocess - do any subroutine preprocessing required by the input format
#
# This routine turns the eccentric synopsis format into the standard one.
# The synopsis is surrounded by completely blank lines (no starting asterisks).
# The synopsis must be the last thing in the comment section.  Several styles
# have been used for this synopsis, so it is simply put into a display section.
# This routine is not applied to the library comments.
#

sub apiPreprocess
    {
    my @outLines = ();
    my @synopsis = ();

    # read in comment part of routine

    my $line;
    while ( @_ )
        {
        $line = shift;
        last if $line =~ /$v\*\//o;

        if ( $line =~ /^(.*)$v\s*$/o )
            {
            my $loc = $1;

            # start of old-style synopsis

            push @outLines, "$loc$v* \\h SYNOPSIS";
            push @outLines, "$loc$v* \\ss";

            $line = shift;
            $line =~ /^(.*)$v/o;
            $loc = $1;

            while ( $line !~ /$v\s*$/ )
                {
                if ( $line =~ /$v\*\// )
                    {
                    print STDERR "$loc: ERROR: only one blank line around synopsis\n";
                    $::errorStatus = 1;
                    last;
                    }
                push @outLines, $line;
                $line = shift;
                }

            push @outLines, "$loc$v* \\se";
            last if $line =~ /$v\*\//;
            }
        elsif ( $line =~ /$v\* (\\h )?SYNOPSIS/ )
            {
            # out-of-order synopsis

            while ( $line !~ /$v\* \\(c|s)e/ )
                {
                push @synopsis, $line;
                $line = shift;
                }
            push @synopsis, $line;
            }
        else
            {
            push @outLines, $line;
            }
        }

    push @outLines, @synopsis;
    push @outLines, $line;

    # copy code portion of routine

    push @outLines, @_;

    return @outLines;
    }


###############################################################################
#
# apiSynopsisGet - find and format a C synopsis
#
# This routine does nothing because an explicit synopsis is required.
#

sub apiSynopsisGet
    {
    return;
    }

1;
