# catgen.tcl - utility to produce man pages in the cat directories
#
# Copyright 2001 Wind River Systems, Inc.
#
# modification history
# --------------------
# 01k,11feb02,wsl  add *.lib and *.rtn files to LIB and RTN files
# 01j,15may01,wsl  Fix sort order and entry format of windex file
# 01i,16apr01,wsl  fix rt angle bracket in a few routine names
# 01h,13apr01,wsl  remove angle brackets from file names
# 01g,13apr01,wsl  allow for routines with a space in name, and replace
#                  routine separator (again)
# 01f,12apr01,wsl  adopt new routine separator
# 01e,12apr01,wsl  fix bug leading to input files not being closed
# 01d,12apr01,wsl  remove premature closure of inFile after error
# 01c,05apr01,wsl  add processing of BSP files, fix some bugs
# 01b,31mar01,wsl  implement changes in response to review
# 01a,26mar01,wsl  written
#
# DESCRIPTION
# This utility takes the .txt files generated by refgen in the $WIND_BASE/man
# directory and creates individual man-style files in the cat1 and cat2
# directories for each library and each routine.  If the cat1 and cat2
# directories do not exist, they are created.  The utility then creates the
# windex database so that apropos and whereis will work.
#
# SYNOPSIS
#
# catgen [-man <manDirectory>] [-cat1 <cat1Directory>] [-cat2 <cat2Directory>]
#
# RESOURCE FILES
# none
#*/

# globals

set	exitStatus	0	;# allows signalling make of non-fatal error
set	manDir		$env(WIND_BASE)/man	;# home of .txt files
set	cat1Dir		$manDir/cat1	;# dir for library man pages
set	cat2Dir		$manDir/cat2	;# dir for routine man pages
set	windexList	{}	;# list of windex entries
set	windexFile	windex	;# file name of windex database

################################################################################
#
# manProc - produce the man pages and windex database
#
# This procedure sets up the cat1 and cat2 directories, then calls other
# procedures to do the main work.
#
# SYNOPSIS
# mainProc
#
# PARMETERS
# none
#
# RETURNS
# 1 if an error, fatal or non-fatal, has occurred,
# 0 otherwise
#
# ERRORS
# N/A
#

proc manProc { argv argc } {

    global env
    global exitStatus
    global manDir	;# complete path for 'man' directory
    global cat1Dir	;# complete path for 'cat1' directory
    global cat2Dir	;# complete path for 'cat2' directory

    # process command-line parameters

    for {set ix 0} {$ix < $argc} {incr ix} {

	switch -- [lindex $argv $ix] {
	
	    "-man" {
		incr ix
		set manDir [lindex $argv $ix]
	    }

	    "-cat1" {
		incr ix
		set cat1Dir [lindex $argv $ix]
	    }

	    "-cat2" {
		incr ix
		set cat2Dir [lindex $argv $ix]
	    }

	    default {
		error "Invalid command-line option"
	    }
	}
    }

    # look for $manDir -- error if not there

    if [catch {cd $manDir}] {
	error "Could not cd to $manDir"
    }

    # create cat1 and cat2 directories -- error if file with those names

    if [file exists $cat1Dir] {
	if {! [file isdirectory $cat1Dir]} {
	    error "File exists with name $cat1Dir"
	}
    } else {
	if [catch {file mkdir $cat1Dir}] {
	    error "Could not create directory $cat1Dir"
	}
    }

    if [file exists $cat2Dir] {
	if {! [file isdirectory $cat2Dir]} {
	    error "File exists with name $cat2Dir"
	}
    } else {
	if [catch {file mkdir $cat2Dir}] {
	    error "Could not create directory $cat2Dir"
	}
    }

    # get a list of all files (no simple way to list only directories)

    set dirList [glob -nocomplain *]
    lappend dirList "."

    # process all text files in each directory

    foreach textDir $dirList {
	if { (! [file isdirectory $textDir]) || \
	     ($textDir == [file tail $cat1Dir]) || \
	     ($textDir == [file tail $cat2Dir]) } {
	    continue
	}

	cd $textDir
	set fileList [glob -nocomplain *.txt]

	# process each .txt file in the directory

	foreach textFile $fileList {
	    if [file isfile $textFile] {
		manSplit $textFile $textDir
	    }
	}

	# clean up some unnecessary files that can grow without bound

	set deleteList [glob -nocomplain \
		LIB *.lib RTN *.rtn libIndex.* rtnIndex.*]
	foreach deleteFile $deleteList {
	    file delete $deleteFile
	}

	cd $manDir

    }

    # create windex database

    manWindex

    # exit

    exit $exitStatus
}


################################################################################
#
# manSplit - Split a text file produced by refgen into man pages
#
# This procedure reads in .txt files produced by refgen and produces a
# library man page and individual man pages for each documented routine
#
# SYNOPSIS
# manSplit <filename> <directoryname>
#
# PARMETERS
# none
#
# RETURNS
# N/A
#
# ERRORS
# N/A
#

proc manSplit {fileName dirName} {

    global exitStatus
    global cat1Dir
    global cat2Dir
    global windexList

    # try to open the file for reading

    if [catch {open $fileName r} inFileId] {
	global errorInfo
	puts "ERROR: could not open $fileName -- continuing"
	puts $errorInfo
	set exitStatus 1
	return
    }

    # get the base name of the file

    set baseName [file rootname $fileName]

    # for BSPs, prepend the BSP name to the output file names

    if { $dirName != "." } {
	set preName "${dirName}_"
    } else {
	set preName ""
    }

    # read through file, creating man pages as names are discovered

    set firstIteration 1
    while { ! [eof $inFileId] } {
	set textLine [gets $inFileId]

	# look for library/routine name and short description

	if [regexp {^([^-\(\)]+)(\(\))?[ ]+-[ ]+(.*)$} $textLine \
		match refName parens description] {

	    # found a library or routine -- create output file
	    # Some routines have spaces in their names (e.g. "operator new");
	    # replace these spaces with underscores.  Also remove angle
	    # brackets from names in template BSPs.

	    set refName [string trim $refName]
	    regsub -all { } $refName {_} refName
	    regsub -all {[<>]} $baseName {} baseName

	    if { $firstIteration == "1" } {
		set firstIteration 0

		# library entry is always the first one

		set fileStemName $preName$baseName
		set outFileName $cat1Dir/$fileStemName.1
		set pageType "Libraries"
		set section "(1)"
	    } else {
		set fileStemName $preName$refName
		set outFileName $cat2Dir/$fileStemName.2
		set pageType "Routines"
		set section "(2)"
	    }

	    if [catch {open $outFileName w} outFileId] {
		global errorInfo
		puts "ERROR: could not open $outFileName for writing"
		puts $errorInfo

		# skip to next routine

		while { (! [eof $inFileId]) && \
			(! [regexp {^________+ @@@ ______+$} $textLine]) } {
		    set textLine [gets $inFileId]
		}		    
		set exitStatus 1
		continue;
	    }

	    # print the header line

	    set header [format "%-20s%50s%s\n\nNAME" $pageType $refName \
		        $section]
	    puts $outFileId $header

	    # print the routine up to the line of underscores
	    # omit multiple blank lines

	    set prevBlank 0

	    while { (! [eof $inFileId]) &&
	            (! [regexp {^__________+ @@@ ________+$} $textLine]) } {
		if [regexp "^\[ \t\]*$" $textLine] {
		    if { $prevBlank == 1 } {
			gets $inFileId textLine
			continue
		    } else {
			set prevBlank 1
			puts $outFileId $textLine
		    }
		} else {
		    set prevBlank 0
		    puts $outFileId $textLine
		}

		gets $inFileId textLine
	    }

	    # close the output file

	    close $outFileId

	    # add windex entries to the global list --
	    # routines only have one entry, but BSP libraries may have more.
	    # There must be at least one entry named after the file.

# this section doesn't seem to meet the requirements for windex -- why??
#	    set nameList [split $refName {,/}]
#	    lappend nameList $fileStemName
#	    foreach rawName $nameList {
#		set shortName [string trim $rawName]
#
#		set windexLine $shortName
#		if { [string length $shortName] < 8 } {
#		    append windexLine "\t"
#		}
#		append windexLine "\t$fileStemName $section\t"
#		if { [string length $fileStemName] < 4 } {
#		    append windexLine "\t"
#		}
#		append windexLine "- $description"
#
#		lappend windexList $windexLine
#	    }

            set windexLine $fileStemName
            if { [string length $fileStemName] < 8 } {
		append windexLine "\t"
	    }
	    append windexLine "\t$fileStemName $section\t"
	    if { [string length $fileStemName] < 4 } {
		append windexLine "\t"
	    }
	    append windexLine "- $description"

	    lappend windexList $windexLine
	}
    }
    close $inFileId
    return
}


################################################################################
#
# manWindex - Write out the windex database
#
# This procedure sorts the list of windex entries and prints it to a file
#
# SYNOPSIS
# manWindex
#
# PARMETERS
# none
#
# RETURNS
# N/A
#
# ERRORS
# N/A
#

proc manWindex {} {

    global exitStatus
    global windexList
    global windexFile

    # sort list

    set windexSort [lsort -ascii $windexList]

    # print out list, skipping exact duplicate lines

    if [catch {open $windexFile w} windexId] {
	puts "ERROR: could not open windex file"
	set exitStatus 1
	return
    }

    set prevLine ""
    foreach windexLine $windexSort {
	if { $windexLine == $prevLine } {
	    continue;
	}

	puts $windexId $windexLine
	set prevLine $windexLine
    }

    close $windexId

    return
}

################################################################################
#
# catgen.tcl - this external code just invokes the main procedure
#
# This procedure invokes manProc, which does the whole job
#

manProc $argv $argc
