# htmlProj.tcl - Generates compiled help information for Windows hosts.
#
# Copyright 1999-2003 Wind River Systems, Inc.
#
# modification history
# --------------------
# 01z,25jun03,fle  Made it use Wind package to be able to use tclsh rather
#                  than wtxtcl shell
# 01y,11sep02,wsl  change sourced files
# 01x,04apr02,wsl  make htmlProj able to create CHM files other than the
#                  Tornado Reference
# 01w,07feb02,wsl  change nav file extensions from .html to .htm
#                  reformat comments in compliance with coding conventions
# 01v,20feb01,tdw  generalize help for add on modules to Tornado
# 01u,14dec00,tdw  fix SPR 62684 - help nodes need proper links
# 01t,07dec00,tdw  fix BSP VTS docs in TOC
# 01s,06dec00,vcm  adjusting .hhp parameters: home page, title
# 01r,01dec00,vcm  adding errno directory entry to index
# 01q,26oct00,tdw  change homepage
# 01p,26oct00,tdw  delete old files before rebuilding tornado reference
# 01o,15sep00,tpw  Do some AE name changes.
# 01n,08jun00,dgp  change BSP_Reference to VxWorks_BSP_Reference and 
#		   Tornado_Reference to Tornado_Tools_Reference.
# 01m,19apr00,tdw  playing around with window declarations.
# 01l,19apr00,tdw  change default .hhc and .hhp file name
# 01k,18apr00,tdw  Changing window declarations
# 01j,12apr00,tdw  fixing links w/ ::
# 01i,05apr00,tdw  added window definitions to template of .hhp file
# 01h,27mar00,tdw  more fixes for new help
# 01g,25mar00,tdw  Bug in TOC creation: Double colons cause incorrect HTML
#                  links to be placed.
# 01f,20nov99,dgp  remove ^Ms
# 01e,08oct99,ren  make paths in text file relative to text file directory
# 01d,07oct99,ren  added batch commandline option
# 01c,06oct99,ren  added windHelpInit call to create TREE.WRS and INDEX.WRS
#                  files if they do not exist.
# 01b,28sep99,ren  added comments and usage
# 01a,22sep99,ren  created

# DESCRIPTION:
# -----------
#
# htmlProj takes the output of htmlBook (TREE.WRS and INDEX.WRS in a
# directory of HTML) and compiles it into a Microsoft HTML Help File
# (.CHM)
#
# There must be a <project-name>.htmlProj file containing HTML Help project
# information in the directory where the help file will be created.
# For example, the command
#
# % wtxtcl htmlProj "Tornado_Reference"
#
# requres that a Tornado_Reference.htmlProj file exist.
#
# SYNOPSIS:
#
# wtxtcl htmlProj <Help-File-Title>
#
# This command must be executed in the directory at the top of the tree of
# HTML files.  The <Help-File-Title> will be used as a title, so it should be
# capitalized appropriately, and should use underscores where spaces are
# desired.
# 
# OUTPUT:
# <Help-File-Title>.chm

package require Wind

source [file join [Wind::base] host resource doctools windHelpLib.tcl]

# This block sets up some variables global to the htmlProj namespace.

namespace eval htmlProj {
# this is the template for the project file format.
# the %% are replaced with real values by writeHtmlProject
    set strProjectFileTemplate {[OPTIONS]
Title=%title%
Contents file=%contentsFile%
Index file=%indexFile%
Display compile progress=Yes
Display compile notes=Yes
Full-text search=Yes
Binary Index=Yes
Default Window=%windowName%
Default topic=%topic%
Language=0x409 English (United States)

[WINDOWS]
%windowDefinition%

[FILES]
}

set strUsage {   Usage:
wtxtcl htmlProj.tcl [-outdir <destination directory>] <Help File Title> 
Help File Title is the title which appears on the title bar of the
help viewer when the help file is displayed.
Compiles all the htmlFiles in the current directory into a CHM file.
Outputs: Help_File_Title.chm

wtxtcl htmlProj.tcl [-outdir <destination directory>]
}
}


###############################################################################
#
# itemInExcludeList - test whether an item is in the excluded list
#
# This routine looks for the given item in the list of excluded topics.
#
# SYNOPSIS
#   itemInExcludeList item
#
# PARAMETERS
#   item : name of a help topic
#
# RETURNS:
#   0 if item is not in list
#   1 if item is in list
#
# ERRORS:
#   N/A
#

proc htmlProj::itemInExcludeList {item} {
    variable HH_excludeNodes
    set found 0
    foreach excludeNode $HH_excludeNodes {
	if {[string first $excludeNode $item] != -1} {
	    set found 1
	}
    }
    return $found
}


###############################################################################
#
# itemInRefList - test whether help item is in list to be included
#
# This routine looks for the given item in the list of help topics
# to be included in the CHM file.
#
# SYNOPSIS
#   itemInRefList item
#
# PARAMETERS
#   item : name of a help topic
#
# RETURNS:
#   0 if item is not in list
#   1 if item is in list
#
# ERRORS:
#   N/A
#

proc htmlProj::itemInRefList {item} {
   variable HH_includedDirectories
    set found 0
    foreach includeDir $HH_includedDirectories {
	if {[string first $includeDir $item] != -1} {
	    set found 1
	}
    }
    return $found
}


###############################################################################
#
# getKeyword - get the keyword from a help item entry
#
# This routine extracts the keyword from the given list specifying a help
# item entry.
#
# SYNOPSIS
#   getKeyword entry
#
# PARAMETERS
#   entry : list specifying all data about a help topic
#
# RETURNS:
#   the keyword
#
# ERRORS:
#   N/A
#

proc htmlProj::getKeyword { entry } {
    lindex $entry 0
}


###############################################################################
#
# getDescription - get the description from a help topic entry
#
# This routine extracts the description information from the given list
# specifying all information about the help topic.
#
# SYNOPSIS
#   getDescription entry
#
# PARAMETERS
#   entry : list specifying all information about the help topic
#
# RETURNS:
#   the description as text
#
# ERRORS:
#   N/A
#

proc htmlProj::getDescription { entry } {
    lindex $entry 1
}


###############################################################################
#
# getURL - get the URL for the given help topic entry
#
# This routine extracts the URL from the list entry describing a help topic.
# The URL is massaged slightly to remove double colons, which originate in
# C++ routine descriptions (?) and cause problems.
#
# SYNOPSIS
#   getURL entry
#
# PARAMETERS
#   entry : list containing all information about a given help topic
#
# RETURNS:
#   a URL
#
# ERRORS:
#   N/A
#

proc htmlProj::getURL { entry } {
    
    #lindex $entry 2

    # This is a hack to fix links with "::" - if
    # the link contains "::", we strip out everything
    # after the hash mark.
    set tempUrl [lindex $entry 2]
    set returnUrl $tempUrl
    set urlIndex [string first :: $tempUrl]
    set hashIndex [string first # $tempUrl]

    if { $hashIndex != -1} {
	if { $urlIndex > $hashIndex } {
	    set returnUrl [string range $tempUrl 0 [expr $hashIndex - 1]]
	}
    }
    return $returnUrl
 
}


###############################################################################
#
# getBookTitle - get the title of the book a given help entry should appear in
#
# This routine extracts the title of the book in which the given help entry
# should appear.
#
# SYNOPSIS
#   getBookTitle entry
#
# PARAMETERS
#   entry : list containing all information about a help topic
#
# RETURNS:
#   a book title
#
# ERRORS:
#   N/A
#

proc htmlProj::getBookTitle { entry } {
    lindex $entry 3
}


###############################################################################
#
# getChapterTitle - get the title of the chapter for a given help topic
#
# This routine extracts the title of the chapter in which the given help
# topic should appear.
#
# SYNOPSIS
#   getChapterTitle entry
#
# PARAMETERS
#   entry : list containing all information about a given help topic
#
# RETURNS:
#   a chapter title
#
# ERRORS:
#   N/A
#

proc htmlProj::getChapterTitle { entry } {
    lindex $entry 4
}


###############################################################################
#
# groupIndex - group an index by keyword
#
# This routine takes an index as returned by windHelpIndexGet, and groups it
# into sublists of entries which all have the same keyword.  For example,
# if the input were:
# \cs
# {accept {accept a connection from a socket} vxworks/ref/sockLib.html#accept
#         {VxWorks Reference Manual} Libraries {}}
# {acos {compute an arc cosine (ANSI)} vxworks/ref/mathALib.html#acos
#         {VxWorks Reference Manual} Libraries {}}
# {acos {compute an arc cosine (ANSI)} vxworks/ref/ansiMath.html#acos
#         {VxWorks Reference Manual} Libraries {}}
# \ce
# the output would be:
# \cs
# {{accept {accept a connection from a socket} vxworks/ref/sockLib.html#accept
#         {VxWorks Reference Manual} Libraries {}}}
# {{acos {compute an arc cosine (ANSI)} vxworks/ref/mathALib.html#acos
#         {VxWorks Reference Manual} Libraries {}}
#  {acos {compute an arc cosine (ANSI)} vxworks/ref/ansiMath.html#acos
#         {VxWorks Reference Manual} Libraries {}}}
# \ce
#
# SYNOPSIS
#   groupIndex listIndex
#
# PARAMETERS
#   listIndex : list of index entries
#
# RETURNS:
#   a list of lists of index entries, each of which shares the same keyword
#
# ERRORS:
#   N/A
#

proc htmlProj::groupIndex { listIndex } {
    if { [llength $listIndex] == 0 } {
	return [list ]
    }

    set listAllGroups [list ]
    set entry0 [lindex $listIndex 0]
    set listGroup [list $entry0]
    set strKeyword [getKeyword $entry0]
    foreach entry [lrange $listIndex 1 end] {
	if { $strKeyword == [getKeyword $entry] } {
	    # it's the same keyword, so put it in the same group.
	    lappend listGroup $entry
	} else {
	    # it's different, so close this group and create
	    # another
	    lappend listAllGroups $listGroup
	    set listGroup [list $entry]
	    set strKeyword [getKeyword $entry]
	}
    }
    # close the final group and return it
    lappend listAllGroups $listGroup
}


###############################################################################
#
# writeHtmlIndex - write an HTML index file
#
# This routine takes an index as returned by windHelpIndexGet, converts it
# into the HtmlHelp index file format, and writes it to $strFilename
#
# SYNOPSIS
#   writeHtmlIndex listIndex strFilename
#
# PARAMETERS
#   listIndex : a list of index entries for help topics
#   strFilename : the desired output file name
#
# RETURNS:
#   N/A
#
# ERRORS:
#   N/A
#

proc htmlProj::writeHtmlIndex { listIndex strFilename } {
    set listGroupedIndex [groupIndex $listIndex]
    set f [open $strFilename w]

    # write the header
    puts $f {<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN">}
    puts $f {<HTML>}
    puts $f {<HEAD>}
    puts -nonewline $f {<meta name="GENERATOR" content="Microsoft&reg; }
    puts $f {HTML Help Workshop 4.1">}
    puts $f {<!-- Sitemap 1.0 -->}
    puts $f {</HEAD><BODY>}
    puts $f {<UL>}

    # now write the index
    foreach listGroup $listGroupedIndex {
	if {[itemInRefList $listGroup] == 1} {
	    puts -nonewline $f [convertGroup $listGroup]
	}
    }

    # now write the footer
    puts $f {</UL>}
    puts $f {</BODY></HTML>}
    close $f
}


###############################################################################
#
# convertGroup - format a group of index entries for HTML Help
#
# This routine takes a list of index entries (index returned by
# windHelpIndexGet), all of which have the same keyword, and converts
# it into the Microsoft HTML Help file format.
#
# SYNOPSIS
#   convertGroup listGroupOfEntries
#
# PARAMETERS
#   listGroupOfEntries : a list of index entries all having the same keyword
#
# RETURNS:
#   N/A
#
# ERRORS:
#   N/A
#

proc htmlProj::convertGroup { listGroupOfEntries } {
    if { [llength $listGroupOfEntries] == 0 } {
	return {}
    }
    # else, first handle the 0 entry
    set strResult "	<LI> <OBJECT type=\"text/sitemap\">\n"
    set entry0 [lindex $listGroupOfEntries 0]
    append strResult "		<param name=\"Name\" value=\"[getKeyword $entry0]\">\n"
    append strResult "		<param name=\"Name\" value=\"[cleanQuotes [getDescription $entry0]]\">\n"
    append strResult "		<param name=\"Local\" value=\"[getURL $entry0]\">\n"

    # now handle the other entries
    foreach entry [lrange $listGroupOfEntries 1 end] {
	append strResult "		<param name=\"URL\" value=\"[getURL $entry]\">\n"
	append strResult "		<param name=\"Name\" value=\"[cleanQuotes [getDescription $entry]]\">\n"
	append strResult "		<param name=\"Local\" value=\"[getURL $entry]\">\n"
    }

    # add closing tag and return
    append strResult "		</OBJECT>\n"
}


###############################################################################
#
# incrIndent - append a tab to the global string named in the input
#
# This routine appends a tab character to the global string whose name is
# stored in its input argument.  (Note that the name "indent" could be
# misleading.  It is really the tab stop that is incremented.)
#
# SYNOPSIS
#   incrIndent varIndent
#
# PARAMETERS
#   varIndent : the name of the string to which to add a tab
#
# RETURNS:
#   N/A
#
# ERRORS:
#   N/A
#

proc htmlProj::incrIndent { varIndent } {
    upvar $varIndent strIndent
    append strIndent "\t"
}


###############################################################################
#
# decrIndent - delete tab character in the string named in the input string
#
# This routine deletes the first tab character found in the global variable
# whose name is stored in the input argument.  Note that decrIndent does not
# reverse the effect of incrIndent unless the global string consists of tab
# characters alone.
#
# SYNOPSIS
#   decrIndent varIndent
#
# PARAMETERS
#   varIndent : the name of the string from which to remove a tab
#
# RETURNS:
#   N/A
#
# ERRORS:
#   N/A
#

proc htmlProj::decrIndent { varIndent } {
    upvar $varIndent strIndent
    regsub "\t" $strIndent {} strIndent
}


###############################################################################
#
# writeNodes - create an HTML Help TOC file
#
# This routine takes a string representing a tree as returned by
# windHelpTreeGet or controlValuesGet <treecontrol>, and converts it into
# an HTML Help table of contents file.
#
# SYNOPSIS
#   writeNodes strTree f
#
# PARAMETERS
#   strTree : string containing representation of HTML TOC information
#   f : file descriptor of output file
#
# RETURNS:
#   N/A
#
# ERRORS:
#   N/A
#

proc htmlProj::writeNodes { strTree f } {
    variable HH_includedDirectories
    set listLines [prepareLines $strTree]
    set strIndent ""
    set cwd [windHelpDocDirGet]
    foreach line $listLines {
	if { $line == "\{" } {
	    puts $f "[set strIndent]<UL>"
	    incrIndent strIndent
	} elseif { $line == "\}" } {
	    decrIndent strIndent
	    puts $f "[set strIndent]</UL>"
	} else {
	    set nodeName [string trim $line " \r\n\t\{\}"]
	    set urlHelpLink [windHelpLinkGet -name $nodeName]
	    set includeThisLink 0
	    foreach pathName $HH_includedDirectories {
		set matchString *
		append matchString $pathName
		append matchString *
		#puts stdout "string match $matchString $urlHelpLink is [string match $matchString $urlHelpLink]\n"
		if { [string match $matchString $urlHelpLink] != 0 } {
		    set includeThisLink 1
		}
	    }
	   
	    if {[itemInExcludeList $nodeName] == 0} {
		puts $f "[set strIndent]<LI> <OBJECT type=\"text/sitemap\">"
		puts $f "[set strIndent]\t<param name=\"Name\" value=\"$nodeName\">"
		if { $includeThisLink == 1 } {

		    set urlIndex [string first :: $urlHelpLink]
		    set hashIndex [string first # $urlHelpLink]

		    if { $hashIndex != -1} {
			if { $urlIndex > $hashIndex } {
			    set urlHelpLink [string range $urlHelpLink 0 [expr $hashIndex - 1]]
			}
		    }
		} else {
		    set urlHelpLink [findHelpFile [bookFileNameGet $nodeName]]
		}
		if {$urlHelpLink != ""} {
		    regsub ^[set cwd] $urlHelpLink {} urlHelpLink
		    puts $f "[set strIndent]\t<param name=\"Local\"value=\"$urlHelpLink\">"
		}
		puts $f "[set strIndent]\t</OBJECT>"
	    }
	}
    }
}


###############################################################################
#
# findHelpFile - find a file in global hash of files
#
# This routine searches for the given file's complete path in the global
# hash `docArray'.
#
# SYNOPSIS
#   findHelpFile fileName
#
# PARAMETERS
#   fileName : base name of file (no path information)
#
# RETURNS:
#   null string if file is not in list
#   full path name of file if file is in list
#
# ERRORS:
#   N/A
#

proc htmlProj::findHelpFile {fileName} {

    global docArray

    if {[info exists docArray($fileName)]} {
	return $docArray($fileName)
    }

    return ""

}


###############################################################################
#
# initDocList - generate hash of all files under a specifed directory
#
# This routine recursively locates all files under a given directory and
# adds them to a global hash.  The hash keys are the base names of the files,
# and the hash values contain the complete path names.
#
# SYNOPSIS
#   initDocList rootDir
#
# PARAMETERS
#   rootDir : the directory with which to start
#
# RETURNS:
#   N/A
#
# ERRORS:
#   N/A
#

proc htmlProj::initDocList {rootDir} {

    global docArray

    set fileList [glob -nocomplain ${rootDir}/*]
    foreach item $fileList {
	if {[file isdirectory $item]} {
	    initDocList $item
	} else {
	    set fileName [file tail $item]
	    set docArray($fileName) $item
	}
    }
}


###############################################################################
#
# bookFileNameGet - convert a text book title into a valid file name
#
# This routine converts a book title into a file name by replacing all
# problematic characters with underscores.
# 
# EXAMPLE:
# \cs
#   bookFileNameGet {Tornado User's Guide (WINDOWS)}
# \ce
# would return the file name "Tornado_User_s_Guide__WINDOWS_.htm"
#
# SYNOPSIS
#   bookFileNameGet bookName
#
# PARAMETERS
#   bookName : the title of a book
#
# RETURNS:
#   string containing a derived file name
#
# ERRORS:
#   N/A
#

proc htmlProj::bookFileNameGet { bookName } {

    regsub -all "\[ \t\n\'\";,\\*\\(\\)\]" $bookName "_" bookFileName
    return ${bookFileName}.htm
}


###############################################################################
#
# cleanQuotes - remove single (and back) quotes from a string
#
# This routine returns a new string, removing any single quote characters
# from the argument
#
# SYNOPSIS
#   cleanQuotes str
#
# PARAMETERS
#   str : a character string
#
# RETURNS:
#   a character string identical to the input with single quotes removed
#
# ERRORS:
#   N/A
#

proc htmlProj::cleanQuotes { str } {
    regsub -all "'`" $str {} str
    return $str
}


###############################################################################
#
# writeHtmlContents - create an HTML Help TOC from a tree string
#
# This routine converts a tree, as returned by `windHelpTreeGet',
# converts it to Microsoft HTML Help file format, and writes it to a file
#
# SYNOPSIS
#   writeHtmlContents treenode filename
#
# PARAMETERS
#   treenode : tree string as returned by `windHelpTreeGet' or
#        `controlValuesGet' <treecontrol>
#   filename : name of the output file, normally contents.hhc
#
# RETURNS:
#   N/A
#
# ERRORS:
#   N/A
#

proc htmlProj::writeHtmlContents { treenode filename } {
    variable HH_title
    variable HH_topic

    set f [open $filename w]
    
    # write the header
    puts $f {<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML//EN">}
    puts $f {<HTML>}
    puts $f {<HEAD>}
    puts -nonewline $f {<meta name="GENERATOR" content="Microsoft&reg; }
    puts $f {HTML Help Workshop 4.1">}
    puts $f {<!-- Sitemap 1.0 -->}
    puts $f {</HEAD><BODY>}
    puts $f {<OBJECT type="text/site properties">}
    puts $f {        <param name="WindowName" value = "TriPane">}
    puts $f {	<param name="ExWindow Styles" value="0x100">}
    puts $f {	<param name="Window Styles" value="0x800025">}
    puts $f {</OBJECT>}
    puts $f {  <UL>}
    puts $f {    <LI> <OBJECT type="text/sitemap">}
    puts $f "        <param name=\"Name\" value=\"$HH_title\">"
    puts $f "        <param name=\"Local\" value=\"$HH_topic\">"
    puts $f {        </OBJECT>}
    puts $f {}

    # Now write the tree nodes
    writeNodes $treenode $f

    # Now write the footer
    puts $f {</UL></BODY></HTML>}
    close $f
}


###############################################################################
#
# prepareLines - separate a string into a list of lines
#
# This routine breaks a string up into a list of lines, removes any empty
# lines, and trims white space from all lines
#
# SYNOPSIS
#   prepareLines treenode
#
# PARAMETERS
#   treenode : a string, usually containing a set of tree nodes
#
# RETURNS:
#   a list of lines
#
# ERRORS:
#   N/A
#

proc htmlProj::prepareLines { treenode } {
    set listResult [list ]
    foreach line [split $treenode "\n"] {
	set line [string trim $line]
	if { $line != "" } {
	    lappend listResult $line
	}
    }
    return $listResult
}


###############################################################################
#
# ensureExtensionIsHHP - append .hhp to a file name
#
# This routine returns a filename and appends the extension .hhp if there is
# no extension.  If there is already an extension, and it is not .hhp,
# then print a warning.
#
# SYNOPSIS
#   ensureExtensionIsHHP filename
#
# PARAMETERS
#   filename : an incomplete file name
#
# RETURNS:
#   completed filename
#
# ERRORS:
#   warning if file already has an extension that is not .hhp
#

proc htmlProj::ensureExtensionIsHHP { filename } {
    set ext [file extension $filename]
    if { [string match $ext .hhp] } {
	return $filename
    } elseif { $ext == "" } {
	append filename .hhp
	return $filename
    } else {
	puts "Warning: $filename does not end in standard help project file extension .hhp."
	return $filename
    }
}


###############################################################################
#
# writeHtmlProject - write an HTML Help project file
#
# This routine writes out the HTML Help project file, which describes what
# files to include in the CHM file and what options to employ
#
# SYNOPSIS
#   writeHtmlProject fileProject fileContents fileIndex title directory
#
# PARAMETERS
#   fileProject : the name of the output project file
#   fileContents : the name of the contents file, usually contents.hhc
#   fileIndex : the name of the index file, usually index.hhk
#   title : the tile of the CHM file, which will appear on the title
#      bar of the HTML Help viewer
#   directory : the directory of HTML files 
#
# RETURNS:
#   N/A
#
# ERRORS:
#   N/A
#

proc htmlProj::writeHtmlProject { fileProject fileContents fileIndex title directory} { 
    variable strProjectFileTemplate
    variable HH_includedFiles
    variable HH_includedDirectories
    variable HH_windowName
    variable HH_topic
    variable HH_windowDefinition

    regsub -all %windowDefinition $strProjectFileTemplate $HH_windowDefinition strProject
    regsub -all %title% $strProject $title strProject
    regsub -all %contentsFile% $strProject $fileContents strProject
    regsub -all %indexFile% $strProject $fileIndex strProject
    regsub -all %topic% $strProject $HH_topic strProject
    regsub -all %windowName% $strProject $HH_windowName strProject
    
    set f [open $fileProject w]
    puts -nonewline $f $strProject
    set listMasks [list *.html *.htm *.css]

#    puts $f [join [collectFiles $directory $listMasks] "\n"]
    foreach refOutputFile $HH_includedFiles {
	set refPath [file join $directory $refOutputFile]
	#puts stdout "writeHtmlProject: writing $refPath"
	append refPath "\n"
	#puts stdout "writeHtmlProject: writing $refPath"
	puts $f $refPath
    }
    foreach refOutputDir $HH_includedDirectories {
	set refPath [file join $directory $refOutputDir]
	#puts stdout "collecting files from $refPath\n"
	puts $f [join [collectFiles $refPath $listMasks] "\n"]
    }
    
    close $f
}


###############################################################################
#
# cleanFileName - replace problem file name characters with underscore
#
# This routine replaces all non-alphanumeric characters except for dots (.)
# with underscores (_)
#
# SYNOPSIS
#   cleanFileName filename
#
# PARAMETERS
#   filename : a string intended for a file name
#
# RETURNS:
#   a string purged of problem characters
#
# ERRORS:
#   N/A
#

proc htmlProj::cleanFileName { filename } {
    regsub -all {[^A-Za-z0-9\.]} $filename _ filename
    return $filename
}


###############################################################################
#
# collectFiles - create a list of files matching a given set of patterns
#
# This routine searches through a directory, recursively by default,
# collecting all files that match any of a list of glob patterns.
# It returns a list of all the files that match. Unless the `norecursion'
# parameter is non-zero, the search extends to all subdirectories.
#
# SYNOPSIS
#   collectFiles directory listMasks [norecurse]
#
# PARAMETERS
#   directory : the start directory for the search
#   listMasks : list of glob patterns to match
#   norecurse : if non-zero, look only in $directory;  default is zero
#
# RETURNS:
#   list of matching files
#
# ERRORS:
#   N/A
#

proc htmlProj::collectFiles {directory listMasks {norecurse 0}} {
    set listCollectedFiles [list ]
    foreach strMask $listMasks {
	eval lappend listCollectedFiles [glob -nocomplain "[file join $directory $strMask]"]
    }
    set listAllFiles [glob -nocomplain "[file join $directory *]"]
    #puts stdout "collectFiles:listAllFiles $directory : \n $listAllFiles\n"
    set listDirectories [list ]
    foreach strFile $listAllFiles {
	#if { $norecurse != 0 } {
	    if { [file isdirectory $strFile]} {
		eval lappend listCollectedFiles [collectFiles $strFile $listMasks]
	    }
	#}
    } 
    return $listCollectedFiles
}


###############################################################################
#
# buildCHM - main procedure -- builds a CHM file in specified directory
#
# This routine builds a CHM file in the specified directory.  The directory
# must contain a tree of HTML files plus TREE.WRS and INDEX.WRS files as
# output from htmlBook.tcl
#
# SYNOPSIS
#   buildCHM title [directory] [dirOutput]
#
# PARAMETERS
#   title : title of CHM content
#   directory : output directory;  default is the current directory
#   dirOutput : unused
#
# RETURNS:
#   N/A
#
# ERRORS:
#   N/A
#

proc htmlProj::buildCHM { title {directory .} {dirOutput {}}} {
    variable HH_contentsFile
    variable HH_indexFile
    set dirOld [pwd]
    cd $directory
    set dirCurrent [pwd]
    puts "Initializing directory $dirCurrent."
    set errmsg ""
    set strProjectFile [ensureExtensionIsHHP [cleanFileName $title]]
    catch {
	file delete -force INDEX.WRS TREE.WRS
	file delete -force ${strProjectFile}.chm
	file delete -force $HH_contentsFile $HH_indexFile
    } errmsg

    if { "$errmsg" == "" } {
        puts "clean completed successfully"
    } else {
        puts "clean reported $errmsg"
    }
    windHelpInit $dirCurrent
    puts "Compiling index $HH_indexFile."
    writeHtmlIndex [windHelpIndexGet $dirCurrent] $HH_indexFile
    puts "Done writing index file."
    puts "Compiling table of contents $HH_contentsFile"
    writeHtmlContents [windHelpTreeGet $dirCurrent] $HH_contentsFile
    puts "Done writing toc file."
    puts "Writing project file $strProjectFile."
    writeHtmlProject $strProjectFile $HH_contentsFile $HH_indexFile $title .
    puts "Done writing project file."
    puts "Execute the following commandline from a WIN32 host to complete the build:"
    puts "\\WIND_BASE\\host\\resource\\doctools\\mshhws\\hhc.exe $strProjectFile"
 
    puts "HtmlProj Done."
    cd $dirOld
}


###############################################################################
#
# printUsage - print a message describing how to use htmlProj
#
# This routine simply prints the usage message for htmlProj.
#
# SYNOPSIS
#   printUsage
#
# PARAMETERS
#   N/A
#
# RETURNS:
#   N/A
#
# ERRORS:
#   N/A
#

proc htmlProj::printUsage {} {
    variable strUsage
    puts $strUsage
}


###############################################################################
#
# readTextFile - read in a list of titles and directories and process them
#
# NOTE: This procedure is not used.
#
# This routine reads a text file for a list of titles and directories, then
# calls buildCHM for each one.  The file must have the format:
# \ss
#   Title
#   Directory
#
#   Title
#   Directory
#
#   ...
# \se
#
# SYNOPSIS
#   readTextFile strTextFile dirOutput
#
# PARAMETERS
#   strTextFile : name of file to read
#   dirOutput : unused
#
# RETURNS:
#   N/A
#
# ERRORS:
#   fails if the input file has an odd number of non-blank lines
#   
#

proc htmlProj::readTextFile { strTextFile dirOutput} {
    set f [open $strTextFile r]
    set strLines [read $f]
    close $f
    set listLines [prepareLines $strLines]
    # check to make sure there's an even number of lines:
    set nLineCount [llength $listLines]
    if { $nLineCount % 2 == 1 } {
	puts "Error: $strTextFile contains an odd number of lines: $nLineCount."
	return
    }
    # else
    set i 0
    while { $i < $nLineCount } {
	set strTitle [lindex $listLines $i]
	incr i
	set strDirectory [lindex $listLines $i]
	incr i
	buildCHM $strTitle $strDirectory $dirOutput
    }
}



#
# main entry point
#

if { [llength $argv] == 0 } {
    htmlProj::printUsage
} else {
    # check for an outdir flag
    set dirOutput {}
    set arg0 [lindex $argv 0]
    if { [regexp -nocase -- {^-outdir} $arg0] } {
	# this bit of cd run-around lets us convert the outdir from a
	# possibly relative path to a full path
	set dirOutput [lindex $argv 1]
	set dirCurrent [pwd]
	cd $dirOutput
	set dirOutput [pwd]
	cd $dirCurrent
	set argv [lrange $argv 2 end]
    }
    htmlProj::initDocList [windHelpDocDirGet]
    # if the arg begins with @, then read the text file
    
    set helpTitle [lindex $argv 0]
    if { [file exists ${helpTitle}.htmlProj] } {
	source ${helpTitle}.htmlProj
    } else {
	puts "Could not find definition file ${helpTitle}.htmlProj"
	exit
    }

    htmlProj::buildCHM $argv . $dirOutput
}
 








