# mg2ref.tcl - Tornado Reference documentation converter
#
# Copyright 1999 Wind River Systems, Inc.
#
# modification history
# --------------------
# 01b,20nov99,dgp  change output file name from outFile.out to outFile.ref to
#                  support target.nr migration
# 01a,23jun99,dgp  written.
#

#
# DESCRIPTION
#
# This tool implements a table-driven process for conversion of documents from
# the old `mangen' markup to the new `refgen' markup.  It can either be called for a
# single file or it can be run on either an explicit list of files or one indicated
# by wild cards.
#
# SYNOPSIS
#   mg2ref  [-cpp] [-l <logFile> ] [-out <outDir>] <sourceFileList>
#
# PARAMETERS
#   -cpp         : indicates that C++ processing is required
#   -l <logFile> : specifies that <logFile> is to be used to log refgen errors,
#                  if `-l' option is not specified, the stdandard output
#                  is used
#   -out <outDir> : the directory in which to place converted files (default is
#               current directory)
#   <sourceFileList> : Any other parameters are taken as names of source files
#                      to convert
#
# INCLUDES
#   refLib.tcl
#   OldMangen.tcl
#   refOutHtml.tcl
#
# SEE ALSO:
# \tb Coding Conventions
#
# NOROUTINES
#

# INTERNAL
#
# PROCEDURES:
#
# mg2refInit		- mg2ref initialization procedure
# mg2refComandParse	- mg2ref comand parser
# mg2ref		- mg2ref main procedure
# mg2refUsage		- mg2ref usage displayer
#


################################################################################
#
# mg2refInit - `mg2ref' initialization procedure
#
# This procedure declares and pre-sets all the global variables and then sources
# all the needed libraries.
#
# SYNOPSIS
#   mg2refInit
#
# PARAMETERS: N/A
#
# ERRORS: N/A
#
# SEE ALSO
# refgen
#

proc mg2refInit {} {

    # global variables initialization

    global SYNSEP                       ;# SYNTAX separation definition table

    # Refgen-specific markup definitions (see refMarkup.tcl)

    global refDisable
    global refMarkSpans

    # OldMangen markup definitions

    global ALTmg			;# HTML replacements for roffish stuff

    # global variables

    global env				;# global user's environment
    global fConfig                      ;# config file to use
    global logFileName                  ;# location of the log file
    global outDir                       ;# output doc files directory
    global srcPath                      ;# global tcl source path

    # global variables initialization

    set fConfig {}
    set logFileName ""
    set outDir "."
    set srcPath "[file dirname [info script]]/"

    # source required library from Tornado tree if possible, or from local dir
    # else

    if { [info exists env(WIND_BASE)] } {
        source "$env(WIND_BASE)/host/resource/doctools/refLib.tcl"
    } else {
        source refLib.tcl
    }
}



################################################################################
#
# mg2refCommandParse - `mg2ref' command parser
#
# This procedure parses and verifies `mg2ref' command line arguments and updates
# some global variables according to those arguments.
#
# SYNOPSIS
#   mg2refCommandParse <argv> <argc>
#
# PARAMETERS: 
#   argv : arguments list vector
#   argc : arguments counter
#
# ERRORS:
#   Displays help message on arguments error.
#
# SEE ALSO
# refgen
#

proc mg2refCommandParse { argv argc } {

    # global variables

    global fConfig                      ;# config file to use
    global logFileName                  ;# location of the log file
    global outDir                       ;# output doc files directory
    global srcPath                      ;# global tcl source path
    global fileList                     ;# global tcl source path
    global isCpp                        ;# C++ processing required

    set isCpp 0

    for {set ix 0} {$ix < $argc} {incr ix} {

        switch -- [lindex $argv $ix] {

	    "-cpp" {
                set isCpp 1
	    }

            "-l" {
                incr ix
                set tmpLog [lindex $argv $ix]
                if { [file exists $tmpLog] && [file writable $tmpLog] } {
                    catch [file delete $tmpLog]
                    set logFileName $tmpLog
                } elseif { [file writable [file dirname $tmpLog]] } {
                    set logFileName $tmpLog
                } else {
                    refErrorLog "refgen wrong <logFile> parameter" \
                                "file is not writable"
                    set logFileName ""
                }
            }

            "-out" {
                incr ix
                set outDir [lindex $argv $ix]
            }

            default {
                lappend fileList [lindex $argv $ix]
            }
        }
    }
}


################################################################################
#
# mg2ref - top-level documentation conversion routine
#
# After having parsed the command line, and got the parameters from it, this
# routine gets the associated markup for each file and calls `refMangenCvt'.
# `refMangenCvt' generates a new file named <outFile> (which is suitable for 
# formatting with refLib) from the existing <inFile> (which was intended for
# formatting with the first-generation WRS `mangen' tool).
#
# SYNOPSIS
#   mg2ref [-l <logFile> ] [-out <outDir>] <sourceFileList>
#
# PARAMETERS
#   -l <logFile> : specifies that <logFile> is to be used to log refgen errors,
#                  if `-l' option is not specified, the stdandard output
#                  is used
#   -out <outDir> : the directory in which to place converted files (default is
#               current directory)
#   <sourceFileList> : Any other parameters are taken as names of source files
#                      to convert
#
# ERRORS: N/A
#
# SEE ALSO
# refgen
#

proc mg2ref {} {

    # global variables initialization

    global SYNSEP                       ;# SYNTAX separation definition table
    global SYNTAX                       ;# language specific syntax array
    global SYNOP                        ;# language specific synopsis syntax array

    # Refgen-specific markup definitions (see refMarkup.tcl)

    global refDisable
    global refMarkSpans

    # OldMangen markup definitions

    global ALTmg                        ;# HTML replacements for roffish stuff

    # global variables

    global env                          ;# global user's environment
    global fConfig                      ;# config file to use
    global logFileName                  ;# location of the log file
    global outDir                       ;# output doc files directory
    global srcPath                      ;# global tcl source path
    global fileList                     ;# global tcl source path
    global inFile                       ;# global file name for conversion
    global isCpp                        ;# C++ processing required

    # local variables

    set outFile {}
    set outText {}

    # take care of the C++ processing

    if { $isCpp } {
        set fConfig Cpp2html
        eval [refConfigParse $fConfig]
        set fileList [mgCPPFileProcess $fileList ]
    }

    # process each file in fileList

    foreach inFile $fileList {

        # Select default config based on file suffix.
        # Default default is C2html; hence no need to recognize .c

        set fileConfig ""
        set fs {}
        set fs [file extension $inFile]
        switch $fs {
            ".tcl"      {set fileConfig Tcl2html}
            ".sh"       {set fileConfig sh2html}
            ".pcl"      {set fileConfig pcl2html}
            ".s"        {set fileConfig Asm2html}
            ".nr"       {set fileConfig bsp2html}
            ".java"     {set fileConfig Java2html}
        }

        # Set default values for all standard configurable parameters
        # Go through mgInFmtParse to avoid dual maintenance of
        # defaults and C2html
        # Override defaults with user's choice if present
        # (may be only a partial override)

        if {0 != [string length "$fConfig"]} {
            eval [mgInFmtParse $fConfig]
        } elseif { [string length $fileConfig] != 0 } {
            eval [mgInFmtParse $fileConfig]
        } else {
            eval [mgInFmtParse C2html]
        }

	# establish output file name and input language

	set root [file root $inFile]

	# make special provision for converting C++ header files
	
	if { $fs == ".h" } {
	    set outFile $outDir/$root.href
	} else {
	    set outFile $outDir/$root.ref
	}

	# convert the file

        refMangenCvt $inFile $outFile $inLang

    }
}


################################################################################
#
# mg2refUsage - mg2ref usage displayer
#
# This routine just displays refgen help message and exits.
#
# SYNOPSIS
#   mg2refUsage
#
# PARAMETERS: N/A
#
# ERRORS: N/A
#

proc mg2refUsage {} {

    puts stdout "\nmg2ref : source file documentation converter\n"
    puts stdout "usage :\n"
    puts stdout "mg2ref\t\[-l <logFile> \] \[-out <outDir>\] \[-nosub\] \[<sourceFileList>\]"
    puts stdout "with :\n"
    puts stdout "\t\t\[-l <logFile>\]\t: specifies that <logFile> is to be used to log"
    puts stdout "\t\t\t\t  refgen errors, if `-l' option is not specified, the standard"
    puts stdout "\t\t\t\t  output is used"
    puts stdout "\t\t\[-out <outDir>\]\t: the directory in which to place converted files"
    puts stdout "\t\t\t\t  (default is current directory)"
    puts stdout "\t\t\[-nosub\]\t: convert only files in the current working directory,"
    puts stdout "\t\t\t\t  no subdirectories"
    puts stdout "\t\t\[<sourceFileList>\]\t: Any other element is taken as a file name\n"
}


################################################################################
#
# mgInFmtParse - - parse configuration spec
#
# `mgInFmtParse' reads a simple configuration file that specifies
# the input and output formats for refgen, and perhaps other parameters.
# Its output is an executable Tcl string that, when run through `eval',
# defines the input specification by sourcing a file of table definitions
#
# By arbitrary fiat, the convention for naming refgen config files is
# to use no suffix.
#
# SYNOPSIS
#   mgInFmtParse fileName
#
# PARAMETERS
#   fileName : name of configuration file (including any suffix)
#
# RETURNS: executable Tcl string ready for eval.
#
# ERRORS: N/A
#

proc mgInFmtParse { fileName } {

    global srcPath

    if [catch {open "${srcPath}$fileName" r} conFid] {
        refErrorExit "cannot open configuration file \"${srcPath}$fileName\""
    }
    set configSpec [read $conFid]
    close $conFid

    # Eliminate comments

    regsub -all "#\[^\n]*\n" $configSpec {} configSpec

    # Turn list of pairs into array; incidentally, this checks pairs syntax

    if [catch {array set config "$configSpec"}] {
        refErrorExit "cannot parse \"$fileName\" as configuration file."
    }

    set zz {}

    # IN parameter is only parameter required for mangen conversion

    append zz "source ${srcPath}refIn$config(IN).tcl\n"
    append zz "set inLang \"$config(IN)\"\n"  ;# caller needs inLang for links
    return $zz
}


################################################################################
#
# mgCPPFileProcess - process files for C++ conversion by mangen
#
# This procedure performs a preprocessing of the C++ files. It looks
# for the IMPLEMENTS tag to get the header file name and sets up a new filelist
# that includes the required header file.
#
# SYNOPSIS
#   mgCPPFileProcess <fileList>
#
# PARAMETERS
#   fileList : the list of files to be processed as C++ files
#
# RETURNS: The list of created temp files.
#
# ERRORS: N/A
#

proc mgCPPFileProcess { fileList } {

    global SYNTAX               ;# language specific syntax array
    global SYNOP                ;# language specific synopsis syntax array

    array set fileClass {}      ;# array of classes by file
    set classList {}            ;# list of the available classes
    set tempFileList {}         ;# list of the temp files created (to return)

    foreach fileName $fileList {

        # get the file content and look for the IMPLEMENT keyword

        if { [catch "open $fileName r" fileId] } {
            refErrorLog "open $fileName r" $fileId
            continue
        }

        set fileContent [read $fileId]
        close $fileId

        # look for the IMPLEMENTS keyword, then store all the file contents in
        # an array

        if { [regexp -indices "$SYNTAX(titlStart)\[ \t]+IMPLEMENTS" \
                     $fileContent implIx] } {

            # isolate the IMPLEMENTS line.

            set implements [string range $fileContent \
                                         [expr [lindex $implIx 1] + 1] end]

            set implements [string range $implements 0 \
                                         [string first \n $implements]]

            # get rid of the comment end markup

            set commentEndMark [string first $SYNOP(cmtEnd) $implements]

            if { $commentEndMark != -1 } {
                set implements [string range $implements 0 \
                                       [expr $commentEndMark - 1]]
            }

            # just keep the header file name and the class name

            set implements [string trim $implements]
            set implLen [string length $implements]

            # the class name is considered to be the last word of the line

            set classNameStart [string wordstart $implements $implLen]
            set className [string range $implements $classNameStart $implLen]

            # the header file name should be everything but the last word

            set classHeaderFile [string range $implements 0 \
                                        [expr $classNameStart - 1]]
            set classHeaderFile [string trim $classHeaderFile]

    	    # now write the original file name and the header file 
	    # name to $tempFileList

	    append tempFileList "$fileName $classHeaderFile"
        }
    }

    return $tempFileList
}


#
# main - the mg2ref main entry
#

mg2refInit
mg2refCommandParse $argv $argc
mg2ref

