# makeErrnoTbl.tcl - script to generate the VxWorksErrnoTbl.tcl file 

# Copyright 2004-2005, Wind River Systems, Inc.
#
# The right to copy, distribute, modify, or otherwise make use 
# of this software may be licensed only pursuant to the terms 
#  of an applicable Wind River license agreement. 

# modification history
# --------------------
# 01d,17jun05,jeg  fixed SPR#109573: VxWorksErrnoTbl.tcl not fully generated
#                  if building for coldfire CPU.
# 01c,01oct04,jeg  Added control to test if the result file is writable.
# 01b,20sep04,jeg  Fixed gnu specific part and updated regular expression
#		   to work with MIPS, STRONGARM and XSCALE. Added management
#		   of errno defined in several headers.
# 01a,02apr04,lcs  written

# DESCRIPTION
# -----------
# This script will take as input the preprocessed and assembly language version
# of the file statTbl.c from which it will produce the Tcl errno table 
# VxWorksErrnoTbl.tcl and copy it to $WIND_BASE/host/resource/tcl
#

###############################################################################
#
# usage - print usage summary
#

proc usage {} {
    puts "Usage: makeErrnoTbl.tcl statTbl.cpp statTcl.s diab|gnu windBase"
    puts "The first file is a preprocessed version of statTbl.c."
    puts "The second file is an assembly version of statTbl.c."
    exit 
}

###############################################################################
#
# generateErrTbl - main work procedure
# 
# This procedure will take as input the preprocessed and assembly files, it
# will parse them using regular expressions to extract the error names and the
# corresponding values and will write the error table to the resource
# file $WIND_BASE/host/resource/tcl/VxWorksErrnoTbl.tcl.
#

proc generateErrTbl {filename asmFile tool windBase} {
    global header

    # open the input files and read their contents
    
    if {[catch {open $filename r} fileId]} {
        error "Can't open file $filename."
    }
    set fileContent [read $fileId]
    close $fileId
    
    if {[catch {open $asmFile r} fileId]} {
        error "Can't open file $asmFile."
    }
    set asmFileContent [read $fileId]
    close $fileId

 
    # locate statTbl in preprocessed file

    regexp {statTbl \[\] =[^;]+} $fileContent newContent
    regexp {\{\{.*} $newContent fileContent
    set data $header

    # locate statTbl in assembly file

    regexp {statTbl:[^:]+} $asmFileContent newAsmContent 
    regsub -all \t $newAsmContent { } newAsmContent
    
    # list of alrready treated errno strings and errno values.
    # These lists are used to avoid :
    #	- errno defined in multiple headers which will generated bad errno 
    #	  value.
    #	- multiple errno with the same value which will generate bad behavior
    #	  of windsh printErrno API.

    set listErrName "" 
    set listErrVal "" 
    set indexName 0

    # define the starting index: it is the first occurence of .L<startingIndex>
    # or .LC<startingIndex> located after "statTbl:"
    
    regexp { +(\.|\$)(L|LC)([0-9]+)} $newAsmContent val val1 startingIndex \
    	startingIndex

    # for each line in the statTbl located in the preprocessed file extract the
    # name of the error, then use the assembly file to determine the 
    # corresponding errno value

    set idx $startingIndex 
    foreach line [split $fileContent \n] {
    	# extract the errno name from the cplus file

	regsub -all , $line { } line
	regsub -all {\(char \*\)} $line {} line
	regsub -all {\{\(\(void \*\)0\)\}} $line {} line
	regsub -all {\{\ 0\ \}} $line {} line
	regsub -all {\(} $line \{ line
	regsub -all {\)} $line \} line

	set name [lindex [lindex $line 0] $indexName]

	if {$name != "" && $name != "\}" && [lsearch $listErrName $name] < 0} {
	    # test if the errno has not already been treated 
	    # to take into account the errno defined in different headers.

	    regexp "\.(L|LC)$idx\.\*" $newAsmContent newAsmContent
	    regexp { +(\.long|\.word|DC\.L) +([0-9]+)} $newAsmContent \
	    	val val1 val2

	    # take care of the special case where different errno 
	    # have the same value. This could generate a windsh issue
	    # when using printErrno() API.

	    if {[lsearch $listErrVal $val2] < 0} {
		append data "set errnoTbl($val2) $name\n"
		append listErrName "$name "
		append listErrVal "$val2 "
	    }
	    incr idx
	}
    }

    # open the file errnoTcl.tcl and write the errno table to it

    set path $windBase 
    append path "/host/resource/tcl/VxWorksErrnoTbl.tcl"

    catch {[file delete -force $path]}

    if {[catch {open $path w+} fileId]} {
        puts "\nWarning: Can't open file $path for writing.\n"
    } else {
	puts $fileId $data
	close $fileId
    }
}

set header \
"# VxWorksErrnoTbl.tcl - correspondance table between error-status value and
#		 error-status string
#
# This files is generated with makeErrnoTbl script, located in 
# \$WIND_BASE/target/src/usr. It will be created when a make occurs in the 
# afore mentioned directory.
#
# DESCRIPTION
# This module only sets the global table errnoTbl with the error-status 
# string corresponding to the error-status value.
#
# RESOURCE FILES: N/A

set errnoTbl(0) OK\n"

set file [lindex $argv 0]
if {$file == ""} {
    usage
}
set asmFile [lindex $argv 1]
if {$asmFile == ""} {
    usage
}
set tool [lindex $argv 2]

# get only tool type 

regexp {diab} $tool tool

set windBase [lindex $argv 3]
if {$windBase == ""} {
    usage
}
generateErrTbl $file $asmFile $tool $windBase
