# makeSymTbl.tcl - built-in system symbol table creation utility
#
# Copyright (c) 2001-2005 Wind River Systems, Inc. 
# 
# The right to copy, distribute, modify or otherwise make use of this software
# may be licensed only pursuant to the terms of an applicable Wind River 
# license agreement.
#
# modification history
# --------------------
# 01m,01dec05,v_r  Integrated ColdFire support from Duster view. Reworked 
#		   comments.
# 01l,11may05,bpn  Added __ieee_status definitions for ARM (SPR#109097)
# 01k,30sep04,jn   Leave vx_offset symbols in symbol table
# 01j,10sep04,j_b  add vxWorksVersion* handling to text section (SPR 100237)
# 01i,03jun04,md   Fix vxWorksVersion* defines
# 01h,13may04,rhe  Fix creationDate, runtimeVersion, runtimeName &
#                  vxWorksVersion
# 01g,05apr04,pad  Adapted to updated (symRef field) SYMBOL type (SPR 95690).
# 01f,19feb04,aeg  strip out absolute symbols representing struct offsets.
# 01e,27jun03,jmp  no more leading underscore on SIMNT.
# 01d,18nov02,mcm  added support for some of Diab's intrinsic routines.
# 01c,31jan02,fmk  write directly to a file to generate symTbl.c rather than
#                  use redirection
# 01b,13dec01,j_s  prepend '0x' to absolute address
# 01a,30oct01,j_s   created based on TAE 3.1 utility (base version:
#                  makeSymTbl.tcl@@/main/tor3_x/10).
#
# SYNOPSIS:
# .tS
# makeSymTbl.tcl <cpu type> <objMod>
# .tE
#
# DESCRIPTION
# This tool creates the C code for a symbol table structure containing the
# names, addresses and types of all meaningful global symbols in the specified
# object module; the generated code is written to standard output.
#
# This tool is used only when creating a standalone system (`vxWorks.st').
# Development systems do not need this since the system symbol table is
# constructed by reading and interpreting the symbol table contained in the
# loadable system symbol table module (`vxWorks.sym'), either from the local
# boot disk (floppy, ata/ide, scsi, etc.) or from a host system over the
# network.
#
# The built-in symbol table is an array of type SYMBOL accessible through
# the global variable `standTbl'.  The array contains `standTblSize' elements.
# These are the only external declarations in the generated code. When the
# standalone VxWorks image is built from a BSP the symbols are inserted in
# the system symbol table  by usrRoot() in usrConfig.c. When the standalone
# VxWorks image is built from a project, the symbols are inserted by the
# usrStandaloneInit() routine in the comps/src/usrStandalone.c configlette.
#
# For an example, see the file $WIND_BASE/target/config/<bspName>/symTbl.c,
# which is generated by this tool for `vxWorks.st' in the same directory.
#
# FILES
# .TS
# tab(|);
# lf3 l.
# symLib.h   |  symbol table header file
# symbol.h   |  VxWorks symbol definitions header file
# .TE
#
# SEE ALSO:
#
# NOROUTINES


# namespace globals

namespace eval makeSymTbl {
    set nmCmd		"nm"
    set nmFlags		"-g"
    set symPrefixToIgnore ""
    set symPrefixToAdd	""
    set cpuType		""
    set fdOut           ""
    set symDecl() ""
    set symDefT() ""
    set symDefD() ""
}

##############################################################################
#
# standTblCreate - create an array of symbol for an object module
#
# This procedure creates an array of symbols (C code) based on a file symbol
# table.
#
# The input of this procedure is a module. The output of this
# procedure is stored in a file named symTbl.c.
#
# SYNOPSIS:
#   standTblCreate { objMod }
#
# PARAMETERS:
#   module name
#
# RETURNS: number of symbols in the module's symbol table
#
# ERRORS: N/A
#

proc makeSymTbl::standTblCreate {modName} {
    variable symPrefixToIgnore
    variable symPrefixToAdd
    variable cpuType
    variable nmCmd
    variable nmFlags
    variable symDecl
    variable symDefT
    variable symDefD
    variable fdOut

    set symbolInfo ""
    set symList {}
    set nsyms 0

    # get all the global/local symbols from the object module

    set symbolInfo [eval exec $nmCmd $nmFlags $modName]

    # replace consecutive whitespaces with single whitespace

    regsub -all { +} $symbolInfo " " symbolInfo

    set symbolInfoListRaw [split $symbolInfo \n]
    set symbolTblEntryList {}

    # Reject the symbols that are either meaningless for VxWorks or local
    # symbols (then can not be linked). These symbols are represented by the
    # following letters in nm's output:
    #
    # G: symbol from an initialized data section for small objects (not
    #    supported in VxWorks).
    # I: indirect reference to another symbol (extension of the a.out format
    #    specific to GNU)
    # N: debugging symbol (no point in having them in the system's symbol
    #    table).
    # S: symbol from an uninitialized data section for small objects (not
    #    supported in VxWorks).
    # U: undefined symbols (hence not part of the system's symbol table).
    # -: stabs symbol in an a.out object file (no point in having them in the
    #    system's symbol table).
    # ?: symbol of unknown type.
    #
    # all lower case letters: local symbols (they cannot be used to link
    # symTbl.c with the VxWorks library). Filtered by the -g option of nm.
    #
    # See the manpage of GNU nm for a complete explanation of the letters
    # representing the symbol types.

    # Define architecture specific symbol exclusion rules

    set archSpecificTest ""

    if {$cpuType == "68k"} {

	# For 68k, all global symbols which don't begin with '_' should be
        # filtered out. Such symbols are from assembly code and can't be
        # referenced in a C file .
	# This is in fact true for ColdFire as well but since Diab outputs such
	# symbols (SPR#80951), we cannot simply strip them out. See below for
	# more information.

	set archSpecificTest "if \{\"\[string index \$name 0\]\" != \"_\"\} \
			      \{continue\}"
    } elseif {$cpuType == "cf"} {

	# For ColdFire, some Diab symbols have to be filtered out. This is also
        # related to SPR#80951.

	set archSpecificTest "if \{\[regexp \{(STATUS|float|double)$\} \$name\]\} \
			      \{continue\}"
    }

    # Parse the symbol list to create the symbol table

    foreach symbolEntry $symbolInfoListRaw {
	# Store output fields
	if {![regexp {^(.*) (.*) (.*)} $symbolEntry dummy addr type name]} {
	    continue
	}

	# Remove unwanted symbols. This also comprises symbols names beginning
	# with a dot : they would trigger a parse error in a C file.
	# Note : locals were already filtered out by the -g option of nm.
        if {[string match {[GINSU\?\-]} $type] 
	    || [string first "." $name] != -1} {
	    continue
	}

	# Do architecture specific tests

	eval $archSpecificTest

        # Convert the filtered nm output to an IMPORT list of symbols;
	#
	# Symbols could be of following types:
	#
	# A: the symbol's value is absolute, and will not be changed by further
	#    linking.
	# B: the symbol is in the uninitialized data section (known as BSS).
	# C: the symbol is common. Common symbols are uninitialized data.
	# D: the symbol is in the initialized data section.
	# R: the symbol is in a read only data section.
	# T: the symbol is in the text (code) section.
	# V: the symbol is a weak object.
	# W: the symbol is a weak symbol that has not been specifically tagged
	#    as a weak object symbol.
	#
	# Some symbols need special handling via the symDecl, symDefT and
	# symDefD assertions. See below for more information.

	regsub $symPrefixToIgnore $name "" cName
            
	switch -glob $type {
	    [A] {
		lappend symbolTblEntryList "\{\{NULL\}, \"$cName\",\
					    (char*) 0x$addr, 0, 0,\
					    SYM_GLOBAL | SYM_ABS\},"
	    }
	    [DR] { 
		if [info exists symDecl($cName)] {
			puts $fdOut "[subst $symDecl($cName)]"
		} else {
		    puts $fdOut "IMPORT int ${cName};"
		}

		if [info exists symDefD($cName)] {
		    lappend symbolTblEntryList "[subst $symDefD($cName)]"
		} else {
		    lappend symbolTblEntryList "\{\{NULL\}, \"$name\",\
						(char*) \&$cName, 0, 0, \
						SYM_GLOBAL | SYM_DATA\},"
		}
	    }
	    [BC] { 
		puts $fdOut "IMPORT int ${cName};"
		lappend symbolTblEntryList "\{\{NULL\}, \"$name\",\
					    (char*) \&$cName, 0, 0, \
					    SYM_GLOBAL | SYM_BSS\},"
	    }
	    [TWV] {
		if [info exists symDecl($cName)] {
		    puts $fdOut "[subst $symDecl($cName)]"
		} else {
		    puts $fdOut "IMPORT int ${cName} ();"
		}

		if [info exists symDefT($cName)] {
		    lappend symbolTblEntryList "[subst $symDefT($cName)]"
		} else {
		    lappend symbolTblEntryList "\{\{NULL\}, \"$name\",\
						(char*) $cName, 0, 0, \
						SYM_GLOBAL | SYM_TEXT\},"
		}
	    }
	    default {
		puts "Warning: makeSymTbl.tcl - invalid symbol information \
			($symbolEntry)"
	    }
	}
    }

    set nsyms [llength $symbolTblEntryList]

    # convert nm output to symbol entries in array

    puts $fdOut ""
    puts $fdOut "SYMBOL ${symPrefixToAdd}standTbl \[$nsyms\] ="
    puts $fdOut "    {"

    foreach symbolTblEntry $symbolTblEntryList {
        puts $fdOut "        $symbolTblEntry"
    }

    puts $fdOut "    };"
    puts $fdOut ""

    return $nsyms
}

##############################################################################
#
# main - entry point of utility
#

# check for correct number of args

if {$argc < 3} {
    puts stderr "Usage: makeSymTbl.tcl <cpu type> <objMod> <file>"
    exit 1
}

set outFile [lindex $argv 2]

# set appropriate version of the nm command for the processor

if [catch {set makeSymTbl::cpuType [lindex $argv 0]}] {
    set makeSymTbl::cpuType ""
}

set makeSymTbl::nmCmd "nm$makeSymTbl::cpuType"

# For sh/68k, the toolchain prepended underscore is stripped from all symbols
# names. 
if {($makeSymTbl::cpuType == "sh") || ($makeSymTbl::cpuType == "68k")} {
    set makeSymTbl::symPrefixToIgnore "_"
}

# SPR#80951: Diab special symbols have no underscore prepended. The solution to
# correctly include them in is to declare/define all symbols with no underscore
# prepended and to build as such with a Diab pragma (see below).
if {($makeSymTbl::cpuType == "cf")} {
    set makeSymTbl::symPrefixToAdd "_"
}

# parse the arguments

set modName [lindex $argv 1]

# Create the module's standalone symbol table

set makeSymTbl::fdOut [open $outFile w+]

# The following (C code) goes into the wrapper file symTbl.c

puts $makeSymTbl::fdOut "/* symTbl.c - standalone symbol tables wrapper */"
puts $makeSymTbl::fdOut ""
puts $makeSymTbl::fdOut "/* CREATED BY $argv0"
puts $makeSymTbl::fdOut " *  WITH ARGS $argv"
puts $makeSymTbl::fdOut " *         ON [clock format [clock seconds]]"
puts $makeSymTbl::fdOut " */"
puts $makeSymTbl::fdOut ""
puts $makeSymTbl::fdOut "#include \"vxWorks.h\""
puts $makeSymTbl::fdOut "#include \"symbol.h\""
puts $makeSymTbl::fdOut ""

if {($makeSymTbl::cpuType == "cf")} {
    # Use a Diab pragma to tell the compiler we are handling underscores
    # in a special way. Note that ColdFire only supports Diab for 6.2 so we use
    # the pragma inline.
    # Note : the pragma is often referred to as -Xunderscore-leading in the
    # diab docs. Both are synonyms.

    puts $makeSymTbl::fdOut "#pragma option -Xleading-underscore=0"

    # Since this fix is only for Diab and we do not support GNU for ColdFire,
    # prevent the build.

    puts $makeSymTbl::fdOut "#define gnu 1234"
    puts $makeSymTbl::fdOut "#if (TOOL_FAMILY == gnu)"
    puts $makeSymTbl::fdOut "#error The compilation of symTbl.c with the\
			    TOOL_FAMILY toolchain is not supported."
    puts $makeSymTbl::fdOut "#endif"
    puts $makeSymTbl::fdOut "#undef gnu"
    puts $makeSymTbl::fdOut ""
}

# The following are for symbols that require special handling. These include :
# 1. Some symbols that are declared in sllLib.h, which is included through
#    symbol.h. They must be declared correctly or they cause multiple 
#    definition complaints.
# 2. Some intrinsic routines for Diab (__ff* functions) also need to be
#    correctly declared.
# 3. vxWorks versioning symbols. These also need special declarations in case
#    special compiler pragmas are used which make them end in the TEXT section
#    (legacy T22 compatibility).
# 4. ieee-related symbol (ARM only).
#
# Since a symbol can have only one type at a time, symDecl is sufficient for
# both DATA and TEXT types. However, as a symbol can have different, manually
# specified, definitions for TEXT and DATA, we need symDefT and symDefD.
# Otherwise, the wrong definition would be picked up.

# DATA symbols that require special DECLARATION handling

set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}sllCreate) "IMPORT SL_LIST *\$cName \(\);"
set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}sllGet) "IMPORT SL_NODE *\$cName \(\);"
set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}sllPrevious) "IMPORT SL_NODE *\$cName \(\);"
set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}sllEach) "IMPORT SL_NODE *\$cName \(\);"
set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}sllPutAtHead) "IMPORT void \$cName ();"
set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}sllPutAtTail) "IMPORT void \$cName ();"
set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}sllRemove) "IMPORT void \$cName ();"

set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}__ff1) "IMPORT unsigned int \$cName (unsigned int);"
set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}__ff1l) "IMPORT unsigned int \$cName (unsigned int);"
set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}__ff1ll) "IMPORT unsigned int \$cName (unsigned long long);"

set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}creationDate) "IMPORT char * \$cName;"
set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}runtimeVersion) "IMPORT char * \$cName;"
set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}runtimeName) "IMPORT char * \$cName;"
set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}vxWorksVersion) "IMPORT char * \$cName;"
set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}vxWorksVersionMajor) "IMPORT const unsigned int \$cName;"
set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}vxWorksVersionMinor) "IMPORT const unsigned int \$cName;"
set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}vxWorksVersionMaint) "IMPORT const unsigned int \$cName;"

set makeSymTbl::symDecl(${makeSymTbl::symPrefixToAdd}__ieee_status) "IMPORT unsigned int \$cName (unsigned int mask, unsigned int flags);"

# TEXT symbols that require special DEFINITION handling
# Note : special handling for the vxWorks versioning symbols was used for legacy
# compatibility in T22 (SPR#100237). The option seem to have been deprecated in
# B6 (has no effect on object files + not documented anymore). Removing these
# assertions could be an improvement for future versions of this tool.

set makeSymTbl::symDefT(${makeSymTbl::symPrefixToAdd}vxWorksVersionMajor) "\{\{NULL\}, \"\$cName\", (char*) \&\$cName, 0, 0, SYM_GLOBAL | SYM_TEXT\},"
set makeSymTbl::symDefT(${makeSymTbl::symPrefixToAdd}vxWorksVersionMinor) "\{\{NULL\}, \"\$cName\", (char*) \&\$cName, 0, 0, SYM_GLOBAL | SYM_TEXT\},"
set makeSymTbl::symDefT(${makeSymTbl::symPrefixToAdd}vxWorksVersionMaint) "\{\{NULL\}, \"\$cName\", (char*) \&\$cName, 0, 0, SYM_GLOBAL | SYM_TEXT\},"

# DATA symbols that require special DEFINITION handling
# (add yours here if needed with the symDefD assertion)

# Create the symbol table

set nsyms [makeSymTbl::standTblCreate $modName]

puts $makeSymTbl::fdOut ""
puts $makeSymTbl::fdOut "ULONG ${makeSymTbl::symPrefixToAdd}standTblSize = ${nsyms};"

close $makeSymTbl::fdOut
