##############################################################################
# scgen.tcl - system call stubs and header generation utility
#
# Copyright (c) 2003-2005 Wind River Systems, Inc.
#    
# The right to copy, distribute, modify or otherwise make use
# of this software may be licensed only pursuant to the terms
# of an applicable Wind River license agreement.
#
# modification history
# --------------------
# 02p,10may06,mcm  Fix for WIND00044193 - scgen is able to introduce padding
#                  for unaligned 8-byte quantities and now can also handle the
#                  case when an unaligned 8-byte value should be retained.
#                  This is done by packing the container structure to 4.
# 02o,02jan06,yvp  SPR 116023: fix wrong handling of duplicate group defs.
# 02n,02dec05,mcm  Changed build environment to :vxworks63
# 02m,06sep05,yvp  Added support for GNU compilers (SPR )
# 02l,31aug05,mcm  Changing dcc build environment to :vxworks62
# 02k,07jul05,dbt  Added support for padding array on sparc.
# 02j,15jul05,rec  coldfire support for syscall.h generation, use WRS_BIT_OR
#                  macro
# 02i,08apr05,rec  Coldfire update
# 02h,03may05,lei  Removed temp file; changed build specs to vxworks61.
# 02g,22apr05,yvp  Direct compilers stderr to a file (SPR 108391).
#                  Changed all dcc build specs to :vxworks61
# 02f,03nov04,mcm  Fix for SPR#103405 - using :vxworks60 as the target 
#                  environment in dccflag
# 02e,14oct04,yvp  Error-check for redifinition of syscall groups (SPR 102396).
# 02d,28apr04,yvp  Added debug instrumentation.
# 02c,13apr04,yvp  Bug fix: eliminated decrement of groupNRoutines (line 986)
# 02b,07apr04,yvp  Handled MIPS64 alignment issues correctly.
# 02a,12mar04,yvp  Split syscall.def into syscallNum.def and syscallApi.def
#                  to separate the number space from syscall API's. Added 
#                  provision for experimental syscall development. 
#                  syscall.h generated directly into target/h & target/usr/h
#                  Used pid and temporary files to fix parallel build issues.
#                  Fix bugs that allowed duplicate groups & syscalls.
#                  Use WIND_USR env. variable for userland files.
#                  
# 01e,15dec03,yvp  Changed names of syscallArgs*.h.
# 01d,12dec03,yvp  Changed syscall.c to syscallTbl.h and location.
# 01c,11dec03,yvp  Incorporated PaulB's modifications: system calls now
#                  defined with a full prototype. This script generates
#                  argument structures keeping ABI specifics in mind.
# 01b,10oct03,yvp  Made script operation modal; based on cmd line options.
# 01a,18sep03,yvp  created 
#
# SYNOPSIS:
# .tS
#
#scgen [-ht] [-a <abiSpec> <CPU> <archDir>] [-A <abiSpec> <CPU> <archDir>] 
#      [-s <archDir>]
#        -a Generate all files
#        -A Generate syscallargs.h
#        -h Generate syscall.h
#        -s Generate stubs in arch
#        -t Generate syscallTbl.h
# .tE
#
# DESCRIPTION
# This tool is used when generating the system call assembly stubs, their
# header file syscall.h and a few other header files. The assembly stubs 
# are then compiled into the user-mode library libvx.a. The generated
# syscallTbl.h contains the initialized System Call Group Table. 
#
# SEE ALSO:
#
# NOROUTINES


#<<DEBUG_CODE>>
proc parray name {
    upvar $name a 
    foreach el [lsort [array names a]] {
	puts "$el = $a($el)"
    }
}
#<<end DEBUG_CODE>>


##############################################################################
#
# syscallHOutputBegin - Ouput the file header for syscall.h
#

proc syscallHOutputBegin {fd} {
puts $fd "/* syscall.h - System Call Group and Number definition file */"
puts $fd ""
puts $fd "/* Copyright (c) 2003-2005 Wind River Systems, Inc. */"
puts $fd ""
puts $fd "/* THIS FILE IS AUTO_GENERATED. PLEASE DO NOT EDIT. */"
puts $fd ""
puts $fd "\#ifndef __INCsyscallh"
puts $fd "\#define __INCsyscallh"
puts $fd ""
puts $fd "\#ifdef __cplusplus"
puts $fd "extern \"C\" \{"
puts $fd "\#endif"
puts $fd ""
puts $fd "    /* defines */"
puts $fd ""
puts $fd "    /*"
puts $fd "     * SYSTEM CALL NUMBERS:"
puts $fd "     * System Call Numbers (SCN) are unique 32-bit numbers identifying "
puts $fd "     * the kernel routine requested by the user. The SCI decodes this  "
puts $fd "     * number to identify and call the requested kernel function on "
puts $fd "     * behalf of the application. The SCN is organized as follows:"
puts $fd "     * "
puts $fd "     *                  SYSCALL_GROUP_NUM_BITS  SYSCALL_ROUTINE_NUM_BITS"
puts $fd "     * -----------------------------------------------------------------"
puts $fd "     * |               |                       |                       |"
puts $fd "     * |   reserved    |         Group#        |        Routine#       |"
puts $fd "     * |               |                       |                       |"
puts $fd "     * -----------------------------------------------------------------"
puts $fd "     * 31                                                              0"
puts $fd "     */"
puts $fd ""
}


##############################################################################
#
# syscallHOutputMiddle - Ouput some middle-of-the-file for syscall.h
#

proc syscallHOutputMiddle {fd} {
puts $fd ""
puts $fd "#define SYSCALL_NUMBER(grp,rtn) \\"
puts $fd "        (((grp) << SYSCALL_GROUP_NO_BIT_START) _WRS_BIT_OR (rtn))"
puts $fd ""
puts $fd ""
puts $fd "    /* List of System Call Groups "
puts $fd "     * "
puts $fd "     * Groups 0 and 1 are reserved for future Wind River use. "
puts $fd "     * Groups 2 through 7 are for customer use."
puts $fd "     * Groups 8 onwards are for Wind River and partner use."
puts $fd "     */"
puts $fd ""
}


##############################################################################
#
# syscallHOutputEnd - Ouput the file ending for syscall.h
#

proc syscallHOutputEnd {fd} {
puts $fd "\#ifdef __cplusplus"
puts $fd "\}"
puts $fd "\#endif"
puts $fd ""
puts $fd "\#endif /* __INCsyscallh */"
}


##############################################################################
#
# isnumber - return 1 if the parameter is a base 10 number
#

proc isnumber {arg} {
    set n ""
    if {$arg == 0} {
	return 1
    }
    regexp {[1-9][0-9]*} $arg n
    if {$n == $arg} {
	return 1
    }
    return 0
}

##############################################################################
#
# putDefines - Print an #if line for the list of defines
#
# putDefines stdout "aa" will output "#if defined(aa)"
# putDefines stdout {aa bbb cccc} will output -
#		    #if defined(aa) || defined(bbb) || defined(cccc)
#

proc putDefines {outFile defList} {
    set i 0
    set reqDef [lindex $defList $i]
    if {$reqDef != ""} {
	puts -nonewline $outFile [format "\#if defined(%s)" $reqDef]
	incr i
	while {[lindex $defList $i] != ""} {
	    puts -nonewline $outFile [format " || defined(%s)" [lindex $defList $i]]
	    incr i
	}
	puts $outFile ""
    }
}

##############################################################################
#
# elementName - Extract the name of a symbol from a C declaration statement.
#
# elementName "int foo;" returns "foo"
# elementName "int foo (void);" returns "foo"
# elementName "int (*foo) (void);" also returns "foo"
#

proc elementName {a} {
    set a [string trim $a]
    set n [string last \; $a]
    incr n -1
    set a [stripFuncInfo [ string range $a 0 $n ]]
    string range $a [string wordstart $a [string length $a]] 99
}


##############################################################################
#
# stripFuncInfo - Strip argument list information from a function signature
#                 (helper function for elementName)
# 
#
# stripFuncInfo "int foo (void)"  returns "int foo"
# stripFuncInfo "int (*foo)(bar)" returns "int (*foo"
# stripFuncInfo "int (*foo)((int)(*bar)(int))" also returns "int (*foo"
#

proc stripFuncInfo {s} {
    set s [string trim [stripParens $s]]
    set c [string index $s end]
    if { $c == ")" || $c == "\]" } {
	stripParens [string range $s 0 end-1]
    } else {
	string trim $s
    }
}


##############################################################################
#
# stripParens - Strip trailing information between parenthesis
#               (helper function for stripFuncInfo)
# 
#
# stripParens "int (*foo)(bar)" returns "int (*foo)"
# stripParens "int (*foo)((int)(*bar)(int))"  returns "int (*foo)"
#

proc stripParens {s} {
    set s [string trim $s]
    set n 1
    set i [string length $s]
    incr i -1
    set c [string index $s $i]
    if {$c == ")"|| $c == "\]" } {
	while { $i > 0 && $n > 0 } {
	    incr i -1
	    set c [string index $s $i]
	    if { $c == ")" || $c == "\]" } {
		incr n;
	    } elseif { $c == "(" || $c == "\[" } {
		incr n -1;
	    }
	}
	incr i -1
	string trim [string range $s 0 $i]
    } else {
	    string trim $s
    }
}

##############################################################################
#
# emitStruct - add any padding and reorganization needed to an argument
#              structure and print it. This is where we consider the target
#              ABI spec and act accordingly.
#

proc emitStruct {fd groupName rtnName eStruct eNames eo es gd sd} {
    upvar $eo eOffsets
    upvar $es eSizes
    global abiSpec
    global elements
    global ScNumber
    global paddingEnumList

# <<DEBUG_MSG>>
#     puts stdout [format "emitStruct: groupName = %s" $groupName]
#     puts stdout [format "emitStruct: rtnName = %s" $rtnName]
#     puts stdout [format "emitStruct: eStruct = %s" $eStruct]
#     puts stdout "emitStruct: eNames :-"
#     puts stdout $eNames
#     puts stdout "emitStruct: eo :-"
#     parray $eo
#     puts stdout "emitStruct: es :-"
#     parray $es
#     puts stdout "emitStruct: gd :-"
#     puts stdout $gd
#     puts stdout "emitStruct: sd :-"
#     puts stdout $sd
# <<end DEBUG_MSG>>

    if {[llength $gd] > 0} {
	putDefines $fd $gd
    }
    if {[llength $sd] > 0} {
	putDefines $fd $sd
    }

    # This loop will search for doubles crossing the register/stack boundary
    # on the sh4

    if {$abiSpec == "SH4"} {
	set ePad 0
	set eUse {};
	set skipNames {}
	foreach eName $eNames {
	    set eSize $eSizes($eName)

	    # Check for doubles crossing the regsiter stack boundary

	    if {$eSize == 8 && $ePad == 12} {
		lappend skipNames $eName
		continue;
	    }

	    # If we are back filling a skipped register, skip forward until
	    # we have a 4 byte register

	    if {$eSize == 8 && [llength $skipNames] != 0} {
		lappend skipNames $eName
		continue
	    }

	    incr ePad $eSizes($eName)
	    lappend eUse $eName

	    # if we skipped any doubles, output them out now

	    if {[llength $skipNames] != 0} {
		foreach eName $skipNames {
		    lappend eUse $eName
		    incr ePad $eSizes($eName)
		}
		set skipNames {}
	    }
	}

	# If we had skipped doubles and there is no trailing 4 byte argument
	# insert the required pad

	if {[llength $skipNames] != 0} {
	    set eSizes(swapPad) 4
	    set elements([format "%s->swapPad" $eStruct]) "int swapPad;"
	    lappend eUse swapPad
	    foreach eName $skipNames {
		lappend eUse $eName
	    }
	}
	set eNames $eUse
    }

    # This loop will insert pads before non-aligned 8 byte parameters

    if {$abiSpec == "pad8"} {
	set ePad 0
	set eUse {};
	set paddingEnum 0
	foreach eName $eNames {
	    set eSize $eSizes($eName)

	    #
	    # Check for unaligned doubles
	    #
	    if {$eSize == 8 && [expr $ePad >> 2 & 1] == 1} {
		set padName [format "%sPad" $eName]
		set eSizes($padName) 4
		set elements([format "%s->%s" $eStruct $padName]) \
		    [format "int %s;" $padName]
		set paddingEnum [expr $paddingEnum | 1 << ($ePad >> 2)]
		incr ePad 4
		lappend eUse $padName
	    }
	    incr ePad $eSizes($eName)
	    lappend eUse $eName
	}
	set eNames $eUse
	if {$paddingEnum != 0} {
	    lappend paddingEnumList "SYSCALL_NUMBER ($groupName, $ScNumber($rtnName)), [format "%#x" $paddingEnum]"
	}
    }

    # This loop will retain unaligned doubles (used for ColdFire for example)

    set doNotAddPadding 0
    if {$abiSpec == "CF"} {
        set eOff 0       
        foreach eName $eNames {
            set eSize $eSizes($eName)

            #
            # Check for unaligned doubles
            #
            if {$eSize == 8 && [expr $eOff >> 2 & 1] == 1} {
                set doNotAddPadding 1
                break
            }
            incr eOff $eSize
        }
    }

    # This loop handles MIPS64 Big-Endian mode.

    if {$abiSpec == "MIPS64BE"} {
	set ePad 0
	set eUse {};

	foreach eName $eNames {
	    set eSize $eSizes($eName)
	    #
	    # Check for 32-bit sized variables. They are placed at 4-byte 
	    # boundaries, and a pad is inserted at 8-byte boundaries.
	    # 64-bit variables are always at 8-byte boundaries.
	    #
	    if { ($eSize == 4 && [expr $ePad >> 2 & 1] == 0) || \
		 ($eSize == 8 && [expr $ePad >> 2 & 1] == 1)} {
		set padName [format "%sPad" $eName]
		set eSizes($padName) 4
		set elements([format "%s->%s" $eStruct $padName]) \
		    [format "UINT32 %s;" $padName]
		incr ePad 4
		lappend eUse $padName
	    }
	    incr ePad $eSizes($eName)
	    lappend eUse $eName
	}
	set eNames $eUse
    }

    # This loop handles MIPS64 Little-Endian mode.

    if {$abiSpec == "MIPS64LE"} {
	set ePad 0
	set eUse {};

	foreach eName $eNames {
	    set eSize $eSizes($eName)
	    #
	    # Check for 32-bit sized variables. They are placed at 8-byte 
	    # boundaries, and a pad is inserted at 4-byte boundaries
	    # 64-bit variables are always at 8-byte boundaries.
	    #
	    if { ($eSize == 4 && [expr $ePad >> 2 & 1] == 1) || \
		 ($eSize == 8 && [expr $ePad >> 2 & 1] == 1)} {
		set padName [format "%sPad" $eName]
		set eSizes($padName) 4
		set elements([format "%s->%s" $eStruct $padName]) \
		    [format "UINT32 %s;" $padName]
		incr ePad 4
		lappend eUse $padName
	    }
	    incr ePad $eSizes($eName)
	    lappend eUse $eName
	}
	set eNames $eUse
    }

    if {$doNotAddPadding == 1} {
        # Note that this construct is Diab specific and is only used for ColdFire
        puts $fd "__packed__(4,4)"
    }
    puts $fd [format "struct %sScArgs \n\t{" $eStruct]
    foreach eName $eNames {
	puts $fd [format "\t%s" $elements([format "%s->%s" $eStruct $eName])]
    }
    puts $fd "\t};"

    if {[llength $gd] > 0} {
	puts $fd "#endif"
    }
    if {[llength $sd] > 0} {
	puts $fd "#endif"
    }
    puts $fd ""
}

proc usage {} {
    global abiSpecs
    global abiHelp

    puts "Usage: scgen \[-ht\] \[-a <abiSpec> <CPU> <archDir>\]"
    puts "                   \[-A <abiSpec> <CPU> <archDir>\]" 
    puts "                   \[-s <archDir>\]"
    puts "        -a Generate all files"
    puts "        -A Generate syscallargs.h"
    puts "        -h Generate syscall.h"
    puts "        -s Generate stubs in arch"
    puts "        -t Generate syscallTbl.h"
    puts ""
    puts "Known ABI specs:"
    set searchID [array startsearch abiSpecs]
    while {[array anymore abiSpecs $searchID] == 1} {
	set a [array nextelement abiSpecs $searchID]
	puts [format "        %8s - %s" $a $abiHelp($abiSpecs($a))]
    }
    exit 1
}

##############################################################################
#
# fileParse - parse an input file 
#
# We store the information in the following arrays and lists:
#
# defineList		List of macros that have been defined
# groupList		List of all groups defined in syscallNum.def
# scNameList		List of all system calls defined in syscallNum.def
# apiFuncList		List of all system call functions whose API is defined
#			in syscallApi.def
# maxGroup		Max group number defined
# groupNames(#)		Group name associated with group number #
# groupNumber(G)	Group number for group G
# groupRoutines(G)	List of routines defined for group G
# groupNRoutines(G)	Max routines number defined for group G
# 
# ScArgs(R)		Structure of parameters from system call R
# ScGroup(R)		Group name for system call R
# ScNumber(R)		System call number for R
# ScDefs(R)		And defines required for system call R
# ScNargs(R)		Number of arguments to system call R
# eStruct		Name of current structure (less the ScArgs)
# eNames		List of elements in current argument structure
# eOffsets(E)		Offset of element E in current argument structure
# eSizes(E)		Size of element E in current argument structure
# elements(S,E)		Actual declared element E in structure S
#
# In order to use elements you need to index as follows:
# 
# 	elements([format "%s->%s" $eStruct $eName])
#

proc fileParse {filePath} {

global maxGroup
global defineList
global defineValue
global groupDefs
global groupNames
global groupNumber
global groupRoutines
global groupNRoutines
global ScArgs
global ScArgList
global ScGroup
global ScNumber
global ScDefs
global ScNargs
global ScNames
global includes
global groupList
global elements
global scNameList
global apiFuncList
global paddingEnumList

set inFile [open $filePath r]

set groupName ""

while {[gets $inFile line] >= 0} {
    
    # Strip comments, they are lines that start with a #
    if {[string index $line 0] == "\#"} {continue}
    
    # Strip blank lines
    if {[string length [string trim $line]] == 0} {continue}
    
    # Do \ processing to concatanate lines
    set line [string trim $line]
    while {$line !=  [string trimright $line "\\"]} {
	set line [string trimright $line "\\"]
        set line [string trim $line]
	gets $inFile eLine
	append line " "
	append line [string trim $eLine]
    }

    if {[lindex $line 0] == "SET"} {
	set defineValue([lindex $line 1]) [lindex $line 2]
	lappend defineList [lindex $line 1]
	continue
    }

    if {[lindex $line 0] == "INCLUDE"} {
	lappend includes [lindex $line 1]
	continue
    }

    if {[lindex $line 0] == "SYSCALL_GROUP"} {
	set groupName [lindex $line 1]
	set groupNo [lindex $line 2]

	if {[info exists groupNumber($groupName)] == 1} {
	    set prevGoupNo $groupNumber($groupName)
	    
	    if {$prevGoupNo == $groupNo} {
		#set groupName ""
		#set groupNo ""
		continue
	    } else {
		puts stdout [format "\nERROR: conflicting group numbers defined for %s" $groupName ]
		puts stdout [format "       %s and %d\n" $prevGoupNo, $groupNo]
		exit 1
	    }
	}

	if {[info exists groupNames($groupNo)] == 1} {
	    set prevGoupName $groupNames($groupNo)

	    if {$prevGoupName == $groupName} {
		#set groupName ""
		#set groupNo ""
		continue
	    } else {
		puts stdout [format "\nERROR: conflicting group names defined for group %d" $groupNo ]
		puts stdout [format "       %s and %s\n" $prevGoupName, $groupName]
		exit 1
	    }
	}

	set defs ""
	set di 3
	while {[lindex $line $di] != ""} {
		set defs [format "%s %s" $defs [lindex $line $di]]
		incr di 1
	}
	if {$defs != ""} {
	    set groupDefs($groupName) $defs
	}

	lappend groupList $groupName
	set groupNames($groupNo) $groupName
	set groupNumber($groupName) $groupNo
	set groupRoutines($groupName) {}
	set groupNRoutines($groupName) 0

	# If this is a numeric group number check to see if it
	# is the largest group number we have seen
	if {[isnumber $groupNo] == 1} {
	    if {$groupNo > $maxGroup} {
		    set maxGroup $groupNo
	    }
	}

	continue
    }

    # Lines starting with a number define the system call number for a
    # given system call (within the current group) and the number of
    # arguments taken.

    if {[isnumber [lindex $line 0]]} {
	set rtnNumber [lindex $line 0]
	set rtnName [lindex $line 1]
	if {$groupName == ""} {
		puts [format "Routine %s has no group" $rtnName]
		continue;
	}

	if {[info exists groupNumber($groupName)] != 1} {
	    puts [format "Error %s: No corresponding SYSCALL_GROUP definition found." \
		 $groupName]
	    exit
	}
	if {[isnumber $rtnNumber] != 1} {
	    puts [format "Error %s: invalid system call number %s." \
		$rtnName $rtnNumber]
	    exit
	}
	if {[info exists ScNumber($rtnName)] == 1} {
	    puts [format "Error %s %s: duplicate system call." $groupName $rtnName]
	    exit
	}

	lappend scNameList $rtnName
	set ScGroup($rtnName) $groupName
	set ScNumber($rtnName) $rtnNumber
	if {$rtnNumber > $groupNRoutines($groupName)} {
		set groupNRoutines($groupName) $rtnNumber
	}
	lappend groupRoutines($groupName) $rtnName

    } else {

	# The first word on the line is not a number. Then it must be an 
	# argument list. Starting with element 1 we first look for the 
	# parameters to the system call. The argument list  will be enclosed 
	# within [ and ] and each argument terminates with a ';'. Following 
	# the parameters there is an optional list of key words, one of which 
	# must be defined in order for this system call to be built into the 
	# system.

	set di 2
	set rtnName  [lindex $line 0]
	set rtnNargs [lindex $line 1]
	set rtnFound 0

	# Check for duplicate API definition
	if {[lsearch $apiFuncList $rtnName] >= 0} {
	    puts stdout [format "duplicate function definition for %s (ignored)" $rtnName]
	    continue
	}

	# Insert the function into the apiFuncList
	lappend apiFuncList $rtnName

	# Check if this system call has a corresponding group,routine 
	# combination defined. If not, print a wraning and ignore it.

	foreach g $groupList {
	    set rtnList $groupRoutines($g)
	    if {[lsearch $rtnList $rtnName] != -1} {
		set rtnFound 1
	    }
	}

	if {$rtnFound == 0} {
	    puts stdout [format "scgen warning: function %s has no system call number defined (ignored)" $rtnName] 
	    continue
	}

	if {[lindex $line $di] == "\["} {
	    incr di 1
	    set arg ""
	    set rtnStruct [format "struct %sScArgs \{" $rtnName]
	    set rtnArgtList { }
	    while {[lindex $line $di] != "\]"} {
		set el [lindex $line $di]
		if {[string trimright $el ";"] != $el} {
		    append arg " "
		    append arg $el
		    append rtnStruct "\n\t"
		    append rtnStruct  $arg
		    lappend rtnArgtList $arg
		    set elements([format "%s->%s" $rtnName \
				      [elementName $arg]]) $arg
		    set arg ""
		} elseif {$arg == ""} {
		    set arg $el
		} else {
		    append arg " "
		    append arg $el
		}
		incr di 1
	    }
	    append rtnStruct "\n\};"
	    set ScArgs($rtnName) $rtnStruct
	    set ScArgList($rtnName) $rtnArgtList
	    incr di 1
	}

	set rtnDefs ""
	while {[lindex $line $di] != ""} {
		set rtnDefs [format "%s %s" $rtnDefs [lindex $line $di]]
		incr di 1
	}

	set ScNargs($rtnName) $rtnNargs
	set ScDefs($rtnName) $rtnDefs
    }
}

close $inFile
}


##############################################################################
#
# main - entry point of utility
#

# Check if the required environment variables are set

set doTab         0
set doHeader      0
set doStructs     0
set doStubs       0
set doSomething   0
set alignmentName ""
set scNameList    ""
set apiFuncList   ""

global elements

global abiSpec
global abiSpecs

array set abiSpecs { ppc pad8 }
array set abiSpecs { x86 none }
array set abiSpecs { arm none }
array set abiSpecs { sh2 none }
array set abiSpecs { sh3 none }
array set abiSpecs { sh4 SH4 }
array set abiSpecs { sparc pad8 }
array set abiSpecs { MIPS32 pad8 }
array set abiSpecs { MIPS64BE MIPS64BE }
array set abiSpecs { MIPS64LE MIPS64LE }
array set abiSpecs { cf CF  }


global abiName
global abiNames

array set abiNames { ppc ppc }
array set abiNames { x86 x86 }
array set abiNames { arm arm }
array set abiNames { sh2 sh2 }
array set abiNames { sh3 sh3 }
array set abiNames { sh4 sh4 }
array set abiNames { sparc sparc }
array set abiNames { MIPS32 MIPS32 }
array set abiNames { MIPS64BE MIPS64BE }
array set abiNames { MIPS64LE MIPS64LE }
array set abiNames { cf cf }

global abiHelp

array set abiHelp { pad8     "align 64-bit elements" }
array set abiHelp { CF       "ok to retain unaligned 64-bit elements for ColdFire" }
array set abiHelp { none     "no alignment needed" }
array set abiHelp { SH4      "align to SH4 conventions" }
array set abiHelp { MIPS64BE "MIPS64 Big-Endian" }
array set abiHelp { MIPS64LE "MIPS64 Little-Endian" }

global underscorePrefix
global underscoreStyle

array set underscoreStyle {ppc      ""}
array set underscoreStyle {x86      ""}
array set underscoreStyle {arm      ""}
array set underscoreStyle {sh2      "_"}
array set underscoreStyle {sh3      "_"}
array set underscoreStyle {sh4      "_"}
array set underscoreStyle {sparc    ""}
array set underscoreStyle {MIPS32   ""}
array set underscoreStyle {MIPS64BE ""}
array set underscoreStyle {MIPS64LE ""}
array set underscoreStyle {cf      ""}

# These flags are passed to dcc to select the correct target arch.
# We only need to select the approproate ABU for the target. FP or endian 
# selection is of no interest here.

global dccflag
global dccflags

array set dccflag {ppc    -tPPCFS:vxworks63}
array set dccflag {x86    -tX86LH:vxworks63}
array set dccflag {arm    -tARMLS:vxworks63}
array set dccflag {sh2    -tSH2ES:vxworks63}
array set dccflag {sh3    -tSH3ES:vxworks63}
array set dccflag {sh4    -tSH4EH:vxworks63}
array set dccflag {sparc  -tSPARCFS:vxworks63}
array set dccflag {MIPS32   "-tMIPS2FS:vxworks63 -DMIPSEB"}
array set dccflag {MIPS64BE "-tMIPS3XH:vxworks63 -DMIPSEB"}
array set dccflag {MIPS64LE "-tMIPS3ZH:vxworks63 -DMIPSEL"}
array set dccflag {cf     -tMCF5407FS:vxworks63}

global gccflag
global gccflags

array set gccflag {ppc      "ccppc"}
array set gccflag {x86      "ccpentium"}
array set gccflag {arm      "ccarm"}
array set gccflag {sh2      "ccsh -m2"}
array set gccflag {sh3      "ccsh -m3"}
array set gccflag {sh4      "ccsh -m4"}
array set gccflag {sparc    "ccsparc"}
array set gccflag {MIPS32   "ccmips -mips2"}
array set gccflag {MIPS64BE "ccmips -mips3 -EB"}
array set gccflag {MIPS64LE "ccmips -mips3 -EL"}

global maxGroup
global cpuDef

set archDir ""
set abiName ""
set cpuDef  ""

# Check if the Diab compiler exists in our path

set compilerIsDiab [catch {eval exec which dcc}]

# Parse the command line

for {set ac 0} {$ac < $argc} {incr ac} {
    set arg [lindex $argv $ac]
    if {[string index $arg 0] != "-"} {
	break;
    }
    for {set ai 1} {$ai < [string length $arg]} {incr ai} {
	set c [string index $arg $ai]
	if {$c == "t"} {
		set doTab 1
	} elseif {$c == "h"} {
		set doHeader 1
	} elseif {$c == "s"} {
		set doStubs 1
		incr ac
		if {$ac >= $argc} {
			usage
		}
		set archDir [lindex $argv $ac]
	} elseif {$c == "a"} {
	    set doStructs 1
	    set doHeader 1
	    set doStubs 1
	    set doTab 1
	    if {[expr $ac + 2] >= $argc} {
		usage
	    }
	    incr ac
	    set alignmentName [lindex $argv $ac]
	    if {[info exists abiSpecs($alignmentName)] == 0} {
		puts [format "%s: invalid alignment style" $alignmentName]
		exit 1
	    }
	    set abiSpec $abiSpecs($alignmentName)
	    if {[info exists abiNames($alignmentName)] == 0} {
		puts [format "%s: invalid ABI name" $alignmentName]
		exit 1
	    }
	    set abiName $abiNames($alignmentName)
	    if {[info exists underscoreStyle($alignmentName)] == 0} {
		puts [format "%s: no underscore prefix info found" $alignmentName]
		exit 1
	    }
	    set underscorePrefix $underscoreStyle($alignmentName)

	    # Check if we find a set of compiler options for the requested
	    # alignment type. Checking either dcc or gcc flags will check
	    # against an unsupported architecture. 
	    
	    if {[info exists dccflag($alignmentName)] == 0} {
		puts [format "%s: no compiler options info found" $alignmentName]
		exit 1
	    }

	    if {$compilerIsDiab == 0} {
	        set compilerInvocation [format "dcc %s" $dccflag($alignmentName)]
	    } else {
		set compilerInvocation $gccflag($alignmentName)
	    }

	    incr ac
	    set cpuDef [lindex $argv $ac]
	    incr ac
	    set archDir [lindex $argv $ac]
	} elseif {$c == "A"} {
	    set doStructs 1
	    incr ac
	    if {$ac >= $argc} {
		usage
	    }
	    set alignmentName [lindex $argv $ac]
	    if {[info exists abiSpecs($alignmentName)] == 0} {
		puts [format "%s: invalid alignment style" $alignmentName]
		exit 1
	    }
	    set abiSpec $abiSpecs($alignmentName)
	    if {[info exists abiNames($alignmentName)] == 0} {
		puts [format "%s: invalid ABI name" $alignmentName]
		exit 1
	    }
	    set abiName $abiNames($alignmentName)
	    if {[info exists underscoreStyle($alignmentName)] == 0} {
		puts [format "%s: no underscore prefix info found" $alignmentName]
		exit 1
	    }
	    set underscorePrefix $underscoreStyle($alignmentName)

	    # Check if we find a set of compiler options for the requested
	    # alignment type. Checking either dcc or gcc flags will check
	    # against an unsupported architecture. 
	    
	    if {[info exists dccflag($alignmentName)] == 0} {
		puts [format "%s: no compiler options info found" $alignmentName]
		exit 1
	    }

	    if {$compilerIsDiab == 0} {
	        set compilerInvocation [format "dcc %s" $dccflag($alignmentName)]
	    } else {
		set compilerInvocation $gccflag($alignmentName)
	    }

	    incr ac
	    set cpuDef [lindex $argv $ac]
	    incr ac
	    set archDir [lindex $argv $ac]
	} else {
		usage
	}
	set doSomething 1
    }
}

if {$doSomething == 0 || $ac != $argc} {
	usage
}

# <<DEBUG_MSG>>
# puts [format "archDir = %s" $archDir]
# puts [format "abiName = %s" $abiName]
# puts [format "CPU = %s" $cpuDef]
# <<end DEBUG_MSG>>

set windBase $env(WIND_BASE)
set windUsr  $env(WIND_USR)

set targetHDir     $windBase/target/h
set shareHDir      $windBase/target/share/h
set srcConfigDir   $windBase/target/src/config
set kernArchHdrDir $windBase/target/h/arch/$archDir
set wrnIncludes    $windBase/target/h/wrn/coreip

set inFilePath1    $shareHDir/syscallNum.def
set inFilePath2    $shareHDir/syscallApi.def
set inFilePath3    $shareHDir/syscallXPNum.def
set inFilePath4    $shareHDir/syscallXPApi.def
set inFilePath5    $shareHDir/syscallUsrNum.def
set inFilePath6    $shareHDir/syscallUsrApi.def
set headerFilePath $targetHDir/syscall.h
set tabFilePath    $targetHDir/syscallTbl.h
set usrHFilePath   $windUsr/h/syscall.h
set usrArchDir     $windUsr/src/arch/$archDir

# We want each architecture's temporary files to be placed in the respective
# architecture directory to shield against scgen being invoked several times
# at once from parallel builds. The best way to do this is using unique 
# process Id's in a temporary directory. Till such modifications are done
# we will use the respective architecture direcory.

set gencFileName [format "genc%s_%s.c" $cpuDef [pid]]
set gensFileName [format "gens%s_%s.s" $cpuDef [pid]]
set compilerErrFileName [format "compilerError%s" [pid]]
set gencFilePath   $targetHDir/arch/$archDir/$gencFileName
set gensFilePath   $targetHDir/arch/$archDir/$gensFileName
set compilerErrFilePath $targetHDir/arch/$archDir/$compilerErrFileName

set includes   {}
set defineList {}
set groupList  {}

set maxGroup   0

# syscallNum.def and syscallApi.def are always parsed. 

puts stdout [format "   parsing %s" $inFilePath1]
fileParse $inFilePath1
puts stdout [format "   parsing %s" $inFilePath2]
fileParse $inFilePath2

# Now we optionally parse the files containing experimental system calls
# in syscallXPNum.def and syscallXPApi.def and also the user-defined ones 
# in syscallUsrNum.def and syscallUsrApi.def.

if {[file exists $inFilePath3] == 1} {
    puts stdout [format "   parsing %s" $inFilePath3]
    fileParse $inFilePath3
} 

if {[file exists $inFilePath4] == 1} {
    puts stdout [format "   parsing %s" $inFilePath4]
    fileParse $inFilePath4
} 

if {[file exists $inFilePath5] == 1} {
    puts stdout [format "   parsing %s" $inFilePath5]
    fileParse $inFilePath5
} 

if {[file exists $inFilePath6] == 1} {
    puts stdout [format "   parsing %s" $inFilePath6]
    fileParse $inFilePath6
} 

foreach func $scNameList {
    set index [lsearch $apiFuncList $func]
    if {$index >= 0} {
	continue
    } else {
	# A function name present in scNameList that is missing from 
	# apiFuncList means there is no API defined for that function even
	# though a system call number for it exists. Such system calls are
	# ignored. Rerferences to this system call are removed from the 
	# arrays that already have it. 

	set index [lsearch $scNameList $func]
	set scNameList [lreplace $scNameList $index $index]
	unset ScGroup($func)
	unset ScNumber($func)

	if {[info exists ScNargs($func)] == 1} {
	    unset ScNargs($func)
	}

	# Delete $func from the group to which it was added.
	foreach g $groupList {
	    set groupRtnList $groupRoutines($g)
	    set index [lsearch $groupRtnList $func]
	    if {$index >= 0} {
		set groupRoutines($g) [lreplace $groupRtnList $index $index]
	    }
	}
    }
}

# <<DEBUG_MSG>>
# puts stdout "\n\ndump of array defineList"
# puts $defineList

# puts stdout "\n\ndump of array defineValue"
# parray defineValue

# puts stdout "\n\ndump of array groupDefs"
# parray groupDefs

# puts stdout "\n\ndump of array groupNames"
# parray groupNames

# puts stdout "\n\ndump of array groupNumber"
# parray groupNumber

# puts stdout "\n\ndump of array groupRoutines"
# parray groupRoutines

# puts stdout "\n\ndump of array groupNRoutines"
# parray groupNRoutines

# puts stdout "\n\ndump of array ScArgs"
# parray ScArgs

# puts stdout "\n\ndump of array ScArgList"
# parray ScArgList

# puts stdout "\n\ndump of array ScGroup"
# parray ScGroup

# puts stdout "\n\ndump of array ScNumber"
# parray ScNumber

# puts stdout "\n\ndump of array ScDefs"
# parray ScDefs

# puts stdout "\n\ndump of array ScNargs"
# parray ScNargs

# puts stdout "\n\ndump of array ScNames\n"
# parray ScNames

# puts "dump of list includes"
# puts $includes

# puts "dump of groupList"
# puts $groupList

# puts "dump of scNameList"
# puts $scNameList

# puts "dump of apiFuncList"
# puts $apiFuncList
# <<end DEBUG_MSG>>

if {$doHeader == 1} {
    #
    # Open the header file syscall.h
    #

    set tmpHeaderFile [format "%s_%s" $headerFilePath [pid]]
    set headerFile [open $tmpHeaderFile w+]

    syscallHOutputBegin $headerFile
    
    # Output the list of #define's.
    
    foreach x $defineList {
	puts $headerFile [format "\#define %s\t\t%s" $x $defineValue($x)]
    }
    
    syscallHOutputMiddle $headerFile

    # Output the list of group #define's.

    foreach x $groupList {
	puts $headerFile [format "\#define %s \t\t%s" $x $groupNumber($x)]
	puts $headerFile [format "\#define %s_Name \t\t\"%sGroup\"" $x \
	    [string range $x 4 end]]
    }

    puts $headerFile ""


    # Write out header file information for each routine

    foreach groupName $groupList {
	if {([llength $groupRoutines($groupName)] == 0)} {
	    continue;
	}

	puts $headerFile [format "/* Routines for %s */\n" $groupName]
	set tableName [format "%sRtnTbl" [string range $groupName 4 end]]
	
	foreach rtnName $groupRoutines($groupName) {
	    puts $headerFile [format "\#define SCN_%s \t\t SYSCALL_NUMBER (%s, %s)" \
		$rtnName $groupName $ScNumber($rtnName)]
	}

	puts $headerFile ""
	puts $headerFile [format "\#define %sNum (sizeof(%s)/sizeof(%s\[0\]))\n" \
	    $tableName $tableName $tableName]
    }
    syscallHOutputEnd $headerFile
    flush $headerFile 
    close $headerFile 
    file rename -force $tmpHeaderFile $headerFilePath
    file copy -force $headerFilePath $usrHFilePath
}

#
# Write out argument structures for each routine
#

if {$doStructs == 1} {
    # Debug Instrumentation for intermittent failures
    set nLoops 0

    # Output the genc.c file to determine the structure alignment. 
    # genc.c is created only for architectures with alignment constraints.
    # If the architecture has no alignment constraints (i.e. when
    # abiSpec is "none", we output directly to genc.s and do 
    # not call the compiler. This can be done without creating a temp 
    # file, but it is easier to make the parsing code think we called 
    # the compiler.

    if {$abiSpec != "none"} {
	set gencFile [open $gencFilePath w+]
	foreach include $includes {
	    puts $gencFile [format "#include %s" $include]
	}
	puts $gencFile ""
    } else {
	set gencFile [open $gensFilePath w+]
    }

    foreach groupName $groupList {
	# Debug Instrumentation for intermittent failures
	incr nLoops

	foreach rtnName $groupRoutines($groupName) {
	    set rtnNo   $ScNumber($rtnName)


	    if {[info exists ScArgs($rtnName)] == 1} {
		if {$abiSpec != "none"} {
		    puts $gencFile $ScArgs($rtnName)
		}
		set o 0
		foreach arg $ScArgList($rtnName) {
		    set e [elementName $arg]
		    set v [format "_0_%s_0_%s_0_%s_0_" $groupName $rtnName $e]
		    if {$abiSpec == "none"} {
			puts $gencFile [format "%selement%s:\n\t.long 4\n\t.long %s" \
			    $underscorePrefix $v $o]
			incr o 4
		    } else {
			set e [format "(((struct %sScArgs *)0)->%s)" $rtnName $e]
			puts $gencFile [format "int element%s\[2\] = \{ sizeof%s," \
			    $v $e]
			puts $gencFile [format "((char *)&%s - (char *)0) \};" $e]
		   }
		}
		puts $gencFile ""
	    } 
	}
    }
    flush $gencFile 
    close $gencFile 

    # Debug Instrumentation for intermittent failures
    if {$nLoops == 0} {
	puts stdout "SCGEN ERROR: nLoops == 0 after creating gencFile??!!!"
	puts stdout "--->dump of groupList follows"
	puts stdout $groupList
	puts stdout "--->dump of array groupRoutines follows"
	parray groupRoutines
    }

    # Compile the genc file to a .s file and parse the output and produce
    # the structures

    if {$abiSpec != "none"} {
	set compilerErrFile [open $compilerErrFilePath w+]
	set compileCommand [format "$compilerInvocation -I $targetHDir -I $wrnIncludes -DCPU=$cpuDef -D_WRS_KERNEL -S $gencFilePath -o $gensFilePath"]
# <<DEBUG_MSG>>
#	puts "COMPILING: dcc $dccflags -I $targetHDir -I $wrnIncludes -DCPU=$cpuDef -D_WRS_KERNEL -S $gencFilePath -o $gensFilePath -@E=$compilerErrFilePath"
	puts stdout [format "COMPILING: $compileCommand"]
# <<end DEBUG_MSG>>

	# If compilation resulted in warnings and/or errors, there is a
	# way to distinguish between the two outcomes - check exit status 

    	set compileResult [catch { eval exec $compilerInvocation {-I $targetHDir -I $wrnIncludes -DCPU=$cpuDef -D_WRS_KERNEL -S $gencFilePath -o $gensFilePath} 2>$compilerErrFilePath} result]

	close $compilerErrFile

	# Did the compiler return an error? If so, abort immediately.
	if {$compileResult != 0} {
	    puts stderr "\n!!!!! COMPILATION RESULTED IN AN ERROR !!!!! BAILING OUT...\n"
	    puts $result
	    set compilerErrFile [open $compilerErrFilePath r]
	    set fileContents [read $compilerErrFile]
	    puts stdout $fileContents
	    close $compilerErrFile
	    file delete $compilerErrFilePath
	    file delete $gencFilePath
	    exit -1
	}
    }

    set gensFile [open $gensFilePath r]

    set argsFileName [format "%s/syscallArgs%s.h" $kernArchHdrDir $abiName]
    set tmpArgsFileName [format "%s_%s" $argsFileName [pid]]
    set argsFile [open $tmpArgsFileName w+]

    puts $argsFile [format "/* Aligned for %s - %s */\n" \
			$alignmentName $abiHelp($abiSpec)]
    puts $argsFile "\n/* THIS FILE IS AUTO_GENERATED. PLEASE DO NOT EDIT. */\n"
    puts $argsFile "#ifndef _ASMLANGUAGE\n"

    foreach include $includes {
	puts $argsFile [format "#include %s" $include]
    }
    puts $argsFile ""

    # Debug Instrumentation for intermittent failures
    set nLoops 0

    while {[gets $gensFile line] >= 0} {
	# Debug Instrumentation for intermittent failures
	incr nLoops

	# Each structure element is of the form:
	#
	# element_0_<group name>_0_<routine name>_0_<element name>_0_:
        #	.long   <element size>
        #	.long   <element offset>
	#
	# Look for element_ at the start of line, if found, split on _0_
	# to get a list which has the structure and element names
	# read the next two lines to get the size and offset of the element

	if {[string match *element*_0_: $line] == 1} {
	    regsub -all -- "_0_" $line " " kw
	    set n [split $kw]
	    set groupName [lindex $n 1]
	    set eName [lindex $n 3]
	    set eSize [lindex [string trim [gets $gensFile]] 1]
	    set eOffset [lindex [string trim [gets $gensFile]] 1]

	    if {$eOffset == 0} {
		if {[array exists eSizes] == 1} {

		    # See comment below about the defines

		    if {[info exists groupDefs($groupName)] == 1} {
			set gd $groupDefs($groupName)
		    } else {
			set gd {}
		    }
		    if {[info exists ScDefs($eStruct)] == 1} {
			set sd $ScDefs($eStruct)
		    } else {
			set sd {}
		    }

		    # Emit the argument structures without any conditional 
		    # defines to scale them in/out.

		    emitStruct $argsFile $groupName $eStruct $eStruct \
		    		$eNames eOffsets eSizes {} {}

		    # Free up our old arrays so we don't reuse the information

		    unset eOffsets
		    unset eSizes
		} 
		set eStruct [lindex $n 2]
		set groupName [lindex $n 2]
		set eNames { }
	    }
	    lappend eNames $eName
	    set eOffsets($eName) $eOffset
	    set eSizes($eName) $eSize
	}
    }
    close $gensFile

    file delete $gensFilePath
    file delete $gencFilePath

    # Debug Instrumentation for intermittent failures
    if {$nLoops == 0} {
	puts stdout "SCGEN ERROR: nLoops == 0 after parsing genSFile??!!!"
	puts stdout "--->dump of groupList follows"
	puts stdout $groupList
	puts stdout "--->dump of array groupRoutines follows"
	parray groupRoutines
    }

    # If we have an argument structure then put it into syscallargs.h. 
    # We can have 2 #if's.  One for over all group inclusion and one for 
    # individual function inclusion

    if {[array exists eSizes] == 1} {
	if {[info exists groupDefs($groupName)] == 1} {
	    set gd $groupDefs($groupName)
	} else {
	    set gd {}
	}
	if {[info exists ScDefs($eStruct)] == 1} {
	    set sd $ScDefs($eStruct)
	} else {
	    set sd {}
	}

	# Structure definitions are emitted without any #defines to 
	# conditionalize inclusion of certain system calls. This is because
	# system calls are scalable only at system build time, not at
	# library build time. 
	#emitStruct $argsFile $groupName $eStruct $eNames eOffsets eSizes $gd $sd

	emitStruct $argsFile $groupName $eStruct $eStruct $eNames eOffsets eSizes {} {}
    } else {
	# Debug Instrumentation for intermittent failures
	puts stdout "warning: array eSizes does not exist when generating structs??!!!"
    }

    puts $argsFile "#endif	/* _ASMLANGUAGE */"
    flush $argsFile
    close $argsFile 
    file rename -force $tmpArgsFileName $argsFileName

    if {$abiName == "sparc"} {
	# On sparc, there is padding in the system call argument structures
	# generated but not in the system call parameters. Because of this,
	# the method used for system call does not work with this architecture.
	# As a workaround we generate an array that contains all the paddings
	# inserted for a specific system call; this will be used by the
	# runtime to fix the structure.

	set argsPaddingFileName [format "%s/syscallArgsPadding%s.h" \
						$kernArchHdrDir $abiName]
	set tmpArgsPaddingFileName [format "%s_%s" $argsPaddingFileName [pid]]
	set argsPadFile [open $tmpArgsPaddingFileName w+]

	puts $argsPadFile [format "/* System call padding for %s - %s */\n" \
				    $alignmentName $abiHelp($abiSpec)]
	puts $argsPadFile \
		    "\n/* THIS FILE IS AUTO_GENERATED. PLEASE DO NOT EDIT. */\n"
	puts $argsPadFile "#ifndef _ASMLANGUAGE\n"

	puts $argsPadFile "#include <syscall.h>\n"

        puts $argsPadFile "typedef struct syscall_args_padding_tbl \{"
	puts $argsPadFile "\tint\tscNum;"
	puts $argsPadFile "\tint\tpaddingEnum;"
	puts $argsPadFile "\} SYSCALL_ARGS_PADDING_TBL;"
	puts $argsPadFile ""
	puts $argsPadFile \
		"SYSCALL_ARGS_PADDING_TBL syscallArgsPaddingTbl\[\] = \{"
    	foreach elem $paddingEnumList {
	    puts $argsPadFile "\t\{ $elem \},"
	}
	puts $argsPadFile "\};"
	puts $argsPadFile ""
	puts $argsPadFile "#endif	/* _ASMLANGUAGE */"
	flush $argsPadFile
	close $argsPadFile 
	file rename -force $tmpArgsPaddingFileName $argsPaddingFileName
    }
}

#
# Write out stubs for each routine
#

if {$doStubs == 1} {
    # Debug Instrumentation for intermittent failures
    set nLoops 0

    foreach groupName $groupList {
	# Debug Instrumentation for intermittent failures
	incr nLoops

	set tableName [format "%sRtnTbl" [string range $groupName 4 end]]
	
	foreach rtnName $groupRoutines($groupName) {
	    set rtnNo   $ScNumber($rtnName)
	    set rtnArgs $ScNargs($rtnName)

	    set asmFileName [format "SYSCALL_STUB_%s.s" $rtnName]
	    set asmFilePath $usrArchDir/$asmFileName
	    set tmpAsmFile [format "%s_%s" $asmFilePath [pid]]
	    set asmFile [open $tmpAsmFile w+]
	    puts $asmFile "\n/* THIS FILE IS AUTO_GENERATED. PLEASE DO NOT EDIT. */\n"
	    puts $asmFile [format "\#define _SYSCALL_FUNC_NAME %s" $rtnName]
	    puts $asmFile [format "\#define _SYSCALL_NUMBER SCN_%s" $rtnName]
	    puts $asmFile ""
	    puts $asmFile "\#define _ASMLANGUAGE"
	    puts $asmFile "\#include \"vxWorks.h\""
	    puts $asmFile ""
	    puts $asmFile "\#include \"syscallStub.s\""
	    flush $asmFile
	    close $asmFile
	    file rename -force $tmpAsmFile $asmFilePath
	}
    }
    # Debug Instrumentation for intermittent failures
    if {$nLoops == 0} {
	puts stdout "SCGEN ERROR: nLoops == 0 after stub generation loop??!!!"
	puts stdout "--->dump of groupList follows"
	puts stdout $groupList
	puts stdout "--->dump of array groupRoutines follows"
	parray groupRoutines
    }

}

if {$doTab == 1} {
    set tmpTabFile [format "%s_%s" $tabFilePath [pid]]
    set tabFile [open $tmpTabFile w+]

    # Debug Instrumentation for intermittent failures
    set nLoops 0

    puts $tabFile "\n/* THIS FILE IS AUTO_GENERATED. PLEASE DO NOT EDIT. */\n"

    # Print function declarations

    foreach groupName $groupList {
	foreach rtnName $groupRoutines($groupName) {
	    puts $tabFile [format "extern int %sSc(void *);" $rtnName]
	}
    }
    puts $tabFile ""

    # Print group tables

    foreach groupName $groupList {
	incr nLoops
	if {([llength $groupRoutines($groupName)] == 0)} {
	    # Debug Instrumentation for intermittent failures
	    #puts stdout [format "warning: no routines in group %s" $groupName]

	    continue;
	}
	set groupRtnList $groupRoutines($groupName)
	set tableName [format "%sRtnTbl" [string range $groupName 4 end]]
	if {[info exists groupDefs($groupName)] == 1} {
	    putDefines $tabFile $groupDefs($groupName)
	}
	set groupDefined($groupNumber($groupName)) True
	puts $tabFile [format "_WRS_DATA_ALIGN_BYTES(16) SYSCALL_RTN_TBL_ENTRY %s \[\] = \{" $tableName]

	set listIndex 0
	for {set i 0} {$i <= $groupNRoutines($groupName)} {incr i} {
	    set rtnName [lindex $groupRtnList $listIndex]
	    if {[info exists ScNumber($rtnName)]} {
		set rtnNo $ScNumber($rtnName)
		set rtnNArgs $ScNargs($rtnName)

	    }

	    if {$i == $rtnNo} {
		incr listIndex
		if {[info exists ScDefs($rtnName)] == 1} {
		    set reqDef  $ScDefs($rtnName)
		    putDefines $tabFile $ScDefs($rtnName)
		} else {
		    set reqDef ""
		}
		puts $tabFile [format "/*%s*/\t\{ %sSc, %s, \"%s\", 0\}," \
				   $i $rtnName $rtnNArgs $rtnName]

		if {$reqDef != "" } {
		    puts $tabFile "\#else\n\t\{ NULL, 0, NULL, 0\},\n\#endif"
		} 
	    } else {
		    puts $tabFile [format "/*%s*/\t\{ NULL, \}," $i]
	    }
	}

	puts $tabFile "\};"
	puts $tabFile [format "\#define %sNum (sizeof(%s)/sizeof(%s\[0\]))\n" \
	    $tableName $tableName $tableName]
	if {[info exists groupDefs($groupName)] == 1} {
	    puts $tabFile "#endif\n"
	}
    }
    # Debug Instrumentation for intermittent failures
    if {$nLoops == 0} {
	puts stdout "SCGEN ERROR: nLoops == 0 after RtnTbl loop??!!!"
	puts stdout "--->dump of groupList follows"
	puts stdout $groupList
	puts stdout "--->dump of array groupRoutines follows"
	parray groupRoutines
    }

    # Print master group table

    puts $tabFile "SYSCALL_GROUP_ENTRY syscallGroupTbl \[SYSCALL_GROUPS_MAX\] = \{"
    for {set i 0} {$i <= $maxGroup} {incr i} {
	if {[info exists groupDefined($i)]} {
	    set n [string range $groupNames($i) 4 end]
	    if {[info exists groupDefs($groupNames($i))] == 1} {
		putDefines $tabFile $groupDefs($groupNames($i))
	    }
	    puts $tabFile [format "\t\{ %sRtnTbl, %sRtnTblNum, \}," $n $n]
	    if {[info exists groupDefs($groupNames($i))] == 1} {
		puts $tabFile "\#else\n\t\{ NULL, 0\},\n\#endif"
	    }
	} else {
	    puts $tabFile "\t\{ NULL, 0\},"
	}
    }
    puts $tabFile "\};\n"

    # Print group names table

    puts $tabFile "char * syscallGroupName \[SYSCALL_GROUPS_MAX\] = \{"
    for {set i 0} {$i <= $maxGroup} {incr i} {
	if {[info exists groupDefined($i)]} {
	    set n [string range $groupNames($i) 4 end]
	    if {[info exists groupDefs($groupNames($i))] == 1} {
		putDefines $tabFile $groupDefs($groupNames($i))
	    }
	    puts $tabFile [format "\t\"%sGroup\"," $n $n $n]
	    if {[info exists groupDefs($groupNames($i))] == 1} {
		puts $tabFile "\#else\n\t\NULL,\n\#endif"
	    }
	} else {
	    puts $tabFile "\t NULL,"
	}
    }
    puts $tabFile "\};\n"
    flush $tabFile
    close $tabFile 
    file rename -force $tmpTabFile $tabFilePath
}
