# FileLib.tcl - file manipulation library

# Copyright 2003-2004 Wind River Systems, Inc.

# modification history
# --------------------
# 01e,09jun04,fle  Added FileLib::copy procedure
# 01d,30mar04,fle  Added find procedure
#                  + fixed FileLib::listGet recursive behaviour
# 01c,23feb04,fle  Made it provide PrjUtils package
# 01b,16sep03,fle  Made listGet also list directories in the list of returned
#                  elements
# 01a,18aug03,fle  written

# DESCRIPTION
# This Tcl library provides with some file manipulation facilities which are not
# part of any other Tcl package, such as the 'FileLib::listGet' procedure.

package provide PrjUtils 1.0

namespace eval FileLib {

    namespace export find listGet listDelete
}

################################################################################
#
# FileLib::listGet - get the list of files for directory <dir>
#
# SYNOPSIS
# \ss
#   FileLib::listGet [dir] [recurse]
# \se
#
# DESCRIPTION
# This procedure gets the list of all regular files in directory <dir>.
#
# If <dir> is ommited, then "." (local directory) is considered as the directory
# to get file list from.
#
# If <recurse> is specified, it should either be '1' or '0'. Default is '0',
# and means that the procedure does not go recursively in the sub directories
# to get the list of regular files there. If it is set to '1', then the files
# from sub directories are also returned in the list of regulat files.
#
# PARAMETERS:
#   dir : the directory to get list of files from (default is ".")
#   recurse : states if the procedure should also get the sub directories files
#             list or not. Its value should either be '1' or '0'.
#
# RETURNS: The list of files within <dir> sorted by file name.
#

proc FileLib::listGet {{dir "."} {recurse 0}} {

    set fileList {}

    # check if <dir> exists and is a directory, else, return an empty list

    if {! [file isdirectory $dir]} {
	return $fileList
    }

    # first go to the directory we should list files from

    set oldPath [pwd]
    cd $dir

    # get the list of all files in <dir> (which is current dir now)

    set tmpList [glob -nocomplain *]
    set tmpList [concat $tmpList [glob -nocomplain .*]]

    # remove the "." and ".." directories, they are useless

    foreach fileName $tmpList {

	if {($fileName != ".") && ($fileName != "..")} {

	    if {$recurse == 1} {

		if {[file isdirectory [file join [pwd] $fileName]]} {

		    # also append the current directory to the file list

		    lappend fileList $fileName

		    # go in that directory and get the file list too

		    set subDirFileList [listGet $fileName $recurse]

		    foreach subDirFileName $subDirFileList {
			lappend fileList [ \
			    file nativename [file join $fileName \
						       $subDirFileName] \
			]
		    }

		} else {
		    lappend fileList [file nativename $fileName]
		}
	    } else {
		lappend fileList [file nativename $fileName]
	    }
	}
    }

    # go back to the directory we started in

    cd $oldPath

    return $fileList
}

################################################################################
#
# FileLib::listDelete - delete files from <dir> except files from <lst>
#
# SYNOPSIS
# \ss
#   FileLib::listDelete <dir> [<lst>]
# \se
#
# DESCRIPTION
# This procedure deletes all the files from directory <dir>, except the files
# listed in the <lst> file list.
#
# It first gets all files from <dir>, calling for the 'FileLib::listGet'
# procedure, then deletes all files from the so-retrieved list, except the files
# found in the <lst> list of files.
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: FileLib::listGet
#

proc FileLib::listDelete {dir {lst {}}} {

    set newFileList [listGet $dir 1]

    foreach fileName $newFileList {

	if {[lsearch $lst $fileName] == -1} {

	    # this file is not in the list of exceptions, it may be removed

	    catch {file delete -force [file join $dir $fileName]}
	}
    }
}

################################################################################
#
# FileLib::find - find specified <files> in given <dirs>
#
# SYNOPSIS
# \ss
#   FileLib::find <dir> <files> [<type>]
# \se
#
# DESCRIPTION
# This procedure recursively looks through <dirs> directories for elements of
# name <files>.
#
# If a <type> is specified, the found element must also be of type <type>,
# where <type>'s possible values are the same as returned by 'file type ...'
# Tcl procedure
#
# Foreach element of <files>, if the element is found, it is added to the
# returned list with its location (<dir>/<files>). If it is not found, it is
# returned as is in the returned list.
#
# If a <type> is specified, but the element was not found in <dir>, the
# element is removed from the returned list.
#
# PARAMETERS:
#   dirs : the directories to look for <files> in
#   files : the list of files to be found in <dirs>
#   type : the type <file> should match (default=any)
#
# RETURNS: <files> with their full path if found
#
# ERRORS: N/A
#
# EXAMPLES:
# Look for files in directories dir1 and dir2, where file1 and 3 exist :
# \cs
# % FileLib::find {dir1 dir2} {file1 file2 file3}
# dir1/dir11/file1 file2 dir2/file3
# %
# \ce
#
# Look for <myDir> in dir1, where <myDir> should be a 'directory', but does
# not exist :
# \cs
# % FileLib::find dir1 myDir directory
# %
# \ce
#
# SEE ALSO: FileLib::listGet
#

proc FileLib::find {dirs files {type ""}} {

    set found		{}

    foreach dir $dirs {

	# get the list of files for that directory

	set fileList	[listGet $dir 1]

	foreach fileName $files {

	    # look for the given files in the file list

	    set fileNotFound	1

	    foreach filePath $fileList {

		if {[file tail $filePath] == $fileName} {

		    # check for the file type too

		    if {$type != ""} {
			if {[file type [file join $dir $filePath]] == $type} {
			    lappend found [file join $dir $filePath]
			}
		    } else {
			lappend found [file join $dir $filePath]
			set fileNotFound 0
		    }
		}
	    }

	    if {($fileNotFound) && ($type == "")} {
		lappend found $fileName
	    }
	}
    }

    return $found
}

################################################################################
#
# FileLib::copy - copy a file to another location
#
# SYNOPSIS
# \ss
#   FileLib::copy <src> <dest> [<force>]
# \se
#
# DESCRIPTION
# As the Tcl builtin 'file copy' procedure does not work so well on Linux hosts
# (problem when copying from a file system to another), this procedure is a
# workaround.
#
# It opens <src> in binary mode, and copies its content to <dest> which was
# opened in binary mode too.
#
# If <force> is set to '1' or 'true', and that the <dest> file already exists,
# <dest> is then first deleted.
#
# NOTE: The file pointed to by <src> must be a regular file (aka, 'file isfile'
# must return '1').
#
# RETURNS: N/A
#
# ERRORS: N/A
#

proc FileLib::copy {src dest {force 0}} {

    if {(! [file isfile $src])} {
	return
    }

    # open the input file in binary mode

    set fileId	[open $src r]
    fconfigure $fileId -encoding binary
    set data [read $fileId]
    close $fileId

    # if the force option has been specified, and the destination file exists,
    # remove it first

    if {($force == 1) || ($force == "true")} {
	if {[file exists $dest]} {
	    file delete -force $dest
	}
    }

    # write content to destination

    set fileId	[open $dest w]
    fconfigure $fileId -encoding binary
    puts $fileId $data
    close $fileId
}
