# bsp2prj.tcl - tcl script to create a project from a BSP build.
# 
# Copyright 1984-2004, Wind River Systems, Inc.
#
# modification history
# --------------------
# 01p,07sep05,pee  Added default profile for scalable BSPs
# 01o,01sep04,fle  Made it use Wind::tgtDir where needed
# 01n,18may04,c_c  Don't break on rom build failure (SPR 93379).
# 01m,23feb04,fle  Made it require OsConfig package
# 01l,30jun03,fle  adapted to new os config tree
# 01k,03jun02,rbl  new fix for bsps outside Tornado tree
# 01j,31may02,cjs  Rollback/overwrite version 5; breaks nightly builds'
#                  project generation
# 01i,23apr02,rbl  fix to cope with bsp directories outside the tornado tree
# 01h,27mar02,dat  Adding new 'clean' option for SPR 72823
# 01g,16jan02,dat  removed build_spec creations, they are done elsewhere now
# 01f,05nov01,dat  Adding additional Builds to default project
# 01e,01oct01,dat  Adding optional CPU TOOL arguments
# 01d,22jan01,sn   allow base directory for project creation to be overridden
#                  by setting WIND_PROJ_BASE 
# 01c,24sep98,ms   undid 01b - workspace procs no longer callable without GUI.
# 01b,07sep98,ms   create a workspace document as well as a project document
# 01a,28aug98,ms   written
#
# DESCRIPTION
# This script is used to generate a project from a BSP.  Optionally, the CPU
# and TOOL make values can be changed.  After the project is created, at least
# one of the defined build specs will be built.  Optionally, all 4 possible
# specs will be built.
#
# The default build specs and their default target are:
# .CS
#     BUILD_SPEC             RULE
#     default                vxWorks
#     default_rom            vxWorks_rom
#     default_romCompressed  vxWorks_romCompressed
#     default_romResident    vxWorks_romResident
# .CE
#
# This script uses the cmpScriptLib.tcl script as its main interface for 
# project creation and manipulation.  The actual project creation code
# though is in the prj_vxWorks.tcl script.  This script is normally invoked
# by the make rules in target/h/make/rules.bsp.
#
# The base project directory can be set by defining the environment
# variable WIND_PROJ_BASE. This feature is used by the host validation
# test. If WIND_PROJ_BASE is not set the base directory defaults to
# $WIND_BASE/target/proj.
#
# If 1 argument is present, it is $CPU (TOOL will default to 'gnu')
# If 2 arguments are present, it is $CPU and $TOOL
# Otherwise, take CPU and TOOL values from bsp/Makefile
#
# If optional 3rd argument is 'all', then 4 build specs are built. Projects
# are always created with all 4 build specs now, this option will do a build
# for all of them.  Without them, only the default (vxWorks) build_spec will
# be built.
#
# If optional 3rd argument is 'clean' then the script will clean up temp and
# intermediate files from all project build specs.

package require OsConfig
package require Wind

set CPU ""
set TOOL "gnu"
set ALL_BUILDS ""

if {$argc >= 1} {
    set CPU [lindex $argv 0]
}
if {$argc >= 2} {
    set TOOL [lindex $argv 1]
} 
if {$argc >= 3} {
    set ALL_BUILDS [lindex $argv 2]
}
if {$argc >= 4} {
    puts "argc $argc"
    exit 1
}

set bsp [pwd]

# Cope with oddities of pwd when using Clearcase, by replacing
# e.g. /vobs/.wpwr_vxw_drv/target/config with /vobs/wpwr/target/config

set targetConfigFromWtxPath [file join [Wind::tgtDir] config]
cd $targetConfigFromWtxPath
set targetConfigFromPwd [pwd]
cd $bsp
regsub $targetConfigFromPwd $bsp $targetConfigFromWtxPath bsp

# determine the base project directory and ensure it exists

set projBase [file join [Wind::tgtDir] proj]

if {[info exists env(WIND_PROJ_BASE)]} {
    set projBase $env(WIND_PROJ_BASE)
}

# create the main project directory if does not exist

file mkdir $projBase

# setup some directory and file names

set projDir	[file join $projBase [format "%s_%s" [file tail $bsp] $TOOL]]
set projFile	[file join $projDir [format "%s_%s.wpj" [file tail $bsp] $TOOL]]

# procedure to clean a single build spec

proc buildSpecClean {dir} {
    if {[file exists $dir]} {
	foreach temp [glob -nocomplain [file join $dir *.o]] {
	    file delete -force $temp
	}
	file delete -force [file join $dir ctdt.c]
	file delete -force [file join $dir symTbl.c]
    }
}

# For clean operation, clean all 4 of the default build specs.

if { $ALL_BUILDS == "clean" } {
	# perform a clean operation for all project build specs
	buildSpecClean [file join $projDir default]
	buildSpecClean [file join $projDir default_rom]
	buildSpecClean [file join $projDir default_romCompress]
	buildSpecClean [file join $projDir default_romResident]
	return 0
}

# create the project

file delete -force $projDir
cmpProjCreate $bsp $projFile vxWorks $CPU $TOOL
set hProj  [cmpProjHandleGet]
set wcc [wccCurrentGet $hProj]
set default [cmpDefaultProfileGet $hProj]
if {$default != ""} {
	set dep [cmpAddDepends $hProj $default]
	set comps [concat $default $dep]
	wccComponentAdd $hProj $wcc $comps
}

# Close/reopen the project to save the .wpj file to disk
# This creates a fence between project creation and project build failures

cmpBuildSpecSet default
cmpProjClose
cmpProjOpen $projFile

if {$ALL_BUILDS == "all"} {

    # Build all 4 build specs

    # build the default build spec first

    if {[catch {cmpBuild ""} error]} {
    	puts stdout "\n Error building default spec\n"
    }

    # build the other build_specs

    cmpBuildSpecSet default_rom
    if {[catch {cmpBuild} error]} {
    	puts stdout "\n Error building default_rom spec\n"
    }

    cmpBuildSpecSet default_romCompress
    if {[catch {cmpBuild} error]} {
    	puts stdout "\n Error building default_romCompress spec\n"
    }

    cmpBuildSpecSet default_romResident
}
# final build, either default, or default_romResident
if {[catch {cmpBuild} error]} {
    set spec ""
    if {$ALL_BUILDS == "all"} {
	set spec "_romResident"
    }
    puts stdout [format "\n Error building default%s spec\n" $spec]
}

# Make sure the default build spec is 'default' before finishing

cmpBuildSpecSet default

cmpProjClose
