# prjBuildLib.tcl - project build manager library
#
# Copyright 1998-2004 Wind River Systems, Inc.
#
# modification history
# --------------------
# 02s,20Jun05,pee  Build command for project file use $(CFLAGS) instead of expanding it.
# 02r,23dec04,c_c  removed references to foundation.
# 02q,22oct04,fle  Do not exit on error on component validation problem
# 02p,21oct04,fle  SPR#102787 : do not build when component configuration is
#                  not valid
# 02o,15oct04,fle  Fixed prjBuildMacroListGet for vxApp
# 02n,01sep04,fle  Made it use Wind::foundation when needed
# 02m,30aug04,fle  Made prjBuild redirect output to a the prj standard output
#                  channel
# 02l,18may04,fle  Fixed prjBuildMacroListGet when there is no current wcc
# 02k,13may04,fle  Added prjBuildIncludePath
# 02j,06may04,fle  Removed all bullet lists refgen markups
# 02i,23feb04,fle  Made it use PrjUtils::relPath instead of prjRelPath
# 02h,18nov03,rp   added include for h/wrn/coreip
# 02g,26sep03,fle  SPR#90639 : added prjBuildIncludeFix procedure
# 02f,16sep03,fle  Improved doc
# 02e,01sep03,fle  SPR#85631 : added prjBuildMacrosExpand procedure
# 02d,09jul03,fle  Some docs enhancements
# 02c,19jun03,fle  Updated to new location
# 02b,29may02,rbl  fix problem where prjBuildCopy was not dealing properly
#                  with _'s in build names
# 02a,13may02,rbl  fix problem where vxWorks_rom.bin rule was appearing twice
#                  in project Makefile if this rule was defined in the bsp
#                  Makefile
# 01z,23mar02,cjs  Report build errors during autoscale (spr 73344)
# 01y,09jan02,rbl  support .hex and .bin targets from gui
# 01x,07dec01,rbl  fix error redirection - merge from T3
# 01w,24nov01,dat  individual build failure does not cause tcl error
# 01v,13nov01,rbl  add .pl partial link target to downloadable projects
# 01u,27sep01,rbl  in toolchains, rename "vendor" to "tool" for consistency
#                  with the makefile TOOL variable
# 01t,13jun01,agf  add logic to check host type on stdout/stderr syntax
# 01s,09may01,agf  undoing last change, @ breaks Win hosts
# 01r,16apr01,agf  fix syntax for redirecting output to stdout/stderr
# 01q,16apr99,j_k  back-off from previous fix and make it right.
# 01p,13apr99,j_k  part of fix for SPR #26620.
# 01o,10mar99,ren  Fixing SPR 25424 by naming the link output file to be the
#                  project name.
# 01n,11feb99,cjs  Changed buildRuleListGet, buildRuleDefault to operate on a
#                  project handle
# 01m,19jan99,cjs  make prjBuildRuleListGet() sensitive to bsp
# 01l,10nov98,ms   fixed bug where all buildSpec's used the same build rule.
# 01k,26oct98,ms   new build interface
# 01j,06aug98,ms   prjBuild now returns to build output
# 01j,29jul98,ms   updated calls to prj[Rel|Full]Path. Change default roPath.
# 01i,29jun98,jmp  added prjBuildRuleDescGet{}.
#		   added prjDepPath[Add|Remove|Get|Default].
# 01h,24jun98,ms   fixed prjBuildCopy().
# 01g,18jun98,jmp  added support for user defined rules.
# 01f,08jun98,cjs  fixed minor bug in prjBuildDelete
# 01e,05jun98,jmp  added prjBuildMacroCheck{}, prjBuildMacroDescGet{}. added
#                  control in prjBuildMacroSet{}.
# 01d,05jun98,jmp  called prjTagRemove instead of prjTagDataRemove in
#                  prjBuildMacroRemove.
# 01c,02jun98,ms   added prjBuildCopy, prjBuildRule* routines.
#		   removed prjBuildInfo*.
# 01b,19may98,ms   added prjBuildMacro* and prjBuildInfo* routines.
# 01a,18mar98,ms   written.

# DESCRIPTION
# This is the project build management library.
# This library provides an API for creating and managing "build specs."
# A build spec contains:
#	a toolchain selection
#	a set of flags for each tool in the toolchain
#	a build rule (e.g., vxWorks_rom)
#	a set of values for configurable aspects of the build (e.g.,
#		RAM_HIGH_ADRS).
#
# This library also provides an API for adding build rules to
# a project. The available build rules are those provided by the
# project manager, plus any custom rules added by the user.
#
# It also provides an API for toolChains to register themselves as follows;
# When this library starts up, it sources all files of the form tc_*.tcl.
# These files create a toolChain namespace and register it by calling
# prjTcAdd.
# Each toolChain (e.g., MC68040gnu) has a type. Currently the only
# type used is "C++", but new types will be added in the future.
# Each toolChain is expected to provide the following functions is
# a namespace:
#        type                           // type of toolchain. e.g., "C++"
#        cpu                       	// e.g. MC68040
#        tool                         // e.g., gnu
#        toolListGet                    // return a list of tool namespaces
#        toolFindbyFile fileName	// returns a tool namespace
#        toolFindbyType toolType	// returns a tool namespace
#        dependFiles    tool flags file // return a list of files that are
#                                   	// make-dependancies for the file
#	 macroListGet			// list of make-macros associated with
#					// the toolchain
#	 macroDefaultValGet macro	// default value for a macro
#
# Each tool is contained in a namespace an supports:
#        type                       // e.g., c compiler
#        inputSuffix                // e.g., .c
#        outputFiles     inFiles    // e.g., regsub .c $files .o
#        tool                       // e.g., cc68k
#        buildCmds       flags file // return a list of commands to build
#                                   // file with flags
#	 flagsAdd	 oldFlags type arg
#
# RESOURCE FILES
# [Wind::base]/host/resource/osconfig/tcl/tc_*.tcl
# [Wind::base]/host/resource/osconfig/tcl/macrosInfo.tcl
#

package require PrjUtils
package require Wind

set prjToolChains ""

###############################################################################
#
# prjTcAdd - add a toolchain
#
# SYNOPSIS
# \cs
#   prjTcAdd <tc>
# \ce
#
# PARAMETERS:
#   tc : the toolchain namespace to be added to toolchain list
#
# RETURNS: N/A
#
# ERRORS: 
# \is
# \i invalid toolChain namespace <tc>
# \ie
#

proc prjTcAdd {tc} {
    global prjToolChains

    if {[catch "namespace eval $tc {info procs tool}" val]} {
        error "invalid toolChain namespace $tc"
    }
    if {$val != "tool"} {
        error "invalid toolChain namespace $tc"
    }

    lappend prjToolChains $tc
}

###############################################################################
#
# prjTcListGet - get a list of toolchains
#
# SYNOPSIS
# \cs
#   prjTcListGet [<cpu> [<tool> [<type>]]]
# \ce
#
# DESCRIPTION
# Get the list of toolchains matching given <cpu>, <tool> and <type>. If any
# of the parameters is ommitted (or set to an empty String), all toolchains
# for the specified element will be returned.
#
# PARAMETERS:
#   cpu : name of the 'CPU' to get toolchains for
#   tool : name of the tool used by the toolchain
#   type : type of toolchain to retrieve
#
# RETURNS: A list of toolchains matching the given parameters.
#
# EXAMPLES
# Get toolchains for <cpu> 'PPC604', with <tool> 'diab', of <type> 'C++' :
#
# \cs
#   % prjTcListGet PPC604 diab "C++"
#   ::tc_PPC604diab
#   %
# \ce
#
# Get all toolchains for <cpu> 'PENTIUM' :
# \cs
#   % prjTcListGet PENTIUM.*
#   ::tc_PENTIUMgnu
#   %
# \ce
#
# ERRORS: N/A
#
# SEE ALSO: prjTcAdd
#

proc prjTcListGet {{cpu ""} {tool ""} {type ""}} {

    global prjToolChains

    set tcList ""
    foreach tc $prjToolChains {
	if {("$cpu" != "") && ([::${tc}::cpu] != $cpu)} {
	    continue
	}
	if {("$tool" != "") && ([::${tc}::tool] != $tool)} {
	    continue
	}
	if {("$type" != "") && ([::${tc}::type] != $type)} {
	    continue
	}
	lappend tcList $tc
    }
    return $tcList
}

###############################################################################
#
# prjBuildListGet - get a list of project build specs
#
# SYNOPSIS
# \cs
#   prjBuildListGet <hProj>
# \ce
#
# DESCRIPTION
# Get the list of build specifications available for the <hProj> project.
#
# PARAMETERS:
#   hProj : handle of the Project to get list of available build specs for
#
# RETURNS: A list of build specification names
#
# ERRORS: N/A
#
# SEE ALSO: prjBuildCurrentSet, prjBuildCurrentGet
#

proc prjBuildListGet {hProj} {
    return [prjTagDataGet $hProj BUILD__LIST]
}

###############################################################################
#
# prjBuildCurrentGet - get the current build spec
#
# SYNOPSIS
# \cs
#   prjBuildCurrentGet <hProj>
# \ce
#
# DESCRIPTION
# Get the name of the current Build specification.
#
# PARAMETERS
#   hProj : project handle to get current build for.
#
# RETURNS: The name of the current build specification for the <hProj> project.
#
# ERRORS: N/A
#
# SEE ALSO: prjBuildCurrentSet
#

proc prjBuildCurrentGet {hProj} {
    return [prjTagDataGet $hProj BUILD__CURRENT]
}

###############################################################################
#
# prjBuildCurrentSet - set the current build spec
#
# SYNOPSIS
# \cs
#   prjBuildCurrentSet <hProj> <buildSpec>
# \ce
#
# DESCRIPTION
# Set build specification of project <hProj> to <buildSpec>
#
# PARAMETERS
#   hProj : handle of the project to set current build for.
#   buildSpec : build specification to set for project <hProj>
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: prjBuilsCurrentGet
#

proc prjBuildCurrentSet {hProj buildSpec} {
    if {[lsearch [prjTagDataGet $hProj BUILD__LIST] $buildSpec] == -1} {
	error "build spec $buildSpec doesn't exist"
    }

    prjTagDataSet $hProj BUILD__CURRENT $buildSpec
}

###############################################################################
#
# prjBuildCreate - create a build spec
#
# SYNOPSIS
# \cs
#   prjBuildCreate <hProj> <buildSpec> <tc>
# \ce
#
# DESCRIPTION
# This routine creates a build spec using the toolChains default flags and then
# runs the project-type managers 'buildModify' procedure to customize the
# <buildSpec> for the project.
#
# By convention, build specs should have the toolChain name at the begining.
# E.g. :
# \cs
#   MC68040_debug
# \ce
#
# PARAMETERS
#   hProj : the project handle to create a build spec for
#   buildSpec : name of the new build specification to create
#   tc : the toolchain to use for this new build specification
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i build spec <buildSpec> already exists
# \ie
#
# SEE ALSO: prjBuildCopy
#

proc prjBuildCreate {hProj buildSpec tc} {
    if {[lsearch [prjTagDataGet $hProj BUILD__LIST] $buildSpec] != -1} {
	error "build spec $buildSpec already exists"
    }

    # add the new buildSpec to the list of buildSpec's, and make it current

    prjTagDataSet $hProj BUILD__LIST \
		[concat [prjTagDataGet $hProj BUILD__LIST] $buildSpec]
    prjTagDataSet $hProj BUILD__CURRENT $buildSpec

    # associate the toolchain and default build flags with the new buildSpec.

    prjTagDataSet $hProj BUILD_${buildSpec}_TC $tc
    foreach macro [${tc}::macroListGet] {
	set macroVal [${tc}::macroDefaultValGet $macro]
	prjBuildMacroSet $hProj $buildSpec $macro $macroVal
    }

    # allow the project-type manager to modify the build as needed

    set prjType [prjTypeGet $hProj]
    ::${prjType}::buildModify $hProj $buildSpec

    # done

    return $buildSpec
}

###############################################################################
#
# prjBuildSpecificTagsGet - get list of BUILD_${build}_ tags for regsub calls
#
# SYNOPSIS
# \cs
#   prjBuildSpecificTagsGet
# \ce
#
# DESCRIPTION
# Get the list of BUILD_${build}_* tags in a regsub-ready form.
#
# NOTE: If you add another build-specific tag, you must add it to this list so
#       that 'prjBuildCopy' will correctly copy it to a new build
#
# RETURNS: list of build-specific tags in regsub "any of these" form, i.e. 
#          \(tag1|tag2|...\). Currently the list is
#
# \cs
# \(TC|MACRO|BUILDRULE|RO_DEPEND_PATH\)
# \ce
#
# ERRORS: N/A
#
# SEE ALSO: prjBuildCopy
#

proc prjBuildSpecificTagsGet {} {
    return "\(TC|MACRO|BUILDRULE|RO_DEPEND_PATH\)"
}

###############################################################################
#
# prjBuildCopy - create a build spec by copying another
#
# SYNOPSIS
# \cs
#   prjBuildCopy <hProj> <buildSpec> <newSpec>
# \ce
#
# DESCRIPTION
# This procedure dupliactes the <buildSpec> build specification in a new
# <newSpec> build specification for the <hProj> project.
#
# PARAMETERS
#   hProj : handle of the project to copy <buildSpec> from, and <newSpec> to
#   buildSpec : build specification to copy <newSpec> from
#   newSpec : name of the new build specification copied from <buildSpec>
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i build spec <buildSpec> doesn't exist
# \i build spec <newSpec> already exist
# \ie
#
# SEE ALSO: prjBuildRemove, prjBuildCreate
#

proc prjBuildCopy {hProj buildSpec newSpec} {

    if {[lsearch [prjTagDataGet $hProj BUILD__LIST] $buildSpec] == -1} {
        error "build spec $buildSpec doesn't exist"
    }
    if {[lsearch [prjTagDataGet $hProj BUILD__LIST] $newSpec] != -1} {
        error "build spec $newSpec already exist"
    }

    foreach tag [prjTagListGet $hProj BUILD_${buildSpec}_] {

	regsub BUILD_${buildSpec}_[prjBuildSpecificTagsGet] \
	       $tag BUILD_${newSpec}_\\1 newTag

	prjTagDataSet $hProj $newTag [prjTagDataGet $hProj $tag]
    }

    prjTagDataSet $hProj BUILD__LIST \
		  [concat [prjTagDataGet $hProj BUILD__LIST] $newSpec]
}

###############################################################################
#
# prjBuildDelete - delete a build spec
#
# SYNOPSIS
# \cs
#   prjBuildDelete <hProj> <buildSpec>
# \ce
#
# DESCRIPTION
# Removes the given <buildSpec> from <hProj> project.
#
# PARAMETERS
#   hProj : handle of the project to remove <buildSpec> from
#   buildSpec : build specification to remove from <hProj>
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i build spec <buildSpec> doesn't exist
# \ie
#
# SEE ALSO: prjBuildCreate
#

proc prjBuildDelete {hProj buildSpec} {
    set buildSpecs [prjTagDataGet $hProj BUILD__LIST]
    set ix [lsearch $buildSpecs $buildSpec]
    if {$ix == -1} {
	error "build spec $buildSpec doesn't exist"
    }

    set buildSpecs [lreplace $buildSpecs $ix $ix]
    prjTagDataSet $hProj BUILD__LIST $buildSpecs
    foreach tag [prjTagListGet $hProj BUILD_${buildSpec}_] {
	prjTagRemove $hProj $tag
    }

    set currentSpec [prjTagDataGet $hProj BUILD__CURRENT]
    if {"$currentSpec" == $buildSpec} {
	if {[llength $buildSpecs] == 0} {
	    prjTagDataSet $hProj BUILD__CURRENT ""
	} else {
	    prjTagDataSet $hProj BUILD__CURRENT [lindex $buildSpecs 0]
	}
    }
}

###############################################################################
#
# prjBuildMacroListGet - get a list of build macros
#
# SYNOPSIS
# \cs
#   prjBuildMacroListGet <hProj> <hSpec> [<rule> [<all>]]
# \ce
#
# DESCRIPTION
# Get the list of macros for the specified Build spec.
#
# PARAMETERS
#   hProj : project handle to get macros list from
#   hSpec : build specification to get macros list for
#   rule : specifies the rule to get macros for
#   all : specify to get all macros (even the hidden macros)
#
# RETURNS: A list of macros for the given build spec in the given project
#
# ERRORS: N/A
#
# SEE ALSO: prjBuildMacroSet, prjBuildMacroGet, prjBuildMacroRemove
# 

proc prjBuildMacroListGet {hProj hSpec {rule ""} {all 0}} {
    global macroHide
    global macroDesc
    global macroRule
    global macroCpu
    global macroComps
    global macrosOfProj

    set tc [prjBuildTcGet $hProj $hSpec]
    set cpu [${tc}::cpu]
    if {$rule == ""} {
	set rule [prjBuildRuleCurrentGet $hProj $hSpec]
    }

    set wcc	[wccCurrentGet $hProj]

    # if there is no Wind Component Configuration, we may be in an
    # application project, just return.

    if {$wcc == ""} {
	set cmpList	{}
    } else {
	set cmpList [wccComponentListGet $hProj $wcc]
    }

    set prjType [prjTypeGet $hProj]

    set tags [prjTagListGet $hProj BUILD_${hSpec}_MACRO_]
    regsub -all BUILD_${hSpec}_MACRO_ $tags "" tags
    set macros ""

    foreach macro [concat $tags $macrosOfProj($prjType)] {

	if {[info exists macroRule($macro)]} {
	    if {[regexp $macroRule($macro) $rule] == 0} continue
	}

	if {[info exists macroCpu($macro)]} {
	    if {[regexp $macroCpu($macro) $cpu] == 0} continue
	}

	if {[info exists macroComps($macro)]} {
	    if {[lsearch -regexp $cmpList $macroComps($macro)] == -1} {
		continue
	    }
	}

	if {($all == 0) && ([lsearch $macroHide $macro] != -1)} {
	    continue
	}

	if {[lsearch $macros $macro] >= 0} {
	    continue
	}

	lappend macros $macro
    }

    return $macros
}

###############################################################################
#
# prjBuildMacroGet - get the value of a build macro
#
# SYNOPSIS
# \cs
#   prjBuildMacroGet <hProj> <hSpec> <macro>
# \ce
#
# DESCRIPTION
# Get the value of macro <macro> for build specification <buildSpec> in project
# <hProj>
#
# PARAMETERS
#   hProj : handle of the project to get macro value for
#   hSpec : build specification to get macro value for
#   macro : name of the macro to get value for
#
# RETURNS:
#
# ERRORS:
#
# SEE ALSO: prjBuildMacroListGet, prjBuildMacroSet, prjBuildMacroDescGet,
#           prjBuildMacroRemove
#

proc prjBuildMacroGet {hProj hSpec macro} {

    global macroDefault

    if {([info exists macroDefault($macro)]) && \
	([lsearch [prjTagListGet $hProj] BUILD_${hSpec}_MACRO_${macro}])== -1} {

	return $macroDefault($macro)
    }

    return [prjTagDataGet $hProj BUILD_${hSpec}_MACRO_${macro}]
}

###############################################################################
#
# prjBuildMacroSet - set the value of a build macro
#
# SYNOPSIS
# \cs
#   prjBuildMacroSet <hProj> <hSpec> <macro> <value>
# \ce
#
# DESCRIPTION
# Set the value of <macro> to <value> for build specification <hSpec> in project
# <hProj>
#
# PARAMETERS
#   hProj : handle of the project to set value
#   hSpec : name of the build specification to set value
#   macro : name of the macro to set a value
#   value : value to give to <macro>
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: prjBuildMacroListGet, prjBuildMacroGet, prjBuildMacroDescGet,
#           prjBuildMacroRemove
#   

proc prjBuildMacroSet {hProj hSpec macro value} {

    prjTagDataSet $hProj BUILD_${hSpec}_MACRO_${macro} $value
}

###############################################################################
#
# prjBuildMacroRemove - remove a build macro
#
# SYNOPSIS
# \cs
#   prjBuildMacroRemove <hProj> <hSpec> <macro>
# \ce
#
# DESCRIPTION
# Removes the specified <macro> macro from build specification <hSpec> in
# project <hProj>
#
# PARAMETERS
#   hProj : handle of the project to remove macro from
#   hSpec : build specification to remove macro for
#   macro : name of the macro to remove for build specification <hSpec> in
#           project <hProj>
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: prjBuildMacroSet, prjBuildMacroListGet
#

proc prjBuildMacroRemove {hProj hSpec macro} {
    prjTagRemove $hProj BUILD_${hSpec}_MACRO_${macro}
}

###############################################################################
#
# prjBuildMacroDescGet - get description of a build macro
# 
# SYNOPSIS
# \cs
#   prjBuildMacroDescGet <hProj> <hSpec> <macro>
# \ce
#
# DESCRIPTION
# Get the descrition field of the given <macro>
#
# PARAMETERS:
#   hProj : handle of the project to get macro description for
#   hSpec : build specification to get macro description for
#   macro : name of the macro to get description for
#
# RETURNS: The description of given <macro>
#
# ERRORS: N/A
#
# SEE ALSO: prjBuildMacroListGet, prjBuildMacroSet, prjBuildMacroGet,
#           prjBuildMacroRemove
#

proc prjBuildMacroDescGet {hProj hSpec macro} {
    variable macroDesc

    set desc ""
    if {[info exists macroDesc($macro)]} {
    	set desc $macroDesc($macro)
    }

    return $desc
}

###############################################################################
#
# prjBuildRuleDescGet - get description of a default rule
# 
# RETURN:  "ruleComment"
#
# NOTE: this is a temporary solution.  Rules should come from the bsp.

proc prjBuildRuleDescGet {hProj hSpec rule} {
	set ruleDesc(clean) \
		"Clean project."
	set ruleDesc(vxWorks) \
		"RAM based VxWorks image, linked to RAM_LOW_ADRS.\
		It is loaded into RAM via some external program\
		such as a bootROM. This is the default development image."
	set ruleDesc(vxWorks_romResident) \
		"ROM resident image. The program text remains in ROM, only\
		the data is copied to RAM. This image has the\
		fastest boot time and uses the least amount of RAM, but\
		runs slower on boards with slow ROM access."
	set ruleDesc(vxWorks_rom) \
		"RAM based image that starts in ROM. The ROM startup code\
		copies the entire image to RAM and then jumps to it. This\
		image generally has a slower startup time, but faster\
		execution time, than vxWorks_romResident"
	set ruleDesc(vxWorks_romCompress) \
		"Compressed RAM based image that starts in ROM. This\
		image can fit almost twice the code as other ROM images.\
		But it has the slowest boot time, since the image must be\
		uncompressed. The run-time speed is the same as for vxWorks_rom"
	set ruleDesc(objects) \
		"Build object files from project source."
	set ruleDesc(archive) \
		"Build project objects, then put them in an archive"
	set ruleDesc([constructProjectOutFileName $hProj]) [concat \
		"Build project objects, then link them together for download.\n" \
		"This rule also performs C++ munching needed to support\n"	\
		"static constructors"]
	set ruleDesc([constructProjectPlFileName $hProj]) \
		"Build project objects, then link them together. \
		This rule does not perform C++ munching needed to support \
		static constructors. It is intended to support sub-projects. \
                It is not intended for downloading. \
                The correct object to download is \
                [constructProjectOutFileName $hProj]."

    set desc ""
    set ext [file extension $rule]
    if {$ext == ".hex" || $ext == ".bin" || $ext == ".old"} {
        set rule [file rootname $rule]
    }

    if {[info exists ruleDesc($rule)]} {
    	set desc $ruleDesc($rule)
    }

    return $desc
}

###############################################################################
#
# prjBuildRuleSet - set or add a custom build rule
#
# SYNOPSIS
# \cs
#   prjBuildRuleSet <hProj> <hSpec> <rule> <cmds>
# \ce
#
# DESCRIPTION
# Create a set of commands for a new Build rule.
#
# This procedure created a new set of commands <cmds> for the new build rule
# <rule>. This rule belongs to the <hSpec> build specification of project
# <hProj>.
#
# PARAMETERS
#   hProj : handle of the project to set or add a custom build to.
#   hSpec : build specification to set or add a custom build to.
#   rule : name of the rule be be added or set for <hSpec> build spec.
#   cmds : build commands to apply to rule <rule>
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: prjBuildRuleCurrentSet, prjBuildRuleCurrentGet
#

proc prjBuildRuleSet {hProj hSpec rule cmds} {

    prjTagDataSet $hProj BUILD_RULE_${rule} $cmds
}

###############################################################################
#
# prjBuildRuleGet - get the value of a custom build rule
#
# SYNOPSIS
# \cs
#   prjBuildRuleGet <hProj> <hSpec> <rule>
# \ce
#
# DESCRIPTION
# Get the value of the <rule> build rule associated with <hSpec> build
# specification for project <hProj>
#
# PARAMETERS:
#   hProj : handle of the project to get build rule for
#   hSpec : build specification to get <rule> value for
#   rule : name of the rule to get value for
#
# RETURNS: The value of <rule> build rule.
#
# ERRORS: N/A
#
# SEE ALSO: prjBuildRuleSet
#

proc prjBuildRuleGet {hProj hSpec rule} {

    return [prjTagDataGet $hProj BUILD_RULE_${rule}]
}

###############################################################################
#
# prjBuildRuleDelete - delete a custom build rule
# 
# RETURN: prjTagRemove return code
#

proc prjBuildRuleDelete {hProj hSpec rule} {
    
    return [prjTagRemove $hProj BUILD_RULE_${rule}]
}

###############################################################################
#
# prjBuildRuleListGet - get a list of build rules
#
# SYNOPSIS
# \cs
#   prjBuildRuleListGet <hProj> <hSpec>
# \ce
#
# DESCRIPTION
# Get the list of build rules for <hSpec> build specification in project
# <hProj>.
#
# PARAMETERS
#   hProj : handle of the project to get build rules from
#   hSpec : build specification to get build rules fro,
#
# RETURNS: The list of build rules for given <hSpec> build specification in
#          <hProj> project.
#
# ERRORS: N/A
#
# SEE ALSO: prjBuildRuleCurrentGet, prjBuildRuleCurrentSet
#

proc prjBuildRuleListGet {hProj hSpec} {
    # get project-type rules
    set prjRules [[prjTypeGet $hProj]::buildRuleListGet $hProj]

    # get custom build rules
    set customRules [prjTagListGet $hProj  BUILD_RULE_]
    regsub -all BUILD_RULE_ $customRules "" customRules
    set ruleList "$prjRules $customRules"
    removeDuplicatesFromList ruleList
    return $ruleList
}

###############################################################################
#
# prjBuildRuleCurrentSet - set the current build rule
#
# SYNOPSIS
# \cs
#   prjBuildRuleCurrentSet <hProj> <hSpec> <rule>
# \ce
#
# DESCRIPTION
# Set the current build rule for <hSpec> build specification to <rule> in
# project <hSpec>.
#
# PARAMETERS:
#   hProj : handle of the project to set current build rule for
#   hSpec : build specification to set current build rule for
#   rule : current rule to use for <hSpec> build specification
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: prjBuildRuleCurrentGet
#

proc prjBuildRuleCurrentSet {hProj hSpec rule} {

    prjTagDataSet $hProj BUILD_${hSpec}_BUILDRULE $rule
}

###############################################################################
#
# prjBuildRuleCurrentGet - get the current build rule
#
# SYNOPSIS
# \cs
#   prjBuildRuleCurrentGet <hProj> <hSpec>
# \ce
#
# Get the current build rule for Build Specification <hSpec> in project <hProj>.
#
# PARAMETERS
#   hProj : handle of the project to get current build rule for
#   hSpec : build specification to get current build rule for
#
# RETURNS: The current build rune for <hSpec> build specification in <hProj>
#          project.
#
# ERRORS: N/A
#
# SEE ALSO: prjBuildRuleCurrentSet
#

proc prjBuildRuleCurrentGet {hProj hSpec} {

    set rule [prjTagDataGet $hProj BUILD_${hSpec}_BUILDRULE]
    if {"$rule" == ""} {
	set rule [[prjTypeGet $hProj]::buildRuleDefault $hProj]
    }

    return $rule
}

###############################################################################
#
# prjBuildTcGet - get the toolChain associated with a build spec
#
# SYNOPSIS
# \cs
#   prjBuildTcGet <hProj> <buildSpec>
# \ce
#
# DESCRIPTION
# Get the toolchain associated with <buildSpec> build specification for
# project <hProj>
#
# PARAMETERS
#   hProj : handle of the project to get toolchain for
#   buildSpec : build specification to get toolchain for
#
# RETURNS: The name of the toolchain associated with <buildSpec> build
#          specification in project <hProj>
#
# ERRORS:
# \is
# \i build spec <buildSpec> doesn't exists
# \ie
#
# SEE ALSO: prjTclListGet
#

proc prjBuildTcGet {hProj buildSpec} {
    if {[lsearch [prjTagDataGet $hProj BUILD__LIST] $buildSpec] == -1} {
	error "build spec $buildSpec doesn't exists"
    }

    return [prjTagDataGet $hProj BUILD_${buildSpec}_TC]
}

################################################################################
#
# prjBuildFlagsSet - set a tool's flags in a build spec
#
# SYNOPSIS
# \cs
#   prjBuildFlagsSet <hProj> <buildSpec> <tool> <flags>
# \ce
#
# DESCRIPTION
# Set the given tool's flags to <flags> for the given build spec (<buildSpec>).
#
# PARAMETERS:
#   hProj : handle of the project to set tool's flags for
#   buildSpec : build specification the flags should apply to
#   tool : tool to set flags for
#   flags : value to give to tool's flags
#
# RETURNS: N/A
#
# ERRROS:
# \is
# \i build spec <buildSpec> doesn't exists
# No build specifcation named <buildSpec> was found for this project
# \ie
#
# SEE ALSO: prjBuildFlagsGet
#

proc prjBuildFlagsSet {hProj buildSpec tool flags} {
    if {[lsearch [prjTagDataGet $hProj BUILD__LIST] $buildSpec] == -1} {
	error "build spec $buildSpec doesn't exists"
    }

    set macro [${tool}::flagsMacro]
    prjBuildMacroSet $hProj $buildSpec $macro $flags
}


################################################################################
#
# prjBuildIncludePath - get or set the build include path for a given tool
#
# SYNOPSIS
# \cs
#   prjBuildIncludePath <hProj> <buildSpec> <tool> [<newPath>]
# \ce
#
# DESCRIPTION
# This procedure always returns the build include path for the given <tool>.
#
# If <newPath> is provided, the current build include path is set to
# <newPath>, and <newPath> is returned.
#
# PARAMETERS:
#   hProj : handle of the project to set build include path for
#   buildSpec : build specification to set build include path for
#   tool : build tool to set include path for
#   newPath : new include path to set this build include path to
#
# RETURNS: The current build include path list
#
# ERRORS: N/A
#
# SEE ALSO: prjBuildFlagsGet, prjBuildMacroGet, prjBuildFlagsSet
#

proc prjBuildIncludePath {hProj buildSpec tool {newPath ""}} {

    set incList	{}
    set flags	[prjBuildFlagsGet $hProj $buildSpec $tool]
    set incTag	[prjBuildMacroGet $hProj $buildSpec OPTION_INCLUDE_DIR]
    set tagLen	[string length $incTag]

    if {$newPath == ""} {

	# look through the flags for flags starting with incTag

	foreach flag $flags {

	    set flagEnd	[expr {[string length $flag]} - 1]
	    if {[regexp "^$incTag" $flag]} {
		lappend incList [string range $flag $tagLen $flagEnd]
	    }
	}
    } else {

	# set the new include path

	set newFlags		""
	set shouldAddPaths	1
	set incList		$newPath

	foreach flag $flags {

	    if {[regexp "^$incTag" $flag]} {

		# do not add it to the new flags, but on the first incTag,
		# insert new include paths

		if {$shouldAddPaths} {

		    set shouldAddPaths	0

		    foreach path $newPath {

			set newFlags [format "%s %s%s" $newFlags $incTag $path]
		    }
		}

	    } else {

		set newFlags	[format "%s %s" $newFlags $flag]
	    }

	    # the flags may be added if there was no <incTag> in the old flags

	    if {$shouldAddPaths} {
		set shouldAddPaths	0
		foreach path $newPath {
		    set newFlags [format "%s %s%s" $newFlags $incTag $path]
		}
	    }
	}

	# now update the flags for this tool

	prjBuildFlagsSet $hProj $buildSpec $tool [string trim $newFlags]
    }

    return $incList
}

###############################################################################
#
# prjBuildFlagsGet - get the tool's flags from a build spec
#
# SYNOPSIS
# \cs
#   prjBuildFlagsGet <hProj> <buildSpec> <tool>
# \ce
#
# DESCRIPTION
# This procedure gets the givens <tool>'s flags for <buildSpec> build
# specification of <hProj> project.
#
# PARAMETERS:
#   hProj : handle of the project to get flags for
#   buildSpec : build specification to get flags for
#   tool : toolchain to get flags for
#
# ERRORS:
# \is
# \i build spec <buildSpec> doesn't exists
# \ie
#
# SEE ALSO: prjBuildCurrentGet, prjBuildListGet
#
# INTERNAL: The <file> argument is ignored for now, but let it here unless all
# the 'prjBuildFlagsGet' call do not use <file> argument anymore.
#

proc prjBuildFlagsGet {hProj buildSpec tool {file ""}} {

    if {[lsearch [prjTagDataGet $hProj BUILD__LIST] $buildSpec] == -1} {
	error "build spec $buildSpec doesn't exists"
    }

    set macro [${tool}::flagsMacro]
    return [prjBuildMacroGet $hProj $buildSpec $macro]
}

###############################################################################
#
# prjBuildCmdsGet - get the build commands for building a file
#
# SYNOPSIS
# \cs
#   prjBuildCmdsGet <hProj> <build> <file>
# \ce
#
# DESCRIPTION
# This procedure gets the command to run to build <file> for the given <hProj>
# project, using <build> Build Specification
#
# PARAMETERS:
#   hProj: handle of the project to get <file>'s build command for
#   build : build specification to get <file>'s build command for
#   file : file to get get build command for
#
# RETURNS: The command to run to build <file>
#
# SEE ALSO: prjBuildTcGet, prjBuildFlagsGet
#

proc prjBuildCmdsGet {hProj build file} {
    set tc    [prjBuildTcGet $hProj $build]
    set tool  [::${tc}::toolFindByFile $file]
    set flags [${tool}::flagsMacro]

    return [::${tool}::buildCmds \$($flags) $file]
}

###############################################################################
#
# prjBuild - build the project with the current settings
#
# SYNOPSIS
# \cs
#   prjBuild <hProj> [<rule> [<spec> [<throwError>]]]
# \ce
#
# DESCRIPTION
# This routine builds the project with the current settings.
#
# It also creates a new configuration out of the tuple (buildSpecCurrent,
# imageTypeCurrent, imageConfigCurrent).
#
# ERRORS:
# \is
# \i Could not build if component configuration is invalid : <error>
# The kernel component configuration is invalid, building such an image
# could result in serious problems. The <error> message describes what
# should be done to be able to build
# \ie

proc prjBuild {hProj {rule ""} {spec ""} {throwError ""}} {

    # SPR#102787 : do not try to build if the component configuration is not
    # valid
    #
    # Do not try to validate if it is just a "clean"

    if {($rule != "clean")} {

	set validation [wccValidate $hProj [wccCurrentGet $hProj]]

	if {$validation != ""} {
	    puts [prjStderr] "Could not build if component configuration is\
			      invalid :\n$validation"
	    error "Could not build if component configuration is invalid\
		   :\n$validation"
	}
    }

    set prjType [prjTypeGet $hProj]
    set info [::${prjType}::buildCmdGet $hProj $rule $spec]
    set cmd [lindex $info 0]
    set prjDir [lindex $info 1]

    # invoke make to do the build

    set dir [pwd]
    cd $prjDir

    #
    # If any of the exec'd commands writes to standard error 
    # (e.g. a compiler warning) and standard error isn't redirected, 
    # then exec will return an error. To avoid this we use 
    # "2>@ stderr", which redirects standard error to 
    # standard error. This has no effect other than to prevent 
    # exec from returning an error on e.g. a compiler warning
    #
    # Update (cjs 23mar2002): not exactly true :-)
    # on real error, error message is no longer returned
    # in its entirety with this redirection in place
    #
    
    set cmdString "eval exec $cmd >@ [prjStdout] 2>@ [prjStderr]"

    # if throwError == "", catch make errors and pass them
    # up as the value of this routine.
    # otherwise, try the command again without redirecting stdout
    # and stderror, so we can throw complete error to caller
    if {[catch $cmdString errMsg]} {
        if {$throwError != ""} {

            # try again w/o redirection so that when we throw an
            # error, it winds up in our error message
            catch {eval exec $cmd} errorMsg
            cd $dir
            error $errorMsg
        }
    } else {
        cd $dir
        return $errMsg
    }
}

################################################################################
#
# prjDepPathAdd - add a directories to the read-only dependancy path
#
# SYNOPSIS
# \cs
#   prjDepPathAdd <hProj> <buildSpec> <newDirList>
# \ce
#
# DESCRIPTION
# Add new directories to the Read-Only dependency path
#
# PARAMETERS:
#   hProj : handle of the project to add directories for
#   buildSpec : build specification to add read-only directories to
#   newDirList : list of directories to be added to the read-only dependency
#                path
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: prjDepPathRemove
#

proc prjDepPathAdd {hProj buildSpec newDirList} {

    set roDepPath	[prjDepPathGet $hProj $buildSpec]
    set prjDir		[file dirname [prjInfoGet $hProj fileName]]

    foreach dir $newDirList {
	lappend roDepPath [PrjUtils::relPath $dir $prjDir]
    }

    prjTagDataSet $hProj BUILD_${buildSpec}_RO_DEPEND_PATH $roDepPath
}

################################################################################
#
# prjDepPathRemove - remove a directory from the read-only dependancy path
#
# SYNOPSIS
# \cs
#   prjDepPathRemove <hProj> <buildSpec> <newDirList>
# \ce
#
# DESCRIPTION
# Remove a directory from the Read-Only dependency path
#
# PARAMETERS:
#   hProj : handle of the project to remove directory for
#   buildSpec : build specification to remove read-only directory from
#   newDirList : directory to be removed to the read-only dependency path
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: prjDepPathAdd
#

proc prjDepPathRemove {hProj buildSpec dir} {

    set roDepPath [prjDepPathGet $hProj $buildSpec]
    set newList ""

    foreach depDir $roDepPath {
	if {$depDir != $dir} {
	    lappend newList $depDir
	}
    }
    prjTagDataSet $hProj BUILD_${buildSpec}_RO_DEPEND_PATH $newList
}

################################################################################
#
# prjDepPathGet - get the read-only dependancy path
#
# SYNOPSIS
# \cs
#   prjDepPathGet <hProj> <buildSpec>
# \ce
#
# DESCRIPTION
# Get the Read-Only dependency path
#
# PARAMETERS:
#   hProj : handle of the project to get read-only dependency path for
#   buildSpec : build specification to get read-only dependency path for
#
# RETURNS: The read-only dependency path
#
# SEE ALSO: prjDepPathSet, prjDepPathAdd, prjDepPathRemove
#

proc prjDepPathGet {hProj buildSpec} {

    set path [prjTagDataGet $hProj BUILD_${buildSpec}_RO_DEPEND_PATH]
    set fullPath ""
    
    foreach dir $path {
	lappend fullPath [prjFullPath $hProj $dir]
    }

    return $fullPath
}

################################################################################
#
# prjDepPathSet - set the read-only dependancy path
#
# SYNOPSIS
# \cs
#   prjDepPathSet <hProj> <buildSpec> <listPaths>
# \ce
#
# DESCRIPTION
# Set the Read-Only dependency path
#
# PARAMETERS:
#   hProj : handle of the project to set read-only dependency path for
#   buildSpec : build sepcification to set read-only dependency path for
#   listPaths : list of directories to set read-only dependency path to
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: prjDepPathGet, prjDepPathAdd, prjDepPathRemove
#

proc prjDepPathSet {hProj buildSpec listPaths} {

    set listRelativePaths	{}
    set prjDir			[file dirname [prjInfoGet $hProj fileName]]

    foreach dir $listPaths {
	lappend listRelativePaths [PrjUtils::relPath $dir $prjDir]
    }

    prjTagDataSet $hProj BUILD_${buildSpec}_RO_DEPEND_PATH $listRelativePaths
}


###############################################################################
#
# prjDepPathDefaults - get the default read-only dependancy paths
#
# SYNOPSIS
# \cs
#   prjDepPathDefaults
# \ce
#
# DESCRIPTION
# Get the default read-only dependency paths
#
# PARAMETERS: N/A
#
# RETURNS: A list of directories which are the default read-only dependency path
#
# SEE ALSO: prjDepPathSet, prjDepPathGet, prjDepPathAdd, prjDepPathRemove
#

proc prjDepPathDefaults {} {
    return [list [Wind::path target h] [Wind::path target src] \
		 [Wind::path target config] [Wind::path target h wrn coreip]]
}

################################################################################
#
# prjBuildMacrosExpand - expand macros of given <flags>
#
# SYNOPSIS
# \ts
#   prjBuildMacrosExpand <hProj> <buildSpec> <flags>
# \te
#
# DESCRIPTION
# This procedure lists all the macros defined for this project and build spec,
# then looks for the $(<MACRO>) string within <flags>, and replace any macro
# with its value.
#
# PARAMETERS:
#   hProj     : project in Workspace
#   buildSpec : the build specification for those <flags>
#   flags     : the flags to expand macros within
#
# RETURNS: The expanded <flags> (if there was any possibility to expand a
#          macro within those <flags>).
#
# SEE ALSO: prjBuildMacroGet
#

proc prjBuildMacrosExpand {hProj buildSpec flags} {

    set macroList [prjBuildMacroListGet $hProj $buildSpec]

    foreach macroName $macroList {

	set macroPattern	[format "\$(%s)" $macroName]

	while {[set macroIx [string first $macroPattern $flags]] != -1} {

	    set macroPatternLen	[string length $macroPattern]
	    set flagsLen	[string length $flags]

	    set start		[string range $flags 0 [expr {$macroIx} - 1]]
	    set macroValue	[prjBuildMacroGet $hProj $buildSpec $macroName]
	    set tail		[string range $flags \
					[expr {$macroIx} + {$macroPatternLen}] \
					[expr {$flagsLen} - 1]]

	    set flags	[format "%s%s%s" $start $macroValue $tail]
	}
    }

    return [prjBuildIncludePathFix $flags]
}

################################################################################
#
# prjBuildIncludePathFix - fixes the include paths to use slash path notation
#
# SYNOPSIS
# \cs
#   prjBuildIncludePathFix <flags>
# \ce
#
# DESCRIPTION
# This procedure parses <flags>, looking for '-I' flag. If one is found, then
# the path attached to is it replaced by a slash path (if it contained backslash
# paths).
#
# PARAMETERS:
#   flags : the compiler flags to replace include paths with their slash
#           notation version
#
# RETURNS: The given <flags> with slash path notation rather than backslash
#
#

proc prjBuildIncludePathFix {flags} {

    regsub -all -- "\t" $flags " " flags

    set newFlags	""
    append flags	" "
    set strLen		[string length $flags]

    while {[regexp -indices { +-I([^ ]+) } $flags ixes]} {

	set startIx	[lindex $ixes 0]
	set endIx	[lindex $ixes 1]

	set start	[string range $flags 0 [expr {$startIx} - 1]]
	set include	[string trim [string range $flags $startIx $endIx]]
	set flags	[string trimleft [string range $flags \
						       [expr {$endIx} + 1] \
						       [expr {$strLen} - 1]]]

	append newFlags $start " " [file join $include] " "

	set strLen [string length $flags]
    }

    # last check on the remaining flags

    if {[string first "-I" [string trim $flags]] == 0} {
	append newFlags	[file join $flags]
    } else {
	append newFlags	$flags
    }

    return [string trimright $newFlags]
}

# source resources files

set prjTclDir [file join [Wind::base] host resource osconfig tcl]

foreach file [glob [file join $prjTclDir tc_*.tcl]] {
    source $file
}

# source macros information file
source [file join $prjTclDir macrosInfo.tcl]
