# prjFileLib.tcl - project file manager library
#
# Copyright 1998-2004 Wind River Systems, Inc.
#
# modification history
# --------------------
# 02j,19Jul05,pee  fixed dependency for romInit.o and romStart.o.
#		   remove dependency generation code that is no longer used.
# 02i,20Jun05,pee  fixed missing parameter in recursion
# 02h,15dec04,fle  Fixed some regular expressions on file paths
# 02g,02nov04,fle  SPR#30565 : remove object files when a file is removed from
#                  project
# 02f,30oct04,fle  Workaround for SPR#103340 : subst // with / in dependencies
# 02e,07oct04,fle  Fixed dependency variable does not exist on failure
# 02d,08sep04,fle  SPR#88074 : Make it use prjMakefile
# 02c,30aug04,fle  Make it use prjSterr to get standard error channel
# 02b,09aug04,fle  SPR#99177 : change working directory when dependencies
#                  failed
# 02a,22jul04,fle  SPR#92405 : dependencies generation should not regenerate
#                  Makefile for each of the files
# 01z,24jun04,fle  SPR#97996 : try to not use [pwd] to get the current di
# 01y,22jun04,fle  SPR#97996 : fixed dependencies when they start with dot
#                  directories
# 01x,28may04,fle  Fixed two letter project path substitution in dependencies
#                  generation
#                  SPR#97784 : do not filter the read-only dependencies anymore
# 01w,06may04,fle  Removed all bullet lists refgen markups
# 01v,20apr04,fle  Fixed dependencies generation when generated from
#                  cmpScriptLib in the project directory
# 01u,31mar04,fle  Fixed a typo in prjFileBuildInfoGen
# 01t,12mar04,fle  Fixed backslash path subtitutions for dependencies
# 01s,12mar04,fle  Fixed .. regexp when dependency file comes last
# 01r,11mar04,fle  Again fixed dependencies because of make undesired outputs
# 01q,10mar04,fle  Made dependencies make call in silent mode to allow bsp2prj
#                  to work
# 01p,04mar04,fle  SPR#91836 : use dependencies through a make rule
# 01o,18nov03,fle  SPR#66928 : special cases if the file to add is a romInit.s
#                  or romStart.c file
# 01n,16sep03,fle  SPR#79359 : cd to project directory when generating
#                  dependencies in prjFileBuildInfoGen
# 01m,10sep03,fle  enhanced doc
# 01l,01sep03,fle  SPR#85631 : call prjBuildMacrosExpand in
#                  prjFileBuildInfoGen
# 01k,02may02,cjs  Fix spr 76532: dependency generation doesn't respect
#                  standard variables
# 01j,04oct01,cjs  Fixing spr 25520 -- dependencies generation error should
#                  propagate to the user
# 01i,18feb99,ren  fixed SPR 24963 -- problems with file remove
# 01h,03dec98,cjs  fixing problemmatic behavior for 'objects' in
#                  prjFileInfoGet()
# 01g,26oct98,ms   prjFileInfo of objects now checks for custom rules.
# 01f,03sep98,ms   keep track of files that need dependency generation
# 01e,12aug98,ms   fixed bug caused by 01d (replacement was incomplete).
# 01d,03aug98,jmp  replaced dependancies by dependencies.
# 01c,05jun98,cjs  fixed minor bug in prjFileAdd 
# 01b,03apr98,ms   compute file dependancies when info is requested
# 01a,18mar98,ms   written.
#
# DESCRIPTION
# This library provides file management within a project.
#
# It works in tandem with prjLib.tcl.
#
# It allows files to be added and removed from a project, and also allows other
# facilities to associate information with a file.
#
# The following information is associated with a file:
# \ts
# 'tool'         | - | tool used to compile this file
# 'objects'      | - | derived files produced by the tool
# 'dependencies' | - | make-dependencies (e.g., #include's)
# 'dependDone'   | - | TRUE if file dependecies have been calculated
# 'customRule'   | - | set to a custom rule name if the file is to use a custom\
#                |   | rule created by 'prjBuildRuleSet'.
# \te
#
# Other information may be added, for example:
# \ts
# 'readonly' | - | generated by the project-type manager (e.g., for vxWorks\
#            |   | project, things under WIND_BASE can optionally be designated\
#            |   | as readonly).
# \te

package require Log

################################################################################
#
# prjFileAdd - add a file to a project
#
# SYNOPSIS
# \cs
#   prjFileAdd <hProj> <file>
# \ce
#
# DESCRIPTION
# Add a file to the project file list.
#
# If the given <file> is just a file name (aka without any prepended path), the
# local folder pattern (".") is added to the file name. In this case, this
# procedure will return './<file>' insteaf of just '<file>'.
#
# PARAMETERS
#   hProj : handle of the project to add a file to
#   file : file to add the the <hProj> project
#
# RETURNS: The name of the added file.
#
# ERRORS:
# \is
# \i <file>: no such file
# \ie
#
# SEE ALSO: prjFileListGet, prjFileRemove
#

proc prjFileAdd {hProj file} {

    prjHandleValidCheck $hProj

    if {![file exists $file]} {
        error "$file: no such file"
    }

    # check if it is a file without any path

    if {[file tail $file] == $file} {
	set file [file join . $file]
    }

    set files [prjTagDataGet $hProj PROJECT_FILES]
    if {[lsearch $files $file] != -1} {		;# file already in project?
	return
    }
    
    if {[catch {prjFileBuildInfoGen $hProj $file FALSE} status]} {
	puts [prjStderr] "Error generating Build info : $status"
    }

    prjTagDataSet $hProj PROJECT_FILES [lappend files $file] 

    # SPR#66928 : check if files are not romInit.s or romStart.c, in which
    # case, their build rules are known, and should be default rules

    set fileName	[file tail $file]

    if {$fileName == "romInit.s"} {

	set build	[prjBuildCurrentGet $hProj]

	prjFileInfoSet $hProj $file customRule romInit.o
	prjFileInfoSet $hProj $file BOOT_FILE TRUE

	prjBuildRuleSet $hProj $build romInit.o \
	    [list {romInit.o : $(BSP_DIR)/romInit.s } {$(CC) $(OPTION_OBJECT_ONLY) $(CFLAGS_AS) \
				  $(PROJECT_BSP_FLAGS_EXTRA) $(ROM_FLAGS_EXTRA)\
				  $< -o $@}]

    } elseif {$fileName == "romStart.c"} {

	set build	[prjBuildCurrentGet $hProj]

	prjFileInfoSet $hProj $file customRule romStart.o
	prjFileInfoSet $hProj $file BOOT_FILE TRUE

	prjBuildRuleSet $hProj $build romStart.o \
	    [list {romStart.o : $(TGT_DIR)/config/comps/src/romStart.c } {$(CC) $(OPTION_OBJECT_ONLY) $(CFLAGS) \
				   $(PROJECT_BSP_FLAGS_EXTRA) \
				   $(ROM_FLAGS_EXTRA) $< -o $@}]
    }

    return $file
}

################################################################################
#
# prjFileRemove - remove a file from a project
#
# SYNOPSIS
# \cs
#   prjFileRemove <hProj> <file>
# \ce
#
# DESCRIPTION
# Remove a file from the kernel configuration file list
#
# PARAMETERS:
#   hProj : handle of the project to remove file for
#   file : path of the file to remove from the kernel configuration file list
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i <file>: no such file
# No such file could be found in the kernel configuration file list
# \ie
#
# SEE ALSO: prjFileListGet, prjFileAdd
#

proc prjFileRemove {hProj file} {

    prjHandleValidCheck $hProj

    # check if it is a file without any path

    if {[file tail $file] == $file} {
	set file	[file join . $file]
    }

    set files	[prjTagDataGet $hProj PROJECT_FILES]
    set ix	[lsearch $files $file]

    if {$ix == -1} {
        error "$file: no such file"
    }

    prjTagDataSet $hProj PROJECT_FILES [lreplace $files $ix $ix]

    # SPR#30565 : remove the object files related to this source file, and
    # also remove the target image so that the project gets rebuilt if one
    # of the project files is removed

    set prjFileDir	[file dirname [prjInfoGet $hProj fileName]]
    set fileInfos	[prjFileInfoListGet $hProj $file]
    set objFiles	[prjFileInfoGet $hProj $file objects]

    # remove the object files for each of the build specs

    foreach objFile $objFiles {

	foreach build [prjBuildListGet $hProj] {

	    set objFilePath	[file join $prjFileDir $build $objFile]

	    if {[file exists $objFilePath]} {
		catch {file delete -force $objFilePath}
	    }

	    # the default image should also been remove

	    set currentRule	[prjBuildRuleCurrentGet $hProj $build]
	    set imageFilePath	[file join $prjFileDir $build $currentRule]

	    if {[file exists $imageFilePath]} {
		catch {file delete -force $imageFilePath}
	    } elseif {[file exists [format "%s.exe" $imageFilePath]]} {
		catch {file delete -force [format "%s.exe" $imageFilePath]}
	    }
	}
    }

    foreach info $fileInfos {
	prjFileInfoDelete $hProj $file $info
    }
}

################################################################################
#
# prjFileListGet - return a list of project files
#
# SYNOPSIS
# \cs
#   prjFileListGet <hProj>
# \ce
#
# DESCRIPTION
# Get the list of files for given project.
#
# PARAMETERS
#   hProj : handle of the project to get file list for.
#
# RETURNS: A list of files for given project
#
# ERRORS: N/A
#
# SEE ALSO: prjOpen
#

proc prjFileListGet {hProj} {
    prjHandleValidCheck $hProj

    return [prjTagDataGet $hProj PROJECT_FILES]
}

################################################################################
#
# prjFileInfoSet - associate some information with a file
#
# SYNOPSIS
# \cs
#   prjFileInfoSet <hProj> <file> <infoType> <infoValue>
# \ce
#
# DESCRIPTION
# Associate some information with a kernel configuration file
#
# PARAMETERS:
#   hProj : handle of the project to set file info for
#   file : path of the kernel configuration file to set info for
#   infoType : type of information to set for <file>
#   infoValue : value to give to the file info
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: prjTagDataSet, prjFileInfoGet, prjFileInfoDelete, prjFileInfoListGet
#

proc prjFileInfoSet {hProj file infoType infoValue} {
    prjTagDataSet $hProj FILE_${file}_${infoType} $infoValue
}

################################################################################
#
# prjFileInfoGet - get any information associated with a file
#
# SYNOPSIS
# \cs
#   prjFileInfoGet <hProj> <file> <infoType>
# \ce
#
# DESCRIPTION
# Get the information of type <infoType> for the kernel configuration file named
# <file>
#
# PARAMETERS:
#   hProj : handle of the project to get file info for
#   file : path of the kernel configuration to get info for
#   infoType : type of information to get for <file>
#
# RETURNS: The information of type <infoType> associated to <file>
#
# SEE ALSO: prjTagDataGet, prjFileInfoSet, prjFileInfoDelete, prjFileInfoListGet
#

proc prjFileInfoGet {hProj file infoType} {
    return [prjTagDataGet $hProj FILE_${file}_${infoType}]
}

################################################################################
#
# prjFileInfoDelete - get info associated with a file
#
# SYNOPSIS
# \cs
#   prjFileInfoDelete <hProj> <file> <infoType>
# \ce
#
# DESCRIPTION
# Delete the information of type <infoType> for kernel configuration file named
# <file>
#
# PARAMETERS:
#   hProj : handle of the project to remove kernelc configuration file info for
#   file : path of the kernel configuration file to remove info for
#   infoType : type of information to remove for <file>
#
# RETURNS: N/A
#
# ERRORS: N/A
# 
# SEE ALSO: prjFileInfoGet, prjFileInfoListGet, prjFileInfoGet
#

proc prjFileInfoDelete {hProj file infoType} {
    prjTagRemove $hProj FILE_${file}_${infoType}
}

################################################################################
#
# prjFileInfoListGet - get a list of info associated with a file
#
# SYNOPSIS
# \cs
#   prjFileInfoListGet <hProj> <file>
# \ce
#
# DESCRIPTION
# Get the list of information types for kernel configuration file named <file>
#
# PARAMETERS:
#   hProj : handle of the project to get kernell configuration file information
#           types for
#   file : path of the kernel configuration file to get information types for
#
# RETURNS: A list of information types associated to <file>
#
# ERRORS: N/A
#
# SEE ALSO: prjTagListGet, prjFileInfoGet, prjFileInfoSet, prjFileInfoDelete
#

proc prjFileInfoListGet {hProj file} {

    set tags [prjTagListGet $hProj]

    set infoTypeList ""
    foreach tag $tags {
	if {[regsub FILE_[EscapeRegexpCharacters $file]_ $tag "" infoType] \
	    != 0} {

	    lappend infoTypeList $infoType
	}
    }

    return $infoTypeList
}

################################################################################
#
# prjRemoveReadOnlyDependencies - remove dependent read only path files
#
# SYNOPSIS
# \cs
#   prjRemoveReadOnlyDependencies <listDependencies> <listReadOnlyDirs>
# \ce
#
# DESCRIPTION
# Remove the dependent files which live in one of the read only paths.
#
# PARAMETERS:
#   listDependencies : a list of fully qualified header file names
#   listReadOnlyDirs : a list of fully qualified directories which are
#                      considered to be read only
#
# RETURNS: Any file in <listDependencies> which live in one of the read only
# directories are removed. The rest are returned.
#
# NOTES:
# This function is a work of art. I originally wrote it the obvious way with two
# nested foreach statements, but that ran in O(n*m) time, and in some cases took
# more time than calling the c preprocessor, effectively more than doubling the
# time to do a dependencey analysis. So, I re-wrote it using the highly
# optimized regular expression procedures. Notice there is not a single loop in
# this function.
#

proc prjRemoveReadOnlyDependencies {listDependencies listReadOnlyDirs} {

    if { [llength $listReadOnlyDirs] == 0 } {
	return $listDependencies
    }

    # join the read only directories into one string, separated by \x82
    # and escape any characters that might screw up a regular expression

    set expReadOnlyDirs [EscapeRegexpCharacters [join $listReadOnlyDirs \x82]]

    # Now take the \x82s, and change them into a big regexp |, which match
    # files (separated by \x81) in those paths 

    regsub -all \x82 "\x81[set expReadOnlyDirs]\[^\x81]*" "\[^\x81]*|\x81" \
	   expReadOnlyDirs

    # Now use this generated regexp to match and remove files in those read
    # only paths

    global tcl_platform

    if { $tcl_platform(platform) == "windows" } {
	regsub -all -nocase [string tolower $expReadOnlyDirs]\
	       "\x81[join $listDependencies \x81]" {} strResult
    } else {
	regsub -all $expReadOnlyDirs "\x81[join $listDependencies \x81]" {} \
	       strResult
    }

    # finally, remove the \x81 separators in the file list and return

    return [split [string range $strResult 1 end] \x81]
}

################################################################################
#
# prjFileBuildInfoGen - generate build information for a file
#
# SYNOPSIS
# \cs
#   prjFileBuildInfoGen <hProj> [<file> [<doDepends>]]
# \ce
#
# DESCRIPTION
# Generate the build information for a kernel configuration file
#
# If no <file> is provided, the procedure will be executed on all kernel
# configuration files (As returned by 'prjFileListGet')
#
# For that file, the 'tool' and 'object' information are set in the
# project. If no 'tool' is associated to the file, its 'dependDone' info is
# also set to 'TRUE'
#
# If <doDepends> is set to '1', the dependencies for <file> are
# generated, and stored unde the 'dependencies' information of the file, the
# 'dependDone' field of this file is then set to 'TRUE'
#
# PARAMETERS:
#   hProj : handle of the project to generate build info for
#   file : file to generate build info for (default is all kernel configuration
#          files)
#   doDepends : states if the dependencies for <file> should also be done
#
# RETURNS: N/A
#
# ERRORS: N/A
#
# SEE ALSO: prjFileListGet, prjFileInfoSet
#   

proc prjFileBuildInfoGen {hProj {file ""} {doDepends TRUE}} {

    # if no file is specified, just take every file in the project files list,
    # and call this procedure on each of those files

    if {$file == ""} {
	set files [prjFileListGet $hProj]
	set nFiles [llength $files]
	set doneFiles 0
	foreach file $files {

	    set fileTail [file tail $file]
	    prjStatusLog "generating dependencies ($fileTail)" $doneFiles \
			 $nFiles

	    if {[catch {prjFileBuildInfoGen $hProj $file $doDepends} status]} {
		puts [prjStderr] "Error generating dependencies : $status"
	    }

	    incr doneFiles
	}

	prjStatusLog "done"
	return
    }

    set build		[prjBuildCurrentGet $hProj]
    set tc		[prjBuildTcGet $hProj $build]
    set toolType	[prjTagDataGet $hProj FILE_${file}_tool]

    if {$toolType != ""} {
	set tool [${tc}::toolFindByType $toolType]
    } else {
	set tool [${tc}::toolFindByFile $file]
    }

    # if there is no tool for the given file, consider its dependencies done

    if {$tool == ""} {
	prjFileInfoSet $hProj $file dependDone TRUE
	return
    }

    # get the compile flags, and expand macros they contain

    set flags [prjBuildFlagsGet $hProj $build $tool $file]
    set flags [prjBuildMacrosExpand $hProj $build $flags]

    prjFileInfoSet $hProj $file tool [${tool}::type]
    prjFileInfoSet $hProj $file objects [${tool}::outputFiles $file]
}
