# prjFsFileLib.tcl - project file system file library
#
# Copyright 2003-2004 Wind River Systems, Inc.
#
# modification history
# --------------------
# 01f,12may04,fle  Fixed a typo in error reporting of prjFsFile::save
# 01e,06may04,fle  Removed all bullet lists refgen markups
# 01d,08mar04,fle  Made prjFsFile::handle check for the VFS_FILE tag
# 01c,11feb04,fle  Added prjFsFileCreate
# 01b,05feb04,fle  Do not create default buildrule in prjFsFileAdd
#                  + made it save information to a .vfs file, not in the project
#                    file anymore
# 01a,15dec03,fle  written

# DESCRIPTION
# This Tcl library defines the Kernel Configuration File System File library
#
# A target file has three elements in this library :
# \is
# \i 'targetPath' : the file path on the target filesystem
# \i 'hostPath' : the file path on the host file system
# \i 'buildRule' : the rule to call at filesystem creation time
# \ie
#

package provide OsConfig 1.0

namespace eval prjFsFile {

    variable mFileName		""	;# path to the .vfs file
    variable mHProj		""	;# current project file handler
    variable mProjChanged	0	;# keeps track of projec file changes

    # XXX : fle : to be used in a later version to cache data

    variable mUpdated		0	;# keeps track of vfs file updates
    variable mTargetNames	{}	;# list of target file names
    variable mFsFileList	{}	;# list of fs files

    namespace export	filename handle _read save
}

################################################################################
#
# prjFsFileAdd - Add a file to the kernel file system
#
# SYNOPSIS
# \cs
#   prjFsFileAdd <hProj> <targetPath> <hostPath> [<buildrule>]
# \ce
#
# DESCRIPTION
# Add a file to the Kernel file system.
#
# The given <targetPath> is added to the list of files of the kernel file
# system.
#
# If a <buildrule> is given, the build rule will be called if the given
# <hostPath> does not exist when calling 'prjFsFileCreate'
#
# PARAMETERS
#   hProj : handle of the project to add file to file system
#   targetPath : path to the file to add to the kernel file system
#   hostPath : path of the file on the host file system
#   buildrule : build rule to call whenever the host file is not there
#
# RETURNS: The target path of the so-added file, or nothing if the file was
# already on the filesystem
#
# ERRORS: N/A
#
# SEE ALSO: prjFsFileRemove, prjFsFileListGet, prjFsFileCreate
#

proc prjFsFileAdd {hProj targetPath hostPath {buildrule ""}} {

    prjHandleValidCheck $hProj
    prjFsFile::handle	$hProj

    set fsFileList	[prjFsFile::_read]

    # check that the given targetPath is not already there

    foreach fsFile $fsFileList {
	if {[lindex $fsFile 0] == $targetPath} {
	    return
	}
    }

    if {$buildrule == ""} {
	lappend fsFileList	[list $targetPath $hostPath]
    } else {
	lappend fsFileList	[list $targetPath $hostPath $buildrule]
    }

    prjFsFile::save $fsFileList

    return $targetPath
}

################################################################################
#
# prjFsFileRemove - remove a File System File, and its associated information
#
# SYNOPSIS
# \cs
#   prjFsFileRemove <hProj> <targetPath>
# \ce
#
# DESCRIPTION
# Remove a file from the Kernel File System. The given <targetPath> is removed
# from the Kernel File System configuration along with all the associated
# information associated with it.
#
# If <targetPath> does not exits, this procedure exits on error with the
# 'no such file' error message (see the 'ERROR' section).
#
# PARAMETERS
#   hProj : handle of the project <targetPath> belongs to
#   targetPath : name of the file to remove from the Kernel File System
#
# RETURNS: N/A
#
# ERRORS
# \is
# \i prjFsFileRemove : could not find file <targetPath>
# \ie
#
# SEE ALSO: prjFsFileLib, prjFsFileAdd, prjFsFileListGet
#

proc prjFsFileRemove {hProj targetPath} {

    prjHandleValidCheck	$hProj
    prjFsFile::handle	$hProj

    # open the vfs file, and do not re-add the fs file to be removed

    set fsFiles		[prjFsFile::_read]
    set fileList	{}
    set fileRemoved	0

    foreach fsFile $fsFiles {
	if {[lindex $fsFile 0] != $targetPath} {
	    lappend fileList	$fsFile
	} else {
	    set fileRemoved	1
	}
    }

    if {$fileRemoved} {
	prjFsFile::save $fileList
    } else {
        error "prjFsFileRemove : could not find file \"$targetPath\""
    }
}

################################################################################
#
# prjFsFileListGet - get the list of files in the kernel file system
#
# SYNOPSIS
# \cs
#   prjFsFileListGet <hProj> [<pattern>]
# \ce
# DESCRIPTION
# Get the list of all files in the kernel file system which name matches
# <pattern>.
#
# If <pattern> is omitted, then all files from the kernel file system are
# returned
#
# PARAMETERS
#   hProj : handle of the kernel configuration project
#   pattern : file name to match. Default is '.*'
#
# RETURNS: The list of files in the kernel file system of the project handled by
# <hProj>
#
# SEE ALSO: prjFsFileLib, prjFsFileAdd, prjFsFileRemove
#

proc prjFsFileListGet {hProj {pattern ".*"}} {

    prjHandleValidCheck	$hProj
    prjFsFile::handle	$hProj

    # get the file content

    set fsFileList	[prjFsFile::_read]
    set targetFileList	{}

    foreach fsFile $fsFileList {
	set targetPath	[lindex $fsFile 0]

	if {[regexp $pattern $targetPath]} {
	    lappend targetFileList $targetPath
	}
    }

    return $targetFileList
}

################################################################################
#
# prjFsFileGet - get given target filesystem file values
#
# SYNOPSIS
# \cs
#   prjFsFileGet <hProj> <targetPath>
# \ce
#
# DESCRIPTION
# Get the given <targetPath> values. A List made of the file's <targetPath>
# and <hostPath> is returned.
#
# If the given file had a build rule, the buils rule is also returned as
# the last element of the list
#
# PARAMETERS:
#   hProj : handle of the project to get filesystem file from
#   targetPath : target file path to get
#
# RETURNS: A list of the target file values, or an error if the file was not
# found
#
# ERRORS:
# \is
# \i prjFsFileGet : Could not find "<targetPath>"
# \ie
#
# SEE ALSO : prjFsFileList, prjFsFileAdd, prjFsFileRemove
#

proc prjFsFileGet {hProj targetPath} {

    prjHandleValidCheck	$hProj
    prjFsFile::handle	$hProj

    # get all the files

    set fsFileList	[prjFsFile::_read]

    foreach fsFile $fsFileList {
	if {[lindex $fsFile 0] == $targetPath} {
	    return $fsFile
	}
    }

    error "prjFsFileGet : Could not find \"$targetPath\" ([prjFsFile::filename]\
	   contains : $fsFileList)"
}

proc prjFsFileCreate {hProj} {

    prjHandleValidCheck	$hProj
    prjFsFile::handle	$hProj

    set fsFileList	[prjFsFile::_read]
    set warnings	{}

    foreach fsFile $fsFileList {

	set targetPath	[lindex $fsFile 0]
	set hostPath	[lindex $fsFile 1]
	set buildRule	[lindex $fsFile 2]

	if {! [file exists $hostPath]} {

	    # try to use the build rule

	    if {$buildRule != ""} {

		if {[catch {eval exec $buildRule} status]} {

		    lappend warnings "Warning : Could not build host file\
				      \"$hostPath\" using build rule\
				      \"$buildRule\""
		}
	    }
	}

	# chek again if host file is there, it may have been generated by build
	# rule

	if {! [file exists $hostPath]} {

	    lappend warnings "Warning : Could not find host file \"$hostPath\""

	} else {

	    # the file can be copied to the romfs dir. Build the target
	    # file path first

	    set destDir	[file join [file dirname [prjFsFile::filename]] romfs]

	    if {[file pathtype $destDir] == "relative"} {
		set destDir [file join [pwd] $destDir]
	    }

	    regsub -all -- {/./} $destDir {/} destDir

	    if {[file pathtype $targetPath] != "relative"} {

		# file is volume relative or absolute, remove the first element
		# to be able to use [file join ...] on it

		set splitted [file split $targetPath]
		set splitted [lreplace $splitted 0 0]

		set targetPath [eval file join $splitted]
	    }

	    set destPath	[file join $destDir $targetPath]

	    # create parent dir if possible

	    if {! [file isdirectory [file dirname $destPath]]} {
		catch {file mkdir [file dirname $destPath]}
	    }

	    if {[catch {file copy -force $hostPath $destPath} status]} {

		lappend warnings "Warning : Could not copy \"$hostPath\" to\
				  \"$destPath\" : \"$status\""
	    }
	}
    }

    return $warnings
}

# ---------------------- prjFsFile namespace procedures ---------------------- #

################################################################################
#
# prjFsFile::handle - get or set the project file handle of the current project
#
# SYNOPSIS
# \cs
#   prjFsFile::handle [<hProj>]
# \ce
#
# PARAMETERS:
#   hProj : new project handle
#
# RETURNS: The current project handle
#
# NOMANUAL
#

proc prjFsFile::handle {{hProj ""}} {

    variable mHProj
    variable mProjChanged

    if {($hProj != "") && ($hProj != $mHProj)} {

	set mHProj		$hProj
	set mProjChanged	1
    }

    return $mHProj
}

################################################################################
#
# prjFsFile::filename - get the kernel filesystem file name
#
# SYNOPSIS
# \cs
#   prjFsFile::filename
# \ce
#
# DESCRIPTION
# If the current project handle changed, the kernel filesystem file name is
# updated. It should be the same file as the project file, except that the
# extension is '.vfs' instead of '.wpj'
#
# RETURNS: The file name of the kernel filesystem file
#
# NOMANUAL
#

proc prjFsFile::filename {} {

    variable mFileName
    variable mProjChanged

    if {($mProjChanged)} {

	set hProj	[prjFsFile::handle]

	# check for the VFS_FILE tag

	if {[set mFileName [prjTagDataGet $hProj VFS_FILE]] == ""} {

	    set prjFileName	[prjInfoGet [prjFsFile::handle] fileName]
	    set mFileName	[format "%s.vfs" [file rootname $prjFileName]]

	    prjTagDataSet $hProj VFS_FILE \
			  [file join [file dirname $prjFileName] $mFileName]
	}

	set mProjChanged	0
    }

    return $mFileName
}

################################################################################
#
# prjFsFile::_read - get the list of target filesystem files
#
# SYNOPSIS
# \cs
#   prjFsFile::_read
# \ce
#
# DESCRIPTION
# Read the filesystem file (as returned by 'prjFsFile::filename), and build
# the list of target filesystem files from it.
#
# Any line of the filesystem file which does not contain at list two elements
# (which should be the target file path, and the host file path), are ignored.
#
# PARAMETERS: N/A
#
# RETURNS: The list of filesystem files. Each element of the list contains
# at least two elements. First element is the target file path, while second
# element is the host file path. If a third element is present, it is the
# build rule to call a filesystem creation time
#
# NOMANUAL
#

proc prjFsFile::_read {} {

    set fsFileList	{}

    if {! [catch {open [prjFsFile::filename] r} fileId]} {

	set fileContent [split [read $fileId] "\n"]
	catch {close $fileId}

	# remove empty elements and elements which have less than 2 elements


	foreach line $fileContent {
	    if {([llength $line] >= 2)} {
		lappend fsFileList [string trim $line]
	    }
	}
    }

    return $fsFileList
}

################################################################################
#
# prjFsFile::save - save the given <fileList> in the filesystem file
#
# SYNOPSIS
# \cs
#   prjFsFile::save <fileList>
# \ce
#
# DESCRIPTION
# Save the given <fileList> in the filesystem file.
#
# Any subsequent call to 'prjFsFile::_read' will return this <fileList> unless
# this procedure failed.
#
# PARAMETERS:
#   fileList : target filesystem file list to save
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i prjFsFile : error writing "<filename>" : <error>
# \ie
#
# EXAMPLES: N/A
#
# SEE ALSO: prjFsFile::_read, prjFsFile::filename
#
# NOMANUAL
#

proc prjFsFile::save {fileList} {

    if {[catch {open [prjFsFile::filename] w} fileId]} {
	error "prjFsFile : error writing \"[prjFsFile::filename]\" : $fileId"
    }

    foreach fsFile $fileList {

	set targetPath	[lindex $fsFile 0]
	set hostPath	[lindex $fsFile 1]
	set buildRule	[lindex $fsFile 2]

	if {$buildRule == ""} {
	    puts $fileId [list $targetPath $hostPath]
	} else {
	    puts $fileId [list $targetPath $hostPath $buildRule]
	}
    }

    catch {close $fileId}
}
