# prjXmlLib.tcl - project tool files xml parser

# Copyright 2003 Wind River Systems, Inc.

# modification history
# --------------------
# 01c,26nov03,fle  Fixed error message when opening file failed in _parse
#                  + fixed if statements not using curlies
#                  + made regsub calls 8.3 compatible
#                  + fixed &quote; parsing
#                  + fixed BUILDRULE against BUILD_RULE tags errors
# 01b,25nov03,p_b  fixed quote problem in DEFAULT_BOOT_LINE
# 01a,19nov03,p_b  written

#
# DESCRIPTION
#
# This library stores all the procedures required to save the project tool 
# internal data to an xml file. You will also find methods to parse a project
# tool xml file.
#
# The public output method is wpjToXml <oldTag> <curTag> <wpjData>
# The public input method is prjXmlOpen <prjFile>
#
# XML tags are really long so some lines may have more than 80 characters. 
#

package require xml
package require struct

namespace eval ::wpj2xml {}

namespace eval ::wpj2xml::prjXmlLib {

    variable mCounter 0

    variable mCommands \
	[ \
	list \
	"parse" \
	]

    namespace export prjXmlLib
}

################################################################################
#
# ::wpj2xml::prjXmlLib::prjXmlLib - Xml Analyser constructor
#
# DESCRIPTION
# This function initialises the xml parser
#
proc ::wpj2xml::prjXmlLib::prjXmlLib {} {

    variable mCounter
    set name	"prjXmlLib"

    # first calculate a possible wpj2xml name

    incr mCounter
    set name [format "%s%i" "prjXmlLib" $mCounter]

    # check if it is a valid name

    if {[llength [info commands ::$name]]} {

	error "Command \"$name\" already exists, unable to create prjXmlLib"
    }

    # set up the name space

    namespace eval ::wpj2xml::prjXmlLib::prjXmlLib$name {

	variable mPath		""
	variable mFileContent	""
	variable mTree		[::struct::tree]
    }

    # create the command to manipulate the wpj2xml

    interp alias {} ::$name {} ::wpj2xml::prjXmlLib::prjXmlLibProc $name

    return $name
}

# ---------------------------- private procedures ---------------------------- #

################################################################################
#
# ::wpj2xml::prjXmlLib::stringAttributesDump - dump a list of string attributes
#               
# SYNOPSIS
# \cs
#   stringAttributesDump <wpjData>
# \ce
# 
# DESCRIPTION
# This function dumps a list of strings into a list of 
#     <stringAttribute value="data"> tags
#

proc ::wpj2xml::prjXmlLib::stringAttributesDump {wpjData} {
    set result ""
    set sA stringAttribute

    foreach element "${wpjData}" {
	set element [lindex [string trim "$element"] 0]
	set result "${result}\n\t<${sA} value=\"$element\"/>"
    }
    return ${result}
}

################################################################################
#
# ::wpj2xml::prjXmlLib::wpjToXml - convert tcl tags into xml tags
#
# SYNOPSIS
# \cs
#   wpjToXml <oldTag> <curTag> <wpjData>
# \ce
#
# DESCRIPTION
# This function converts a tcl tag+data into an xml statement.
# The previous tag (oldTag) is also passed to this function, this is required
# to close <listAttribute></lisAttribute> blocs.
#

proc ::wpj2xml::prjXmlLib::wpjToXml {oldTag curTag wpjData} {

    set key ""
    set clListAtt ""

    # different XML tags

    set CA ConfigurationAttributes
    set sA stringAttribute
    set lA listAttribute

    if {[regexp BUILD_.*_MACRO_.* $curTag]} {

	regsub BUILD_.*_MACRO_ "$curTag" "" macro

	if {[regexp BUILD_.*_MACRO_.* $oldTag]} {

	    return "\t<${sA} key=\"$macro\" value=\"$wpjData\"/>"

	} else {

	    regsub BUILD_ $curTag "" spec
	    regsub _MACRO_.* $spec "" spec

	    set key "${CA}::BuildInfo::${spec}::Macro"

	    return [format "<%s key=\"%s\">\n\t<%s key=\"%s\" value=\"%s\"/>" \
			   $lA $key $sA $macro $wpjData]
	}
    }

    if {[regexp WCC_.*_PARAM_.* $curTag]} {

	regsub WCC_.*_PARAM_ $curTag "" param

	if {[regexp WCC_.*_PARAM_.* $oldTag]} {

	    # format wpjData to be XML compatible

	    regsub -all {\"} $wpjData "\\&quot;" wpjData
	    regsub -all "<" $wpjData "\\&lt;" wpjData
	    regsub -all ">" $wpjData "\\&gt;" wpjData

	    return "\t<${sA} key=\"$param\" value=\"${wpjData}\"/>"

	} else {

	    regsub WCC_ "$curTag" "" wcc
	    regsub _PARAM_.* "$wcc" "" wcc
	    set key "${CA}::ProjectInfo::Components::Bsp::${wcc}::params"

	    return [format "<%s key=\"%s\">\n\t<%s key=\"%s\" value=\"%s\"/>" \
			   $lA $key $sA $param $wpjData]
	}
    }

    # close the <listAttribute> bloc

    if {[regexp BUILD_.*_MACRO_.* $oldTag] || \
	[regexp WCC_.*_PARAM_.* $oldTag]} {

	set clListAtt "</${lA}>\n"
    }

    if {[regexp WCC_.*_COMPONENTS "$curTag"]} {

	regsub WCC_ $curTag "" wcc
	regsub _COMPONENTS $wcc "" wcc
	set key "${CA}::ProjectInfo::Components::Bsp::${wcc}::names"

	return [format "%s<%s key=\"%s\">%s\n</%s>" $clListAtt $lA $key \
		       [stringAttributesDump $wpjData] $lA]
    }

    if {[regexp BUILD_.*_RO_DEPEND_PATH $curTag]} {

	regsub BUILD_ $curTag "" buildspec
	regsub _RO_DEPEND_PATH $buildspec "" buildspec

	set key [format \
		 "%s::BuildInfo::%s::BuildRule::object_file::RoDependPath" $CA \
		 $buildspec]

	return [format "%s<%s key=\"%s\">%s\n</%s>" $clListAtt $lA $key \
		       [stringAttributesDump $wpjData] $lA]
    }

    switch -exact $curTag {
	"BUILD__LIST" { set key "${CA}::BuildInfo::BuildSpecList" }
	"PROJECT_FILES" { set key "${CA}::ProjectInfo::Files" }
	"WCC__CDF_PATH" { set key "${CA}::ProjectInfo::Components::cdfPath" }
	"WCC__MXR_LIBS" { set key "${CA}::ProjectInfo::Components::mxrLibs" }
	"WCC__LIST" { set key "${CA}::ProjectInfo::Components::BspList" }
    }

    if { "$key" != "" } {
	return [format "%s<%s key=\"%s\">%s\n</%s>" $clListAtt $lA $key \
		       [stringAttributesDump $wpjData] $lA]
    }

    if {[regexp BUILD_.*_TC "$curTag"]} {

	regsub BUILD_ $curTag "" buildspec
	regsub _TC $buildspec "" buildspec
	set key "${CA}::BuildInfo::${buildspec}::Tc"
	return "${clListAtt}<${sA} key=\"$key\" value=\"$wpjData\"/>"
    }

    if {[regexp BUILD_.*_BUILDRULE "$curTag"]} {

	regsub BUILD_ $curTag "" buildspec
	regsub _BUILDRULE $buildspec "" buildspec
	set key "${CA}::BuildInfo::${buildspec}::BuildRule::object_file::Rule"

	# some < or > could be found in the rules

	regsub -all {\"} $wpjData "\\&quot;" wpjData
	regsub -all "<" $wpjData "\\&lt;" wpjData
	regsub -all ">" $wpjData "\\&gt;" wpjData

	return "${clListAtt}<${sA} key=\"$key\" value=\"$wpjData\"/>"
    }

    if {[regexp BUILD_RULE_.* "$curTag"]} {

	regsub BUILD_RULE_ $curTag "" rule
	set key "${CA}::BuildInfo::Rule::${rule}::Rule"

	# some < or > could be found in the rules

	regsub -all {\"} $wpjData "\\&quot;" wpjData
	regsub -all "<" $wpjData "\\&lt;" wpjData
	regsub -all ">" $wpjData "\\&gt;" wpjData

	return "${clListAtt}<${sA} key=\"$key\" value=\"$wpjData\"/>"
    }

    if {[regexp FILE_.*_.* $curTag]} {

	regsub FILE_ $curTag "" file
	regsub _\[^_\]*\$ $file "" file
	regsub FILE_.*_ $curTag "" info

	set key "${CA}::ProjectInfo::File::${file}::${info}"

	if {$info != "dependencies" } {
	    return "${clListAtt}<${sA} key=\"$key\" value=\"$wpjData\"/>"
	} else {
	    return "${clListAtt}<${lA} key=\"$key\">[stringAttributesDump\
		    ${wpjData}]\n</${lA}>"
	}
    }

    # straight forward tags

    switch -exact "$curTag" {

	"BUILD__CURRENT" {set key "${CA}::BuildInfo::CurrentBuildSpec"}
	"CORE_INFO_TYPE" {set key "${CA}::ProjectInfo::CoreInfo::Type"}
	"CORE_INFO_VERSION" {set key "${CA}::ProjectInfo::CoreInfo::Version"}
	"CORE_INFO_SUBTYPE" {set key "${CA}::ProjectInfo::CoreInfo::SubType"}
	"BSP_DIR" {set key "${CA}::ProjectInfo::BspDir"}
	"WCC__CURRENT" {set key "${CA}::ProjectInfo::Components::CurrentBsp"}

	"COMPONENT_ERROR_STATUS" {
	    set key "${CA}::ProjectInfo::Components::errorStatus"
	}

	"userComments" {set key "${CA}::ProjectInfo::Components::${curTag}"}

	# must assume user comment ?

	default {set key "UNKNOWN ${curTag}" }
    }

    return "${clListAtt}<${sA} key=\"$key\" value=\"$wpjData\"/>"
}

################################################################################
#
# ::wpj2xml::prjXmlLib::prjXmlLibProc - procedure wrapper for all prjXmlLib
#                                          procs
#
# SYNOPSYS:
#   ::wpj2xml::prjXmlLib::prjXmlLibProc <name> [<cmd>] <args>
#

proc ::wpj2xml::prjXmlLib::prjXmlLibProc {name {cmd ""} args} {

    # Do minimal args checks here

    if { [llength [info level 0]] == 2 } {
	error "wrong # args: should be \"$name option ?arg arg ...?\""
    }
    
    # Split the args into command and args components

    if {[llength [info commands ::wpj2xml::prjXmlLib::_$cmd]] == 0} {

	variable mCommands

	set optlist [join $mCommands ", "]
	set optlist [linsert $optlist "end-1" "or"]
	error "bad option \"$cmd\": must be $optlist"
    }

    eval [list ::wpj2xml::prjXmlLib::_$cmd $name] $args
}

################################################################################
#
# ::wpj2xml::prjXmlLib::_parse - parse one ar several xml files
#

proc ::wpj2xml::prjXmlLib::_parse {name {path ""}} {

    upvar ::wpj2xml::prjXmlLib::prjXmlLib${name}::mPath	mPath
    upvar ::wpj2xml::prjXmlLib::prjXmlLib${name}::mFileContent mFileContent
    upvar ::wpj2xml::prjXmlLib::prjXmlLib${name}::mTree	mTree

    # first check that there is a valid file path

    if {($mPath == "") && ($path == "")} {

	error "No path set to this prjXmlLib, please provide with a <path>\
	       ..."
    }

    # now set the mPath if path is not null

    if {$path != ""} {

	set mPath		$path
    }

    unset	mFileContent
    set		mFileContent ""

    foreach filePath $mPath {

	if {[catch {open $filePath r} fileId]} {

	    error "Error while opening file \"$filePath\" : \"$fileId\""
	}

	set mFileContent [read $fileId]

	close $fileId

	# now we can parse this file

	set parser \
	    [ \
	    xml::parser \
	    -elementstartcommand "::wpj2xml::prjXmlLib::elementStart" \
	    -elementendcommand "::wpj2xml::prjXmlLib::elementEnd" \
	    ]
    }

    $parser parse $mFileContent
}

# ------------------------ parser related procedures ------------------------- #

################################################################################
#
# ::wpj2xml::prjXmlLib::store - store key data
#
# SYNOPSIS
# \cs
#   store <key> <data>
# \ce
#
# DESCRIPTION
# This function fills in the hArray array. The storeEx version does not reset
# listkey and listvalue, i.e. we are still in a <listAttribute></listAttribute>
# bloc.
#

global listkey listvalue bodylist hArray

# key xml tag for a <listAttribute key=... > statement.
set listkey ""

# data concatenated for a list of <stringAttribute value=... > statements.
set listvalue ""

# are we in a <listAttribute></listAttribute> statement
set bodylist 0

proc ::wpj2xml::prjXmlLib::storeEx {key data} {

    global hArray

    regsub -all {\n} $data "" data
    regsub -all {\\} $data "" data

    #puts stdout "$key = $data"

    set hArray($key) $data
}

proc ::wpj2xml::prjXmlLib::store {key data} {

    global hArray listkey listvalue

    regsub -all {\n} $data "" data
    regsub -all {\\} $data "" data

    #puts stdout "$key = $data"

    set hArray($key) $data

    set listkey ""
    set listvalue ""
}

################################################################################
#
# ::wpj2xml::prjXmlLib::elementStart - Xml element start hook
# 
# SYNOPSIS
# \cs
#   elementStart <tagname> <attlist> <args>
# \ce
#

proc ::wpj2xml::prjXmlLib::elementStart {tagname attlist args} {

    global listkey bodylist listvalue

    set CA ConfigurationAttributes
    set sA stringAttribute
    set lA listAttribute

    # parse <stringAttribute key= value= > tags
    if {[regexp ".* key .*" "$attlist"]} {
	set key [lindex "$attlist" 3] 

	if {[regexp "^value .*" "$attlist"]} {
	    set value [lindex "$attlist" 1]
	}

	if {$bodylist} {
	    if {[regexp "${CA}::BuildInfo::.*::Macro" $listkey]} {

		regsub ConfigurationAttributes::BuildInfo:: $listkey "" \
		       buildspec

		regsub ::Macro $buildspec "" buildspec
		storeEx BUILD_${buildspec}_MACRO_${key} "$value"
		return
	    }

	    if {[regexp "${CA}::ProjectInfo::Components::Bsp::.*::params" \
			$listkey]} {

		regsub ConfigurationAttributes::ProjectInfo::Components::Bsp:: \
		       $listkey "" wcc
		regsub ::params $wcc "" wcc
		storeEx WCC_${wcc}_PARAM_${key} $value
		return
	    }
	}

	if {[regexp "${CA}::BuildInfo::.*::Tc" $key]} {

	    regsub ConfigurationAttributes::BuildInfo:: "$key" "" buildspec
	    regsub ::Tc "$buildspec" "" buildspec
	    store BUILD_${buildspec}_TC "$value"
	    return
	}

	if {[regexp "${CA}::BuildInfo::.*::BuildRule::object_file::Rule" \
		    $key]} {

	    regsub ConfigurationAttributes::BuildInfo:: $key "" buildspec
	    regsub ::BuildRule::object_file::Rule:: $buildspec "" buildspec
	    store BUILD_${buildspec}_BUILDRULE $value
	    return
	}

	if {[regexp "${CA}::BuildInfo::Rule::.*::Rule" $key]} {
	    regsub ConfigurationAttributes::BuildInfo::Rule:: $key "" rule
	    regsub ::Rule $rule "" rule
	    store BUILD_RULE_${rule} $value
	    return
	}

	if {[regexp "${CA}::ProjectInfo::File::.*::.*" $key]} {
	    regsub ConfigurationAttributes::ProjectInfo::File:: $key "" file
	    regsub ::.* $file "" file
	    regsub ConfigurationAttributes::ProjectInfo::File::.*:: $key "" info
	    store FILE_${file}_${info} $value
	    return
	}

	switch -exact $key {

	    "ConfigurationAttributes::BuildInfo::CurrentBuildSpec" {
		store BUILD__CURRENT $value
	    }

	    "ConfigurationAttributes::ProjectInfo::CoreInfo::Type" {
		store CORE_INFO_TYPE $value
	    }

	    "ConfigurationAttributes::ProjectInfo::CoreInfo::Version" {
		store CORE_INFO_VERSION $value
	    }

	    "ConfigurationAttributes::ProjectInfo::CoreInfo::SubType" {
		store CORE_INFO_SUBTYPE $value
	    }

	    "ConfigurationAttributes::ProjectInfo::BspDir" {
		store BSP_DIR $value
	    }

	    "ConfigurationAttributes::ProjectInfo::Components::CurrentBsp" {
		store WCC__CURRENT $value
	    }

	    "ConfigurationAttributes::ProjectInfo::Components::errorStatus" {
		store COMPONENT_ERROR_STATUS $value
	    }

	    "ConfigurationAttributes::ProjectInfo::Components::userComments" {
		store userComments $value
	    }
	}

    } else {

	# parse <listAttribute key= > tags

	if {[regexp "^key .*" "$attlist"]} {

	    set key [lindex "$attlist" 1]

	    if {$tagname == "listAttribute" } {
		set listkey $key
		set bodylist 1
		return
	    }

	} else {

	    # parse <stringAttribute value= > tags

	    if {[regexp "^value .*" "$attlist"]} {
		set value [lindex $attlist 1]
		set listvalue [concat $listvalue $value]
	    }
	}
    }
}

################################################################################
#
# ::wpj2xml::prjXmlLib::elementEnd - Xml element end hook
#
# SYNOPSIS
# \cs
#   elementEnd <tagname> <args>
# \ce
#

proc ::wpj2xml::prjXmlLib::elementEnd {tagname args} {

    global listvalue listkey bodylist

    set ca	ConfigurationAttributes

    # parse </listAttribute> tag

    if {$tagname == "listAttribute"} {

	set bodylist 0

	if {[regexp \
	     "${ca}::BuildInfo::.*::BuildRule::object_file::RoDependPath" \
	     "$listkey"]} {

	    regsub ${ca}::BuildInfo:: $listkey "" buildspec
	    regsub ::BuildRule::object_file::RoDependPath $buildspec "" \
		   buildspec
	    store BUILD_${buildspec}_RO_DEPEND_PATH "$listvalue"
	    return
	}

	if {[regexp "${ca}::ProjectInfo::Components::Bsp::.*::names"\
	     "$listkey"]} {

	    regsub ConfigurationAttributes::ProjectInfo::Components::Bsp:: \
		   $listkey "" wcc
	    regsub ::names $wcc "" wcc
	    store WCC_${wcc}_COMPONENTS "$listvalue"
	    return
	}

	if {[regexp "${ca}::ProjectInfo::File::.*::dependencies" $listkey]} {

	    regsub ConfigurationAttributes::ProjectInfo::File:: $listkey "" \
		   file
	    regsub ::dependencies $file "" file
	    store FILE_${file}_dependencies "$listvalue"
	    return
	}

	switch -exact "$listkey" {

	    "ConfigurationAttributes::ProjectInfo::Files" {
		store PROJECT_FILES $listvalue
	    }

	    "ConfigurationAttributes::BuildInfo::BuildSpecList" {
		store BUILD__LIST $listvalue
	    }

	    "ConfigurationAttributes::ProjectInfo::Components::cdfPath" {
		store WCC__CDF_PATH $listvalue
	    } 

	    "ConfigurationAttributes::ProjectInfo::Components::mxrLibs" {
		store WCC__MXR_LIBS $listvalue
	    }

	    "ConfigurationAttributes::ProjectInfo::Components::BspList" {
		store WCC__LIST $listvalue
	    }
	}
    }
}

################################################################################
#
# prjXmlOpen - open an xml file and store its content in an array
#
# SYNOPSIS
# \cs
#   prjXmlOpen <fProg>
# \ce
#   
# DESCRIPTION
# This function opens an xml file and puts all the information in an array.
# The array entries are of the form array(wpjkey)=wpjdata
#

proc prjXmlOpen {fProj} {

    global hArray

    # shortcut to xml parser
    set xmlAna [::wpj2xml::prjXmlLib::prjXmlLib]

    # parse all xml files
    $xmlAna parse $fProj

    # return array as list of pairs
    if {[info exists hArray]} {
	return [array get hArray]
    }
}
