# tcInfoGen.tcl - generate info on the GNU tool family
#
# modification history
# --------------------
# 02h,15sep05,pee  fixed cPrepend for simlinux and simpc
# 02g,23dec04,c_c  removed references to foundation.
# 02f,22sep04,fle  Create the toolchain info file directory if needed
# 02e,06sep04,fle  Made it inside a namespace to avoid global variables
#                  override
# 02d,01sep04,fle  Made it use Wind::foundation when needed
# 02c,25jun04,fle  SPR#98486 : do not generate tcInfo files when no toolchain
#                  is found
# 02b,11may04,fle  Fixed ldpartial called by oldDependFiles
# 02a,10may04,fle  Special cases for NM and SIZE macros with gnu toolchain
# 01z,31mar04,fle  Fixed Wind::which use
#                  + made tools path use Wind::tcPath
#                  + made a tc file info generation check
# 01y,25feb04,fle  Renamed gnuInfoGen.tcl in tcInfoGen.tcl
# 01x,16feb04,fle  Made it use the PrjLib package
# 01w,26nov03,fle  Made it use tcInfoFileGet
# 01v,23sep03,fle  Fixed toolchain file generation syntax errors
# 01u,28feb03,jeg  remove SIMSPACSOLARIS specific definitions
# 01v,19jun03,fle  Updated to new location
# 01u,17feb03,fle  fixed Tcl believes DEFINE_CC is a call
# 01t,28jan03,rbl  check in makefiles rather than generating them - this
#                  fixes SPR 82348, Two different users can not start 
#                  Tornado on the same UNIX machine
# 01s,20may02,rbl  show status when building toolchain info at IDE startup
#                  time
# 01r,15may02,cjs  Fix spr 71679 -- toolchain info file generation should
#                  write to temporary files to writeable location
# 01q,30apr02,cym  adding HEX_FLAGS
# 01p,17dec01,rbl  make tcInfo_* production robust against diab not being
#                  installed
# 01o,05nov01,rbl  switch to -g, no optimization for project builds
# 01n,24oct01,rbl  get dependency generation working with diab compiler
# 01m,17sep01,rbl  merge changes from T3: (1) make generation of
#                  tcInfo_<family> more robust (2) add LD_PARTIAL (3) use make
#                  to extract CPU and TOOL (4) parse makemacros using regsub
#                  so that "windows hack" is not needed (5) make sure that
#                  absolute paths don't show up in tcInfo_<family>
# 01n,25sep01,dat  moved makefiles to h/tool/, renamed vendor to family
# 01m,17sep01,sn  reinstated 01i and 01k (initial problems fixed by 01j)
# 01l,31may01,kab  Removed -nostdinc from simulator
# 01k,28mar01,sn  Removed SIMNT hacks (no longer necessary)
# 01j,12mar01,sn  Spell -x assembler-with-cpp without the space to 
#                 ensure correct handling by cppFlagsGet
# 01k,19jan01,mem  Undo change 01i, breaks rebuild all from project facility.
# 01j,12jan01,mem rewrote detection loop to not assume TOOL=VENDOR, but
#		  just rather a subset.
# 01i,06dec00,sn  removed -xc from tool_cpp
# 01h,16mar99,pr  removed -Ox option from compiler flags (SPR 25729)
# 01g,26oct98,ms  reworked for diab integration.
# 01f,25aug98,ms  modified to output a table, instead of TCL code.
#		  that way 3rd party vendors don't have to learn TCL.
#		  made it work with non-gnu toolchains.
# 01e,11aug98,ren refined previous check in.  Now, nm called without -g
#                 option only for those three architectures.
# 01d,11aug98,ren removed -g option from nm commandline.
#                 necessary for PPC, SIMSOLARIS and 960 architectures,
#                 but shouldn't affect others.
# 01c,27jul98,ms  added nm, size, and ar utilities. Fixed sim flags parsing.
# 01b,25jun98,ms  combined C and C++ compiler into one tool. Added LDFLAGS.
# 01a,08apr98,ms  written.
#
# DESCRIPTION
# This library generates inforamation on the GNU (or other) tool family
# suitable for importing into tc_cplus.tcl (which exposes the
# information to the configuration tool).
#
# USAGE
# \cs
#   tclsh tcInfoGen.tcl [family]
# \ce
#

package require PrjUtils
package require Wind

# begin namespace ::toolInfoGenProgress

namespace eval ::toolInfoGenProgress {

    variable family	gnu

    proc progressInit {toolFamily} {
	variable family
	set family $toolFamily
    }

    proc progressSetTaskBeginText {message} {
	puts -nonewline $message
	flush stdout
    }

    proc progressSetTaskEndText {message} {
	puts $message
	flush stdout
    }

    proc progressUpdate {percent} {
	puts -nonewline "."
	flush stdout
    }

    proc progressErrorReport {message} {
	puts -nonewline "\n\[$message\]\n"
	flush stdout
    }

    proc progressFinish {} {
    }
}

namespace eval TcInfoGen {

    namespace export init
}

proc TcInfoGen::init {argc argv} {

    global env

    if {$argc == 1} {
	set family [lindex $argv 0]
    } else {
	set family gnu
    }

    # Save the tc info in a temp file. If the temp file already exists, it means
    # that another process is doing it too. Just wait for the other process to
    # end. If the temp file size does not change after one minute, consider the
    # other process is dead, and do it ourself

    set toolchainInfoFile	[tcInfoFileGet $family]
    set tmpTcFile		[format "%s.temp" $toolchainInfoFile]
    set sizeChanges		1
    set unchangeCount		0
    set tcFileDone		0

    # check if the toolchain info file directory exists

    if {! [file exists [file dirname $tmpTcFile]]} {
	if {[catch {file mkdir [file dirname $tmpTcFile]} mkdirStatus]} {
	    puts [prjStderr] "Could not create [file dirname $tmpTcFile]\
			      directory : \"$mkdirStatus\""
	}
    }

    if {[file exists $tmpTcFile]} {

	set tcFileDone	1
	set oldSize		[file size $tmpTcFile]
	set newSize		$oldSize

	while {[file exists $tmpTcFile] && ($sizeChanges)} {

	    set newSize	[file size $tmpTcFile]

	    if {$oldSize != $newSize} {
		set oldSize	$newSize
	    } else {
		incr unchangeCount
		if {$unchangeCount > 5} {
		    set tcFileDone	0
		    set sizeChanges	0
		}
	    }
	    after 10000
	}
    }

    # toolchain file has been generated by someone else

    if {$tcFileDone} {
	return
    } else {

	file delete -force $toolchainInfoFile

	set fd		[open $tmpTcFile w+]
	fconfigure $fd -translation lf
	puts $fd "# [file tail $toolchainInfoFile] - generated by\
		  tcInfoGen.tcl\n\n"
	puts $fd "package require Wind 1.1\n"
	close $fd
    }

    # if this is called from a higher level makefile that had CPU and TOOL
    # passed in on the command line, this will mess up the toolchain information
    # gathering. These values may also be passed on through the environment
    # variable MAKEFLAGS. To avoid this we unset the environment variables CPU,
    # TOOL and MAKEFLAGS, and reset them when we are done

    array set saveEnv {}

    foreach envVar {CPU TOOL MAKEFLAGS ADDED_CFLAGS} {
	if {[info exists env($envVar)]} {
	    set saveEnv($envVar) $env($envVar)
	    set env($envVar) ""
	}
    }

    # use makefiles to get the default build
    # flags for each tool family

    set makefile [file join [Wind::base] host resource osconfig make Makefile]

    # determine the list of toolchains by looking for CPU= and TOOL= in files
    # make.*${family}*

    ::toolInfoGenProgress::progressInit $family
    ::toolInfoGenProgress::progressSetTaskBeginText "Finding $family toolchains"

    # finding toolchains takes roughly 25 % of the time ...

    set findPercent 25
    set extractPercent [expr 100 - {$findPercent}]

    set toolchainList ""
    set makeStubs [glob -nocomplain [file join [Wind::tgtDir] h tool \
					       ${family} make.*${family}*]]

    set nMakeStubs	[llength $makeStubs]
    set iMakeStub	0
    set cpuRegexp	"^CPU\[ \t]*=\[ \t]*(\[A-Za-z0-9]+)"
    set toolRegexp	"^TOOL\[ \t]*=\[ \t]*(\[A-Za-z0-9]+)"

    foreach makeStub $makeStubs {

	incr iMakeStub

	regsub -all "make." [file tail $makeStub] "" toolchain

	set tool ""
	set cpu ""

	if {![catch {open $makeStub r} fileId]} {

	    set fileContent [split [read $fileId] "\n"]
	    catch {close $fileId}

	    foreach line $fileContent {
		set line [string trim $line]
		if {[string first "#" $line] != 0} {
		    if {[regexp $cpuRegexp $line]} {
			regsub -all -- $cpuRegexp $line "\\1" cpu
		    }
		    if {[regexp $toolRegexp $line]} {
			regsub -all -- $toolRegexp $line "\\1" tool
		    }
		}
	    }
	}

	if {($cpu != "") && ($tool != "")} {
	    set cpuForToolChain($toolchain) $cpu
	    set toolForToolChain($toolchain) $tool
	    lappend toolchainList $toolchain
	    set percent [expr {double($iMakeStub) / $nMakeStubs * $findPercent}]
	    ::toolInfoGenProgress::progressUpdate [expr {int($percent)}]
	} else {
	    ::toolInfoGenProgress::progressErrorReport "skipping $makeStub\
							\n(does not define CPU\
							and TOOL)"
	}
    }

    ::toolInfoGenProgress::progressSetTaskEndText "Done"

    # use the Makefile to gather info on each cpu/tool pair

    ::toolInfoGenProgress::progressSetTaskBeginText "Getting $family toolchain\
						     information"
    set iMakeStub 0
    set nToolchains [llength $toolchainList]

    foreach toolchain $toolchainList {

	incr iMakeStub
	set cpu	$cpuForToolChain($toolchain)
	set tool	$toolForToolChain($toolchain)

	set percentMore [expr {double($iMakeStub) / $nToolchains * \
			      $extractPercent}]

	::toolInfoGenProgress::progressUpdate [ \
	    expr {int($findPercent + $percentMore)} \
	]

	# get all macros

	set lines [exec make -f $makefile makeMacros CPU=$cpu \
			     TOOL_FAMILY=${family} TOOL=$tool CC_INCLUDE= \
			     CC_DEFINES= ADDED_C++FLAGS=$\(DEFINE_CC\) \
			     TOOL_FAMILY=${family} BSP2PRJ=TRUE]

	# parse result of make command to extract macros

	foreach line [split $lines \n] {

	    set macro [lindex $line 1]

	    if {[regsub {[^=]*=} $line {} val] != 1} {
		continue
	    }

	    set val   [string trim $val]

	    switch -- $macro {

		CC	{
		    set tool_cc($toolchain) [lindex $val 0]
		    set tool_as($toolchain) [lindex $val 0]
		    set tool_cpp($toolchain) "[lindex $val 0] -E -P"
		}

		CPP	{
		    if {"$family" != "gnu"} {
			set tool_cpp($toolchain) "$val"
		    }
		}            

		CC_ARCH_SPEC {
		    set tool_cc_arch_spec($toolchain) "$val"
		}

		OPTION_DEPEND	{
		    set tool_depend($toolchain) "$val"
		}

		DEPEND_GEN_UTIL	{
		    set tool_depend_gen_util($toolchain) "$val"
		}

		LD	{
		    set tool_ld($toolchain) "$val"
		}

		LDFLAGS	{
		    set flags_ld($toolchain) "$val"
		}

		LD_PARTIAL	{
		    set tool_ldp($toolchain) "$val"
		}

		LD_PARTIAL_FLAGS	{
		    set flags_ldp($toolchain) "$val"
		}

		HEX_FLAGS {
		    set flags_hex($toolchain) "$val"
		}

		CFLAGS_PROJECT {
		    set flags_cc($toolchain) "$val"
		}

		CFLAGS_AS_PROJECT {
		    set flags_as($toolchain) "$val"
		}

		AR {
		    set util_ar($toolchain) "$val"
		}

		NM {
		    switch -glob $cpu { 
			SIMSPARC*  { set util_nm($toolchain) "$val" }
			PPC*       { set util_nm($toolchain) "$val" }
			*960*      { set util_nm($toolchain) "$val" }
			default   {set util_nm($toolchain) "$val -g"}
		    }
		}

		SIZE {
		    set util_size($toolchain) "$val"
		}
	    }
	}

	# When two concurrent processes are trying to generate the tc info file,
	# a check of the temp file size is done. Add one line to be sure the
	# file size is changed, and concurrent process will not try to override
	# job done by this process

	set fd		[open $tmpTcFile a]
	fconfigure $fd -translation lf
	puts $fd "# toolchain : $toolchain"
	file stat $tmpTcFile fileStat
	close $fd
    }

    ::toolInfoGenProgress::progressSetTaskEndText "Done"
    ::toolInfoGenProgress::progressFinish

    # write out tcInfo_${family}

    if {[Wind::isWindows]} {
	set fileExt ".exe"
	set pathSeparators ";"
    } else {
	set fileExt ""
	set pathSeparators ":;"
    }

    # re-open the temp file to dump all toolchain info

    set fd		[open $tmpTcFile a]
    fconfigure $fd -translation lf
    puts $fd ""

    set tcCount	0

    foreach toolchain [lsort $toolchainList] {

	set cpu $cpuForToolChain($toolchain)
	set tool $toolForToolChain($toolchain)

	set cc 	[format "%s%s" $tool_cc($toolchain) $fileExt]
	set ccPath	[file join [Wind::tcPath $family] bin $cc]

	# expose only the installed toolchains to the configuration tool

	if {![file exists [file join [Wind::tgtDir] h tool $family \
				     make.$cpu$tool]]} {
	    continue
	}

	# seems like some macros need to be added

	set md	"-MD"

	set str "set tcInfo_${toolchain}(TOOL_FAMILY) $family\n"
	append str "set tcInfo_${toolchain}(CC) \"$tool_cc($toolchain)\"\n"
	append str "set tcInfo_${toolchain}(CPP) \"$tool_cpp($toolchain)\"\n"
	append str "set tcInfo_${toolchain}(AS) \"$tool_as($toolchain)\"\n"
	append str "set tcInfo_${toolchain}(LD) \"$tool_ld($toolchain)\"\n"
	append str "set tcInfo_${toolchain}(LD_PARTIAL)\
		    \"$tool_ldp($toolchain)\"\n"
	append str "set tcInfo_${toolchain}(NM) \"$util_nm($toolchain)\"\n"
	append str "set tcInfo_${toolchain}(SIZE) \"$util_size($toolchain)\"\n"
	append str "set tcInfo_${toolchain}(AR) \"$util_ar($toolchain)\"\n"

	append str "set tcInfo_${toolchain}(CFLAGS) \"[PrjUtils::relPath\
		    $flags_cc($toolchain)]\"\n"
	append str "set tcInfo_${toolchain}(CFLAGS_AS) \"[PrjUtils::relPath\
		    $flags_as($toolchain)]\"\n"
	append str "set tcInfo_${toolchain}(LDFLAGS) \"[PrjUtils::relPath\
		    $flags_ld($toolchain)]\"\n"
	append str "set tcInfo_${toolchain}(LD_PARTIAL_FLAGS)\
		    \"[PrjUtils::relPath $flags_ldp($toolchain)]\"\n"
	append str "set tcInfo_${toolchain}(HEX_FLAGS) \"[PrjUtils::relPath\
		    $flags_hex($toolchain)]\"\n"
	append str "set tcInfo_${toolchain}(OPTION_INCLUDE_DIR) -I\n"
	append str "set tcInfo_${toolchain}(OPTION_DEFINE_MACRO) -D\n"
	append str "set tcInfo_${toolchain}(OPTION_UNDEFINE_MACRO) -U\n"
	append str "set tcInfo_${toolchain}(OPTION_LANG_C) -xc\n"
	append str "set tcInfo_${toolchain}(OPTION_GENERATE_DEPENDENCY_FILE)\
		    $md\n"

	if {[info exists tool_depend($toolchain)]} {
	    append str "set tcInfo_${toolchain}(OPTION_DEPEND)\
			\"$tool_depend($toolchain)\"\n"
	} else {
	    append str "set tcInfo_${toolchain}(OPTION_DEPEND) -M\n"
	}

	if {[info exists tool_cc_arch_spec($toolchain)]} {
	    append str "set tcInfo_${toolchain}(CC_ARCH_SPEC)\
			\"$tool_cc_arch_spec($toolchain)\"\n"
	} else {
	    append str "set tcInfo_${toolchain}(CC_ARCH_SPEC) \"\"\n"
	}

	puts $fd $str

	# C compiler

	set str "namespace eval ::tc_${toolchain}::tool_cc \{\n\n"
	append str "    proc type \{\} \{\n\treturn \"C/C++\
		    compiler\"\n    \}\n\n"
	append str "    proc inputSuffixes \{\} \{\n\treturn \".c .cxx .cpp .cc\
		    .C\"\n    \}\n\n"
	append str "    proc outputFiles \{inFile\} \{\n\treturn \[file\
		    rootname \[file tail \$inFile]].o\n    \}\n\n"

	append str "    proc buildCmds \{flags file\} \{\n\treturn \[list\
		    \"\\\$(CC) \$flags -c \$file\"]\n    \}\n\n"
	append str "    proc toolMacro \{\} \{\n\treturn CC\n    \}\n\n"
	append str "    proc flagsMacro \{\} \{\n\treturn CFLAGS\n    \}\n\}\n"

	puts $fd $str

	# assembler

	set str "namespace eval ::tc_${toolchain}::tool_as \{\n\n"
	append str "    proc type \{\} \{\n\treturn assembler\n    \}\n\n"
	append str "    proc inputSuffixes \{\} \{\n\treturn \".s\"\n    \}\n\n"
	append str "    proc outputFiles \{inFile\} \{\n\treturn \[file\
		    rootname \[file tail \$inFile]].o\n    \}\n\n"
	append str "    proc buildCmds \{flags file\} \{\n"
	append str "\tset tc \[namespace parent\]\n"
	append str "\tif \{\[\$\{tc\}::asmHasCpp\]\} \{\n"
	append str "\t    return \[list \"\\\$(AS) \$flags -c \$file -o\
		    \[outputFiles \$file]\"]\n\t\}\n"
	append str "\tglobal tcInfo_\[$\{tc\}::name]\n"
	append str "\tregsub -all -- \" $md\" \$flags \"\" cppFlags\n"
	append str "\tset tmpFile \[file rootname \$file]_tmp.s\n"
	append str "\treturn \[list \"\\\$(CPP) \$cppFlags \$file >\
		    \$\{tmpFile\}\" \"\\\$(AS) \$flags -c \$\{tmpFile\} -o\
		    \[outputFiles \$file\]\" \"\\\$(RM)\
		    \$\{tmpFile\}\"]\n    \}\n\n"

	append str "    proc toolMacro \{\} \{\n\treturn AS\n    \}\n\n"
	append str "    proc flagsMacro \{\} \{\n\treturn\
		    CFLAGS_AS\n    \}\n\}\n"

	puts $fd $str
     
	# linker
     
	set str "namespace eval ::tc_${toolchain}::tool_ld \{\n\n"
	append str "    proc type \{\} \{\n\treturn \"linker\"\n    \}\n\n"
	append str "    proc inputSuffixes \{\} \{\n\treturn \"\"\n    \}\n\n"
	append str "    proc outputFiles \{inFile\} \{\n\treturn\
		    \[file rootname \[file tail \$inFile]].o\n    \}\n\n"
	append str "    proc buildCmds \{flags file\} \{\n\treturn \[list\
		    \"[PrjUtils::relPath $tool_ld($toolchain)] \$flags -c\
		    \$file\"]\n    \}\n\n"
	append str "    proc toolMacro \{\} \{\n\treturn LD\n    \}\n\n"
	append str "    proc flagsMacro \{\} \{\n\treturn LDFLAGS\n    \}\n\}\n"
     
	puts $fd $str
     
	# the whole toolchain

	set str "namespace eval ::tc_${toolchain} \{\n\n"
	append str "    proc type \{\} \{\n\treturn C++\n    \}\n\n"
	append str "    proc tool \{\} \{\n\treturn $tool\n    \}\n\n"
	append str "    proc family \{\} \{\n\treturn $family\n    \}\n\n"
	append str "    proc cpu \{\} \{\n\treturn $cpu\n    \}\n\n"
	append str "    proc archDir \{\} \{\n\treturn [PrjUtils::archDirFind\
		    $cpu]\n    \}\n\n"
	append str "    proc name \{\} \{\n\treturn $toolchain\n    \}\n\n"
	append str "    proc toolListGet \{\} \{\n\treturn \[namespace children\
		    \[namespace current] tool_*]\n    \}\n\n"

	append str "    proc toolFindByFile \{file\} \{\n"
	append str "\tforeach tool \[toolListGet\] \{\n"
	append str "\t    if \{\[lsearch \[\${tool}::inputSuffixes\]\
		    \[file extension \$file\]\] != -1\} \{\n"
	append str "\t\treturn \$tool\n\t    \}\n\t\}\n"
	append str "\tprjWarnLog \"no tool works on file \$file\"\n\treturn\
		    \"\"\n    \}\n\n"

	append str "    proc toolFindByType \{type\} \{\n"
	append str "\tforeach tool \[toolListGet\] \{\n"
	append str "\t    if \{\[\$\{tool\}::type] == \$type\} \{\n"
	append str "\t\treturn \$tool\n\t    \}\n\t\}\n"
	append str "\tprjWarnLog \"no tool of type \$type\"\n"
	append str "\treturn \"\"\n    \}\n\n"

	append str "    proc dependFiles \{tool flags file\} \{\n"
	append str "\tregsub -all -- \" $md\" \$flags \"\" cppFlags\n"
	append str "\tif \{\[catch \{set bUseQuickScan\
		    \[::Dependencies::UseQuickScan?]\}] || !\$bUseQuickScan ||\
		    \[catch \{set result\
		    \[DependencyAnalyzer::GenerateDependenciesFromFile\
		    \$cppFlags \$file]\}]\} \{\n"
	append str "\t    set result \[oldDependFiles \$tool \$flags \$file]\n"
	append str "\t\}\n\treturn \$result\n    \}\n\n"
     
	# get the default depend tool

	if {[info exists tool_depend_gen_util($toolchain)]} {
	    set depend $tool_depend_gen_util($toolchain)
	} else {
	    set depend [format "%s %s" $tool_cpp($toolchain) \
			       $tool_depend($toolchain)]
	}

	append str "    proc oldDependFiles \{tool flags file\} \{\n\n"
	append str "\tglobal tcInfo_\[name]\n\n"
	append str "\tregsub -all -- \" $md\" \$flags \"\" cppFlags\n"

	# If the compiler writes to standard error (e.g. a compiler warning) and
	# standard error isn't redirected, then exec will return an error. To
	# avoid this we redirect stderr to a temporary file, which we can then
	# use to inform the user of the errors. 

	append str "\tset errorFile \[tempFileGet\]\n"
	append str "\tif \{\[catch \{eval exec $depend \$cppFlags \$file 2> \
		    \[filenameQuoteIfNecessary \$errorFile\]\} depend\]\} \{\n"
	append str "\t    set fd \[open \$errorFile r\]\n"
	append str "\t    set dependError \[read \$fd\]\n"
	append str "\t    close \$fd\n"
	append str "\t    error \"dependency generation failed:\
		    \$dependError\"\n"
	append str "\t\}\n\tfile delete \$errorFile\n"

	# strip off line continuation

	append str "\tregsub -all \{ \\\\\} \$depend \{\} depend\n"

	# convert windows style path to unix style.

	append str "\tregsub -all \{\\\\\} \$depend \{\\/\} depend\n"
	append str "\treturn \[lrange \$depend 2 end]\n    \}\n\n"

	append str "    proc macroListGet \{\} \{\n\tglobal\
		    tcInfo_\[name\]\n\treturn \[array names\
		    tcInfo_\[name]]\n    \}\n\n"
	append str "    proc macroDefaultValGet \{macro\} \{\n\tglobal\
		    tcInfo_\[name]\n\treturn \[set\
		    tcInfo_\[name](\$macro)]\n    \}\n\n"

	# the following routines are specific to C++ toolchains
	# XXX - maybe move to architecturedb one day
	# if module1 references a common block symbol in module2, does
	# the linker drag in the whole module, or just the symbol?
     
	append str "    proc commSymLink \{\} \{\n\tswitch -glob \[archDir]\
		    \{\n\t    mc68k \{return symbol\}\n\t    default \{return\
		    module\}\n\t\}\n    \}\n\n"


	# given a file, the default wtxdownload flags to be used when
	# downloading. this is especially useful because C and C++ object file
	# use different downloading options, because they do different stuff
	# with common block symbols

	append str "    proc defaultWtxDownloadFlags \{strFilename\} \{\n"
	append str "\tset bufNmOutput \{\}\n"

	# remove the -g flag which request only globals, because we're
	# interested in the non-global cplusplus symbol

	append str "\tregsub -- -g \[macroDefaultValGet NM] \{\} strCmd\n"
	append str "\tset bFailed \[catch \{set bufNmOutput \"\\n\[eval\
		    exec \$strCmd \$strFilename ]\" \} ]\n"
	append str "\tif \{\$bFailed ||\
		    \[regexp \"\\n0+\\\[ \\t]+\\\[A-Za-z]\\\[\
		    \\t]+___gnu_compiled_cplusplus\" \$bufNmOutput] \} \{\n"
     
	# it's C++ or the exec command failed (assume C++)

	append str "\t    return\
		    \{LOAD_GLOBAL_SYMBOLS|LOAD_COMMON_MATCH_ALL\}\n"
	append str "\t\} else \{\n"

	# it's C 

	append str "\t    return\
		    \{LOAD_GLOBAL_SYMBOLS|LOAD_COMMON_MATCH_NONE\}\n"
	append str "\t\}\n    \}\n\n"
     
	# character prepended to "C" symbols by the compiler
     
	append str "    proc cPrependChar \{\} \{\n"
	switch -glob $cpu {
	    MIPS*		{append str "\treturn \"\"\n    \}\n\n"}
	    PPC*		{append str "\treturn \"\"\n    \}\n\n"}
	    ARM*		{append str "\treturn \"\"\n    \}\n\n"}
	    STRONGARM*		{append str "\treturn \"\"\n    \}\n\n"}
	    XSCALE*		{append str "\treturn \"\"\n    \}\n\n"}
	    PENTIUM*		{append str "\treturn \"\"\n    \}\n\n"}
	    SIMHPPA*		{append str "\treturn \"\"\n    \}\n\n"}
	    SIMSPARCSOLARIS	{append str "\treturn \"\"\n    \}\n\n"}
	    SIMLINUX		{append str "\treturn \"\"\n    \}\n\n"}
	    SIMNT		{append str "\treturn \"\"\n    \}\n\n"}
	    ULTRA*		{append str "\treturn \"\"\n    \}\n\n"}
	    default		{append str "\treturn \"_\"\n    \}\n\n"}
	}

	# does the assembler have a preprocessor?

	append str "    proc asmHasCpp \{\} \{\n"

	if {($family == "gnu") || ($family == "diab")} {
	    append str "\treturn 1\n    \}\n\n"
	} else {
	    append str "\treturn 0\n    \}\n\n"
	}

	append str "\}\n"

	puts $fd $str

	# register the toolchain with the project facility

	set str "if \{\[info commands prjTcAdd] == \"prjTcAdd\"\}\
		 \{\n    prjTcAdd ::tc_${toolchain}\n\}"

	puts $fd $str

	incr tcCount
    }
    close $fd

    if {$tcCount > 0} {
	file rename -force ${toolchainInfoFile}.temp ${toolchainInfoFile}
    } else {
	file delete -force ${toolchainInfoFile}.temp
    }

    # reset CPU, TOOL, MAKEFLAGS environment variables

    foreach envVar [array names saveEnv] {
	set env($envVar) $saveEnv($envVar)
    }
}

TcInfoGen::init $argc $argv
