# prepare.tcl - prepares the kernel configuration migration

# Copyright 2004 Wind River Systems, Inc.

# modification history
# --------------------
# 01e,09nov04,fle  Fixed dependencies that cannot be migrated
# 01d,29oct04,fle  Also save the default list of components to the diff file
# 01c,05oct04,fle  Fixed path of generated projects
# 01b,14may03,fle  check for bspDir value for vxApp
# 01a,27apr04,fle  written

# SYNOPSIS
# \cs
#   wtxtcl prepare.tcl [-tmp <tmpDir>] [-q] -w <windbase> -p <projFile>
# \ce

namespace eval Prepare {

    namespace export copy create diff move _read

    variable mQuiet		0
    variable mProjFiles		""
    variable mWindBase		""
    variable mTmpDir		""
    variable mScriptPath	""
    variable mLogFile		[file join c:/ temp prepare.log.txt]
    variable mLogId		-1
}

################################################################################
#
# Prepare::copy - copy a directory to another place
#
# SYNOPSIS
# \cs
#   Prepare::copy <src> <dest>
# \ce
#

proc Prepare::copy {args} {

    # check arguments

    if {[llength $args] < 2} {
	error "Prepare::copy - too few arguments"
    } elseif {[llength $args] > 2} {
	error "Prepare::copy - too many arguments"
    }

    set src		[lindex $args 0]
    set dest		[lindex $args 1]
    set destPath	[file join $dest [file tail $src]]

    if {[file exists $destPath]} {

	# remove this destination

	if {! [Prepare::quiet]} {
	    puts stdout "Removing $destPath ..."
	}

	if {[catch {file delete -force $destPath} status]} {
	    error "Could not remove $destPath : $status"
	}
    }

    file copy -force $src $dest

    if {! [Prepare::quiet]} {
	puts stdout "Copied [file tail $src] to $dest ..."
    }
}

################################################################################
#
# Prepare::move - move a directory to another destination
#
# SYNOPSIS
# \cs
#   Prepare::move <src> <dest>
# \ce
#

proc Prepare::move {args} {


    # check arguments

    if {[llength $args] < 2} {
	error "Prepare::move - too few arguments"
    } elseif {[llength $args] > 2} {
	error "Prepare::move - too many arguments"
    }

    set src		[lindex $args 0]
    set dest		[lindex $args 1]
    set destPath	[file join $dest [file tail $src]]

    if {[file exists $destPath]} {

	# remove this destination

	if {! [Prepare::quiet]} {
	    puts stdout "Removing $destPath ..."
	}

	if {[catch {file delete -force $destPath} status]} {
	    error "Could not delete $destPath : $status"
	}
    }

    file copy -force $src $dest

    if {! [Prepare::quiet]} {
	puts stdout "Moved [file tail $src] to [file tail $dest] ..."
    }
}

################################################################################
#
# Prepare::_read - read a project and writes useful info to <destFile>
#
# SYNOPSIS
# \cs
#   Prepare::_read <prjFile> <destFile>
# \ce
#

proc Prepare::_read {base tmpDir prjFile destFile} {

    # compute the wtxtcl command line

    set cmd "wtxtcl [Prepare::migrateScripts] prjRead $prjFile $destFile"

    tcMigrateLib::execute [base] [tmpDir] $cmd

    if {! [quiet]} {
	puts stdout "Save [file tail $prjFile] info in $destFile"
    }
}

################################################################################
#
# Prepare::create - create a new default project from old one
#
# SYNOPSIS
# \cs
#   Prepare::create <base> <tmpDir> <infoScript>
# \ce
#
# DESCRIPTION
# Create a new default project from given project file
#
# PARAMETERS:
#
# RETURNS: N/A
#
# SEE ALSO: Prepare::_read
#

proc Prepare::create {base tmpDir prjDestFile infoScript} {

    if {! [file exists $infoScript]} {
	error "Prepare::create - info script $infoScript does not exist"
    }

    # sourcing the info script should provide with the prjFile, bspDir, cpu,
    # tool and type values

    source $infoScript

    # check for bspDir, it may not exist if it is a vxApp

    if {$bspDir == ""} {
	set bspDir	"\"\""
    }

    # create the wtxtcl command

    set cmd "wtxtcl [Prepare::migrateScripts] prjCreate $prjDestFile $bspDir "
    append cmd " \"\" " $cpu " " $tool " " $type

    tcMigrateLib::execute $base $tmpDir $cmd

    if {! [quiet]} {
	puts stdout "Created $prjFile project"
    }
}

################################################################################
#
# Prepare::diff - create the diff script between current and default project
#
# SYNOPSIS
# \cs
#   Prepare::diff <infoScript> <defaultScript> <outputScript>
# \ce
#

proc Prepare::diff {oldPrjFile defaultPrjFile outputScript} {

    # compute the wtxtcl command line

    set cmd "wtxtcl [Prepare::migrateScripts] prjDiff $oldPrjFile\
	     $defaultPrjFile $outputScript"

    tcMigrateLib::execute [base] [tmpDir] $cmd

    if {! [quiet]} {
	puts stdout "Save [file tail $prjFile] info in $outputScript"
    }
}

# --------------------------- utilities procedures --------------------------- #

################################################################################
#
# Prepare::migrateScripts - get the path of the tc migration scripts library
#
# SYNOPSIS
# \cs
#   Prepare::migrateScripts
# \ce
#

proc Prepare::migrateScripts {} {

    variable mScriptPath

    return [file nativename [file join $mScriptPath tcMigrateScripts.tcl]]
}

################################################################################
#
# Prepare::parse - parses this tool's command line arguments
#
# SYNOPSIS
# \cs
#   Prepare::parse <argc> <argv>
# \ce
#

proc Prepare::parse {argc argv} {

    variable mProjFile	""

    for {set ix 0} {$ix < $argc} {incr ix} {

	set arg [lindex $argv $ix]

	switch -- $arg {

	    -h -
	    -help -
	    --h -
	    --help {

		# ask for help

		Prepare::help
	    }

	    -tmp {

		# temp directory specification

		incr ix
		if {$ix < $argc} {

		    Prepare::tmpDir [lindex $argv $ix]
		}
	    }

	    -q -
	    -quiet {

		# quiet mode

		Prepare::quiet 1
	    }

	    -w {

		# windbase specification

		incr ix

		if {$ix < $argc} {

		    Prepare::base [lindex $argv $ix]
		}
	    }

	    -p -
	    -project {

		# add a project to the project file list

		incr ix

		if {$ix < $argc} {

		    Prepare::projects [lindex $argv $ix]
		}
	    }
	}
    }
}

################################################################################
#
# Prepare::quiet - Get/Set the prepare tool quiet mode
#
# SYNOPSIS
# \cs
#   Prepare::quiet [<mode>]
# \ce
#

proc Prepare::quiet {{mode ""}} {

    variable mQuiet

    if {$mode != ""} {

	if {($mode == 1) || ($mode == "on")} {
	    set mQuiet	1
	} elseif {($mode == 0) || ($mode == "off")} {
	    set mQuiet	0
	}
    }

    return $mQuiet
}

################################################################################
#
# Prepare::tmpDir - Get/Set the prepare tool temp directory
#
# SYNOPSIS
# \cs
#   Prepare::tmpDir [<dir>]
# \ce
#

proc Prepare::tmpDir {{dir ""}} {

    variable mTmpDir

    if {$dir != ""} {

	set mTmpDir	$dir

	# create it if needed

	if {! [file exists $dir]} {
	    if {[catch {file mkdir $dir} status]} {
		error "Prepare::tmpDir - Could not create $dir : $status"
	    }
	}
    }

    return $mTmpDir
}

################################################################################
#
# Prepare::base - Get/Set the prepare tool Wind Base
#
# SYNOPSIS
# \cs
#   Prepare::base [<dir>]
# \ce
#

proc Prepare::base {{dir ""}} {

    variable mWindBase

    if {$dir != ""} {
	if {! [file exists $dir]} {
	    error "Invalid Wind Base $dir"
	}

	set mWindBase	$dir
    }

    return $mWindBase
}

################################################################################
#
# Prepare::projects - Get/Add project files
#
# SYNOPSIS
# \cs
#   Prepare::projects [<prjFile>]
# \ce
#

proc Prepare::projects {{prjFile ""}} {

    variable mProjFiles

    if {$prjFile != ""} {

	# check if that file exists

	if {[file exists $prjFile]} {
	    lappend mProjFiles	$prjFile
	} else {
	    puts stderr "Project File $prjFile does not exist ... Ignored"
	}
    }
    return $mProjFiles
}

################################################################################
#
# Prepare::check - check that all require info is available
#
# SYNOPSIS
# \cs
#   Prepare::check
# \ce
#

proc Prepare::check {} {

    if {[Prepare::base] == ""} {

	puts stderr "Please provide with a Wind Base. Please use the -w option"
	Prepare::help

    } elseif {[Prepare::tmpDir] == ""} {

	# try to find one using the Wind package

	if {[catch {package require Wind}]} {
	    puts stderr "Now way to get temp directory. Please use the -tmp\
			 option."
	    Prepare::help
	}

	Prepare::tmpDir [Wind::tmpDirGet]

    } elseif {[llength [Prepare::projects]] == 0} {

	puts stderr "Please provide with at least one project file."
	Prepare::help

    } else {

	# check that there is no space in tmp dir

	if {[regexp " \t" [Prepare::tmpDir]]} {

	    puts stderr "Temp directory is set to [Prepare::tmpDir]"
	    puts stderr "Please provide with a temp directory without any space\
			 character usinf the\n-tmp option."
	    Prepare::help
	}
    }
}

################################################################################
#
# Prepare::help - preint this tool's help message and exit with '-1' code
#
# SYNOPSIS
# \cs
#   Prepare::help
# \ce
#

proc Prepare::help {} {

    puts stderr "usage: wtxtcl prepare.tcl -help"
    puts stderr "       wtxtcl prepare.tcl \[-q] \[-tmp <tmpDir>] -w <windbase>\
		 -p <prjFile>"
    puts stderr "                          \[-p <prjFile> ...]"

    exit -1
}

################################################################################
#
# Prepare::init - initialise the prepare script
#
# SYNOPSIS
# \cs
#   Prepare::init
# \ce
#

proc Prepare::init {} {

    variable mScriptPath

    set scriptPath	[info script]
    set scriptDir	[file dirname $scriptPath]
    if {$scriptDir == "."} {
	set scriptDir	[pwd]
    }

    set mScriptPath	$scriptDir

    source [file join $scriptDir tcMigrateLib.tcl]
}

proc Prepare::log {msg} {

    variable mLogFile
    variable mLogId

    if {$mLogId == -1} {
	set mLogId	[open $mLogFile a]
    }

    puts $mLogId	$msg
}

proc Prepare::logClose {} {

    variable mLogId

    if {$mLogId != -1} {

	catch {close $mLogId}
	set mLogId -1
    }
}

# ----------------------- prepare.tcl main entry point ----------------------- #

Prepare::parse $argc $argv
Prepare::check
Prepare::init

set filesToRemove	{}

foreach project [Prepare::projects] {

    set oldPrjPath		[file dirname $project]
    if {$oldPrjPath == "."} {
	set oldPrjPath		[pwd]
    }

    set oldPrjFile		[file tail $project]
    set oldPrjName		[file root $oldPrjFile]
    set oldPrjDir		[file tail $oldPrjPath]

    set tmpPrjPath		[file join [Prepare::tmpDir] $oldPrjDir]
    set tmpPrjFilePath		[file join $tmpPrjPath $oldPrjFile]

    set oldInfoScript		[file join [Prepare::tmpDir] \
					   [format "%s.info.tcl" $oldPrjName]]
    set diffInfoScript		[file join [Prepare::tmpDir] \
					   [format "%s.diff.tcl" $oldPrjName]]

    # copy project to migrate, and save its info

    Prepare::_read [Prepare::base] [Prepare::tmpDir] $project $oldInfoScript

    # create a default project based on the very same bsp, and save its info

    Prepare::create [Prepare::base] [Prepare::tmpDir] $tmpPrjFilePath \
		    $oldInfoScript

    # create the difference script

    set fileId	[open $diffInfoScript w]
    puts $fileId "# initial project location"
    puts $fileId "set prjDir \"$oldPrjPath\""
    close $fileId

    Prepare::diff $project $tmpPrjFilePath $diffInfoScript

    lappend filesToRemove $tmpPrjPath
    lappend filesToRemove $oldInfoScript

    puts stdout $diffInfoScript
}

# remove uneeded files

foreach filePath $filesToRemove {
    catch {file delete -force $filePath} status
}

Prepare::logClose
