# tcMigrate.tcl - Project Toolchain migration tool

# Copyright 2003-2005 Wind River Systems, Inc.

# modification history
# --------------------
# 01k,11jan05,fle  Check when script directory is not absolute
#                  + remove trailing slash in prjDir
# 01j,26nov04,fle  made all paths be full paths, not relative paths anymore
#                  + updated doc
# 01i,19nov04,fle  adapt to new prepare behaviour
# 01h,15nov04,fle  SPR#104182 : change build spec for build macro when
#                  migrating macros
# 01g,09nov04,fle  Better use cmpAdd and cmpRemove too add/remove components
# 01f,28oct04,fle  Project Workspace files should be copied in destination
#                  directory
# 01e,27sep04,fle  SPR#100083 : fixed modifed tags processing
#                  + added a bit of verbosity
# 01d,19aug04,fle  Made it executable
# 01c,14may04,fle  Fixed help message display
#                  + adapted to vxApp project types
# 01b,13apr04,fle  Adapted to new resources location
# 01a,19may03,fle  written

# SYNOPSIS
# \cs
#     tcMigrate -help
#     tcMigrate -prepare [-tmp <tmpDir>] -w[indbase] <oldBase> -p[roject]
#               <prjFile>
#     tcMigrate -continue <script> [-tmp <tmpDir>] [-tc <toolchain>]
#               [-o[ut] <outDir>] [-y]
#     tcMigrate [-t[mp] <tmpDir>] [-y] [-o[ut] <outDir>] 
#               [-tc <toolchain>]  -w[indbase] <oldBase> -p[roject] <prjFile>
# \ce
#
# DESCRIPTION
# This tool is aimed to migrate existing Tornado 2.x projects (either vxWorks
# kernel projects, or vxWorks application projects) to the current version
# of vxWorks projects
#
# It can either migrate some .wpj file (vxWorks project files), or some .wsp
# files (Tornado Workspace files), in which case all projects pointed to by the
# Workspace file are migrated
#
# OPTIONS
# \is
# \i `-help'
# print this tool's help message, and exit with status '-1'
# \i `-prepare'
# only do the preparation step, and save prepare script and required files in
# the <tmpDir> directory
# \i `-continue'
# continue migration using <script>, which is the prepare script created
# by the 'prepare' step (see '-prepare' option)
# \i `-o[ut]'
# migrated project is saved in <outDir> instead of WIND_BASE/target/proj
# \i `-t[mp]'
# <tmpDir> is used for temporary directory (default /tmp on Unixes, C:/temp on
# Windows)
# \i `-tc'
# migrated project should use <toolchain> as its toolchain, instead of the
# toolchain which was used by the old project
# \i `-w[indbase]'
# <oldBase> is the installation path of old Tornado
# \i `-p[roject]'
# <project> is the project file to migrate. May be a .wpj file (vxWorks
# project file), or a .wsp file (Tornado Workspace file)
# \i `-y'
# do not prompt user for changes
# \ie
#
# LIMITATIONS
# \is
# \i `Supported Board Support Packages'
# This tool can only migrate projects from an existing Board Support Package
# (BSP) to a BSP which is supported by the current vxWorks installation
# \i `Specific compile flags'
# When migrating a project from a toolchain to another (for example from 'gnu'
# to 'diab'), if some 'gnu' specific flags were added to the build flags,
# they may be useless when migrated to the new toolchain
# \i `Unsupported components'
# If the Tornado 2 project was using some kernel components which are not
# supported anymore, the migration may complete successfully, but the
# resulting kernel configuration may be wrong. For example, if some header
# files were related to a given kernel component, they may not be present
# anymore, and make the project build fail. Please see the
# 'UNSUPPORTED COMPONENTS' section for more information on this
# \ie
#
# CAVEATS
# \is
# \i `Dependencies files'
# Files included by dependency, but which are not part of the project itself
# (for example header files), and which are not in the project directory, will
# be copied in the destination project directory
#
# For example, if a.h is included from somewhere in the host file system through
# a '-I' path, will be copied in the migrated project directory
#
# This is to help migrating projects from a platform to another (let's say from
# Windows to Linux, or from Solaris to Windows ...)
# \ie
#
# MIGRATION STEPS
# There are two possibilities to migrate projects. Either both software
# installations are available on the same host at the same time, either they
# are on different hosts
#
# If everything is on the same platform, using 'tcMigrate' can be done in
# just one step, else, the 'prepare' and 'continue' steps are required
#
# \is
# \i `Migration on a same platform'
# The command line in this case is of the form :
# \cs
#   wtxtcl tcMigrate.tcl [-t[mp] <tmpDir>] [-y] [-o[ut] <outDir>]\
#                        [-tc <toolchain>]  -w[indbase] <oldBase> 
#                        -p[roject] <prjFile>
# \ce
# Please read the 'OPTIONS' section to know the meaning of all the parameters
#
# \i `Migration on different platform'
# For example, migrating from a Tornado 2.2 on a Windows host to a Linux
# host
#
# In this case the 'prepare' step and 'continue' steps are required
#
# \i `Preparing migration'
# This step must be performed in the old Tornado install environment. This
# requires a command line shell to be started
#
# On unix, start a terminal, and the source the
# WIND_BASE/host/WIND_HOST_TYPE/bin/torVars.(c)sh file (.sh for sh shell, .csh
# for csh shells)
#
# On Windows, start a command line prompt, and execute the
# WIND_BASE\host\WIND_HOST_TYPE\bin\torVars.bat file
#
# Go in the directory where the tcMigrate.tcl script is, and issue the prepare
# step command, where the command should be :
#
# \cs
#   wtxtcl tcMigrate.tcl -prepare [-tmp <tmpDir>] -w[indbase] <oldBase>
#                        -p[roject] <prjFile>
# \ce
#
# The preparation script is saved in the <tmpDir> directory, with all the files
# required for the migration. Thus, this <tmpDir> must be accessible from
# the new installation file system (may be copied there ...)
#
# Some warnings may be issued during preparation step, and the script to
# use for the 'continue' step is printed on the standard output at the end
# of the 'prepare' step
#
# Please read the 'OPTIONS' section for more information on the options and
# parameters of this command
#
# \i `Continuing migration'
# This step must be started in the new installation environment, using the
# information and files collected during the 'prepare' step
#
# The command to continue the migration is :
#
# \cs
#   wtxtcl tcMigrate -continue <script> [-tmp <tmpDir>] [-tc <toolchain>]
#                    [-o[ut] <outDir>] [-y]
# \ce
#
# Where <script> is the prepare script generated by the 'prepare' step
#
# Please read the OPTIONS section for more information on the options and
# parameters of this command
# \ie
#
# UNSUPPORTED COMPONENTS
# There are four types of unsupported kernel components
# \is
# \i 'Removed components'
# The kernel component does not exist anymore with this new version of the
# vxWorks kernel, but it has been replaced by some other components which should
# automatically be added. Such components are removed silently, and the
# migration log keeps track of it
# \i 'Renamed components'
# The component name simply changed. If it has been added (or removed) in the
# old project, the migrated project will have the renamed component added (or
# removed)
# \i `Removed components warned'
# The kernel component does not exist anymore, and cannot be added (removed)
# from the migrated kernel configuration, but the user has to know about it,
# since he may have actions to take about this component. A warning is issued on
# the standard error channel during migration
# \i `Renamed components warned'
# The kernel component has been renamed, and its new version has been added
# (removed) from the migrated kernel configuration, but the user has to know
# about it, since he may have actions to take about this component. A warning is
# issued on the standard error channel during migration
# \ie
# Unsupported kernel components are listed in the 'include' directory of the
# `tcMigrate' tool. There is a 'README' file here which explains how to add
# unsupported components
#
# RETURNS: N/A
#
# SEE ALSO: cmpScriptLib
#
# NOROUTINES
#

set executable	[file root [file tail [info nameofexecutable]]]

if {! [regexp -nocase -- "^wtx" $executable]} {

    # we have to rely on the Wind package

    if {[catch {package require Wind} status]} {

	# well, not in a wtxtcl executable, and no Wind package, there
	# are no ways to guess for the Wind Host Type and Wind Path, abort

	puts stderr "Please use wtxtcl instead of $executable. Exiting ..."
	exit -1
    }

    proc wtxHostType {} {
	return [Wind::hostType]
    }

    proc wtxPath {args} {

	set windPath	[eval Wind::path $args]
	return [format "%s/" [file join $windPath]]
    }
}

global env

namespace eval tcMigrate {

    variable mMigrateScripts	""
    variable mNewToolchain	""
    variable mNewWindBase	[file join [wtxPath]]
    variable mOldWindBase	""
    variable mOldProj		""
    variable mOldHostType	[wtxHostType]
    variable mOutDir		[file join [wtxPath] target proj]
    variable mScriptPath	""

    variable mPrepareOnly	0
    variable mContinueOnly	0
    variable mContinueScript	""
    variable mAlwaysYes		0
}

################################################################################
#
# init - initialise what tcMigrate needs to work properly
#

proc tcMigrate::init {} {

    global env
    global tcl_platform

    variable mMigrateScripts	""
    variable mNewWindBase
    variable mOldWindBase
    variable mScriptPath

    # first keep the current path in mind

    set scriptDir	[file dirname [info script]]

    if {$scriptDir == "."} {
	set scriptDir	[pwd]
    } elseif {[file pathtype $scriptDir] != "absolute"} {
	set scriptDir	[file join [pwd] $scriptDir]
    }

    set mScriptPath	[file join $scriptDir]
    set mMigrateScripts	[file join $mScriptPath tcMigrateScripts.tcl]
}

# --------------------------- utilities procedures --------------------------- #

################################################################################
#
# tcMigrate::argsParse - parse tcMigrate arguments
#
# SYNOPSIS
# \ts
#   tcMigrate::argsParse <arguments>
# \te
#
# DESCRIPTION
# This procedure simply parse the given <arguments> to initialise some of the
# 'tcMigrate' variables
#
# PARAMETERS:
#   arguments : the list of arguments to parse
#
# RETURNS: N/A
#
# NOMANUAL
#

proc tcMigrate::argsParse {arguments} {

    variable	mNewToolchain
    variable	mOldProj
    variable	mOldWindBase
    variable	mOldHostType
    variable	mOutDir
    variable	mPrepareOnly
    variable	mContinueOnly
    variable	mContinueScript
    variable	mAlwaysYes

    for {set ix 0} {$ix < [llength $arguments]} {incr ix} {

	set arg	[lindex $arguments $ix]

	switch -- $arg {

	    -h		-
	    --h		-
	    --help	-
	    -help	{
		tcMigrate::usage
	    }

	    -w		-
	    -windbase	{

		incr ix
		set arg [file join [lindex $arguments $ix]]

		# make it in full path

		if {[file pathtype $arg] != "absolute"} {
		    set arg	[file join [pwd] $arg]
		    while {[regsub -- {/[^/]+/\.\.} $arg "" arg]} {}
		    while {[regsub -- {/\.} $arg "" arg]} {}
		}

		# check if the given old WIND_BASE really exists

		if {! [file exists $arg]} {
		    puts stderr "\"$arg\" does not exist ... Cannot use it for\
				 old WIND_BASE"
		} else {
		    set mOldWindBase	$arg
		}
	    }

	    -p		-
	    -proj	-
	    -project	{

		incr ix
		set arg [file join [lindex $arguments $ix]]

		# make it in full path

		if {[file pathtype $arg] != "absolute"} {
		    set arg	[file join [pwd] $arg]
		    while {[regsub -- {/[^/]+/\.\.} $arg "" arg]} {}
		    while {[regsub -- {/\.} $arg "" arg]} {}
		}

		# check if given project file really exists

		if {! [file exists $arg]} {
		    puts stderr "\"$arg\" does not exist ... Cannot use it for\
				 old project"
		} else {

		    # file does exist, but check it is a regular file

		    if {[file isfile $arg]} {
			set mOldProj	$arg
		    } else {
			puts stderr "\"$arg\" is not a regulr file ... Cannot\
				     use it for old project"
		    }
		}
	    }

	    -tmp	-
	    -temp	{

		incr ix
		set arg [file join [lindex $arguments $ix]]

		# make it in full path

		if {[file pathtype $arg] != "absolute"} {
		    set arg	[file join [pwd] $arg]
		    while {[regsub -- {/[^/]+/\.\.} $arg "" arg]} {}
		    while {[regsub -- {/\.} $arg "" arg]} {}
		}

		tcMigrateLib::tmpDir $arg
	    }

	    -tc		-
	    -toolchain	{

		incr ix
		set arg [lindex $arguments $ix]
		set mNewToolchain $arg
	    }

	    -o		-
	    -out	{

		incr ix
		set arg [file join [lindex $arguments $ix]]

		# make it in full path

		if {[file pathtype $arg] != "absolute"} {
		    set arg	[file join [pwd] $arg]
		    while {[regsub -- {/[^/]+/\.\.} $arg "" arg]} {}
		    while {[regsub -- {/\.} $arg "" arg]} {}
		}


		# try to create it if it does not exist

		if {! [file exists $arg]} {
		    if {[catch {file mkdir $arg} status]} {
			puts stderr "Could not create \"$arg\" : \"$status\""
			puts stderr "Using $mOutDir instead ..."
		    } else {

			set mOutDir $arg
		    }
		} else {
		    set mOutDir $arg
		}
	    }

	    -prepare	{
		set mPrepareOnly	1
	    }

	    -continue	{
		incr ix

		if {$ix < [llength $arguments]} {
		    set mContinueOnly	1
		    set arg		[lindex $arguments $ix]
		    set mContinueScript	$arg
		}
	    }

	    -y	{
		set mAlwaysYes	1
	    }
	}
    }
}

################################################################################
#
# tcMigrate::argsCheck - check that all the required arguments are there
#
# SYNOPSIS
# \cs
#   tcMigrate::argsCheck
# \ce
#
# DESCRIPTION
# This procedure just checks that all the required arguments were given
# to the command line
#
# If some arguments are not there, a message is prompted, and the program
# exits with code '-1'
#
# PARAMETERS: N/A
#
# RETURNS: N/A
#
# SEE ALSO: tcMigrate::argsParse
#

proc tcMigrate::argsCheck {} {

    variable mPrepareOnly
    variable mOldWindBase
    variable mContinueOnly

    # check that all the needed value have been set by the arguments parsing

    if {($tcMigrate::mOldProj == "") && ($mContinueOnly != 1)} {
	puts stderr "Please provide with the old project path ..."
	tcMigrate::usage
    } elseif {($mOldWindBase == "") && ($mContinueOnly != 1)} {
	puts stderr "Please provide with the old Tornado installation path ..."
	tcMigrate::usage
    }

}

################################################################################
#
# tcMigrate::usage - prints the tcMigrate tool usage message
#
# SYNOPSIS
# \ts
#   tcMigrate::usage
# \te
#
# DESCRIPTION
# This procedure prints the 'tcMigrate' tool help message and exist
#
# PARAMETERS: N/A
#
# RETURNS: N/A
#
# NOMANUAL
#

proc tcMigrate::usage {} {

    puts stdout "tcMigrate - project toolchain migration tool"
    puts stdout "usage: tcMigrate -help"
    puts stdout "       tcMigrate -prepare \[-tmp <tmpDir>] -w\[indbase]\
		 <oldBase> -p\[roject]"
    puts stdout "                 <prjFile>"
    puts stdout "       tcMigrate -continue <script> \[-tmp <tmpDir>] \[-tc\
		 <toolchain>]\n                 \[-o\[ut] <outDir>] \[-y]"
    puts stdout "       tcMigrate \[-t\[mp] <tmpDir>] \[-y] \[-o\[ut] <outDir>]\
				  \n                 \[-tc <toolchain>] \
				  -w\[indbase] <oldBase> -p\[roject] <prjFile>"
    puts stdout "where:"
    puts stdout "    -help        : print this help message"
    puts stdout "    -prepare     : only do the preparation step"
    puts stdout "    -continue    : continue migration using <script>"
    puts stdout "    -o\[ut]       : migrated project will be saved in <outDir>"
    puts stdout "                   instead of WIND_BASE/target/proj"
    puts stdout "    -t\[mp]       : <tmpDir> is used for temporary directory"
    puts stdout "                   (default /tmp on Unixes, C:/temp on\
		 Windows)"
    puts stdout "    -tc          : migrated project should use <toolchain> as"
    puts stdout "                   its toolchain"
    puts stdout "    -w\[indbase]  : <oldBase> is the installation path of old"
    puts stdout "                   Tornado"
    puts stdout "    -p\[roject]   : <project> is the project file to migrate"
    puts stdout "    -y           : do not prompt user for changes"

    exit 1
}

################################################################################
#
# tcMigrate::gatherIndices - gathers all consecutive <lst> elements of value -1
#
# SYNOPSIS
# \cs
#   tcMigrate::gatherIndices <lst> [<start>]
# \ce
#
# DESCRIPTION
# Gets a list of all consecutive elements of <lst> which value is -1
#
# PARAMETERS
#   lst : list to look for -1 elements
#   start : start index in <lst> (default is 0)
#
# RETURNS: a list of consecutive indices
#
# ERRORS: N/A
#
# EXAMPLES:
# \cs
# % set lst [list -1 -1 2 3 4 -1 6]
# % tcMigrate::gatherIndices $lst
# {0 1} {5 5}
# %
# \ce
#

proc tcMigrate::gatherIndices {lst {start 0}} {

    set ix		0
    set startIx		-1
    set endIx		-1
    set gathered	{}

    # shorten input list if needed

    if {$start > 0} {
	set lst [lreplace $lst 0 [expr {$start} -1]]
    }

    # create a list of consecutive -1 values

    foreach elt $lst {
	if {$elt == -1} {
	    if {$startIx == -1} {
		set startIx	$ix
		set endIx	$ix
	    } else {
		set endIx	$ix
	    }
	} else {
	    if {$startIx != -1} {
		lappend gathered	$startIx
		set startIx		-1
		set endIx		-1
	    }
	}

	incr ix
    }

    return $gathered
}

################################################################################
#
# tcMigrate::componentsFix - apply some components removal/updated to <comps>
#
# SYNOPSIS
# \cs
#   tcMigrate::componentsFix <comps>
# \ce
#
# DESCRIPTION
# Applies the known components renaming/migration/removal to <comps>, and
# returns the modified component list
#
# PARAMETERS:
#   comps : list of components which should be modified to be migrated
#
# RETURNS: The list of modified components to fit with migration
#

proc tcMigrate::componentsFix {comps logId} {

    # SELECT_MMU has disappeared, but it should not be in the components
    # list, so we may ignore it

    array set renamed		{}
    array set warnRenamed	{}
    array set warnRemoved	{}

    set removed	{}

    # source all files from include dir

    set fixFileDir	[file join [file dirname [info script]] include]
    set fixFileList	[glob -nocomplain [file join $fixFileDir *.tcl]]

    foreach fileName $fixFileList {
	source $fileName
    }

    # warned removed components

    foreach comp [array names warnRemoved] {

	set compWarn	[lindex $warnRemoved($comp) 0]

	if {[set ix [lsearch -exact $comps $comp]] != -1} {

	    set comps	[lreplace $comps $ix $ix]
	    puts $logId "Removed $comp component"
	    puts stderr $compWarn
	}

    }

    # remove components which are not supported anymore

    foreach comp $removed {
	if {[set ix [lsearch -exact $comps $comp]] != -1} {
	    set comps	[lreplace $comps $ix $ix]
	    puts $logId "Removed unsupported component $comp"
	}
    }

    # warned replaced components

    foreach comp [array names warnRenamed] {

	set compName	[lindex $warnRenamed($comp) 0]
	set compWarn	[lindex $warnRenamed($comp) 1]

	if {($compName != "") &&
	    ([set ix [lsearch -exact $comps $comp]] != -1)} {

	    set comps	[lreplace $comps $ix $ix $compName]
	    puts $logId "Renamed $comp component to $compName"
	    puts stderr $compWarn
	}
    }

    # rename the renamed components

    foreach comp [array names renamed] {
	if {[set ix [lsearch -exact $comps $comp]] != -1} {
	    set comps	[lreplace $comps $ix $ix $renamed($comp)]
	    puts $logId "Renamed $comp component to $renamed($comp)"
	}
    }

    return $comps
}

# ------------------------ tcMigrate main entry point ------------------------ #

tcMigrate::init

# source the needed project libraries

set rcDir	[file join [wtxPath] host resource osconfig tcl]

if {! [file exists [file join $rcDir cmpScriptLib.tcl]]} {

    # try with the old file location

    set rcDir	[file join [wtxPath] host resource tcl app-config Project]
}

if {! [file exists [file join $rcDir cmpScriptLib.tcl]]} {

    # well, try with the founctaion path

    if {[info exist env(WIND_FOUNDATION_PATH)]} {
	set rcDir	[file join $env(WIND_FOUNDATION_PATH) resource \
				   osconfig tcl]
    }
}

source [file join $rcDir cmpScriptLib.tcl]
source [file join $tcMigrate::mScriptPath tcMigrateLib.tcl]

tcMigrate::argsParse $argv
tcMigrate::argsCheck

if {! $tcMigrate::mContinueOnly} {

    # preparation step

    puts stdout "Preparing migration ..."

    set prepareScript	[file join $tcMigrate::mScriptPath prepare.tcl]
    set tmpDir		[file nativename [tcMigrateLib::tmpDir]]
    set oldBase		[file nativename $tcMigrate::mOldWindBase]

    set project		[file nativename $tcMigrate::mOldProj]

    # check if it is a .wsp file, or a regular .wpj

    if {[file extension $project] == ".wsp"} {

	# read the project list

	set prjDir	[file dirname $project]

	# if the project directory is at root, there may be a trailing slash.
	# Remove it to avoid double slashes

	regsub -all -- {/$} $prjDir "" prjDir

	set contList	{}

	set hProj	[prjOpen $project]
	set prjList	[prjTagDataGet $hProj projectList]
	prjClose $hProj

	foreach prjFile $prjList {

	    regsub -all -- "$prjDir/" [file join $prjFile] "" projectFile

	    set prjTmpDir	[file join $tmpDir [file dirname $projectFile]]

	    set cmd "wtxtcl [file nativename $prepareScript]"	;# prepare step
	    append cmd " -tmp " [file dirname $prjTmpDir]	;# tmp directory
	    append cmd " -q "			;# quiet mode
	    append cmd " -w " $oldBase		;# old install base
	    append cmd " -p " $prjFile		;# project to prepare

	    puts stdout "    Preparing migration for [file tail $prjFile] ..."

	    if {[catch {tcMigrateLib::execute [wtxPath] $tmpDir $cmd} \
		       diffScript]} {

		puts stderr "Error : $diffScript"
	    }

	    # just for one moment, read the diff script, and report errors
	    # and warnings

	    if {[info exists dependencies]} {
		unset dependencies
	    }

	    source $diffScript

	    foreach errMsg $errors {
		puts stderr "    $errMsg"
	    }

	    foreach warnMsg $warnings {
		puts stderr "    $warnMsg"
	    }

	    unset prjFile bsp cpu tool type desc addedComps removedComps \
		  addedFiles removedFiles dependencies addedTags modifiedTags \
		  removedTags errors warnings

	    # path stored in the diff script should be relative paths

	    regsub -all -- "[file join $tmpDir]/" [file join $diffScript] "" \
		   diffScript

	    lappend contList	$diffScript
	}

	# copy the wsp file to the tmp dir

	if {[catch {file copy -force $project $tmpDir} status]} {
	    puts stderr "Could not copy [file tail $project] to $tmpDir :\
			 \"$status\""
	}

	set prjName	[file root [file tail $project]]
	set diffScript	[file join $tmpDir [format "%s.prepare.tcl" $prjName]]

	# write the continueScript list in it

	catch {

	    set fileId	[open $diffScript w]
	    if {[llength $contList] == 0} {
		puts $fileId	"set continueScriptList \{\}"
	    } else {
		puts $fileId	"set continueScriptList \[list $contList\]"
	    }

	    # copy wsp file to tmp dir, and let the prepare script know

	    catch {file copy -force $project $tmpDir}

	    puts $fileId	"set workspaceFile [file tail $project]"
	    close $fileId
	}

	set tcMigrate::mContinueScript	$diffScript

    } else {

	set cmd "wtxtcl [file nativename $prepareScript]"	;# prepare step
	append cmd " -tmp " $tmpDir		;# temp directory
	append cmd " -q "			;# quiet mode
	append cmd " -w " $oldBase		;# old install base
	append cmd " -p " $project		;# project to prepare

	if {[catch {tcMigrateLib::execute [wtxPath] $tmpDir $cmd} diffScript]} {

	    puts stderr "Error : $diffScript"

	} else {

	    # just for one moment, read the diff script, and report errors
	    # and warnings

	    if {[info exists dependencies]} {
		unset dependencies
	    }

	    source $diffScript

	    foreach errMsg $errors {
		puts stderr "    $errMsg"
	    }

	    foreach warnMsg $warnings {
		puts stderr "    $warnMsg"
	    }

	    unset prjFile bsp cpu tool type desc addedComps removedComps \
		  addedFiles removedFiles dependencies addedTags modifiedTags \
		  removedTags errors warnings
	}
    }

    set tcMigrate::mContinueScript	$diffScript
    puts stdout "Migration info saved in [file nativename $diffScript]"
}

if {$tcMigrate::mPrepareOnly} {

   # we should be done, exit with code 0

   exit 0
}

# ----------------------- create a new default project ----------------------- #

# get information from the continue script

if {! [file exists $tcMigrate::mContinueScript]} {
    puts stderr "Could not find continue script $tcMigrate::mContinueScript"
    exit -1
}

# get the directory where the continue script is

set continueScriptDir	[file dirname $tcMigrate::mContinueScript]
if {$continueScriptDir == "."} {
    set continueScriptDir	[pwd]
}

source $tcMigrate::mContinueScript

# check if the continueScriptList is defined. If yes, it means that there
# are several projects to migrate. Source the single continue script if
# needed, this will help avoid to write code several times

if {! [info exists continueScriptList]} {

    set continueScriptList	$tcMigrate::mContinueScript

    unset prjFile bsp cpu tool type desc addedComps removedComps \
	  addedFiles removedFiles addedTags removedTags modifiedTags \
	  dependencies errors warnings
}

if {[info exists workspaceFile]} {
    set wspFile	[file join $continueScriptDir $workspaceFile]
    puts stdout "Copying file [file nativename $wspFile] to [file nativename\
		 $tcMigrate::mOutDir] ..."
    catch {file copy -force $wspFile $tcMigrate::mOutDir}
}

puts stdout "Continuing migration ..."

foreach continueScript $continueScriptList {

    if {[info exists dependencies]} {
	unset dependencies
    }

    if {[file pathtype $continueScript] != "absolute"} {
	set continueScript [ \
	    file join [file dirname $tcMigrate::mContinueScript] \
		      $continueScript \
	]
    }

    source $continueScript

    set prjDir	[file dirname $prjFile]
    set prjFile	[file join [tcMigrateLib::tmpDir] $prjFile]

    puts stdout "    Continuing migration for [file tail $prjFile] ..."

    if {$type == "vxWorks"} {

	# now create the new project with the new toolchain, and report all
	# changes from old project

	set bspDir	[file join [wtxPath] target config $bsp]

	# check that this BSP exists in the current installation

	if {! [file exists [file join $bspDir Makefile]]} {
	    puts stderr "BSP [file tail $bspDir] is not supported for this\
			 installation. Ignoring it."
	    continue
	}

    } elseif {$type == "vxApp"} {

	set bspDir "\"\""

    } else {

	puts stderr "Project type \"$type\" is not supported. Ignoring it."
	continue
    }

    if {$tcMigrate::mNewToolchain == ""} {
	set newToolchain	$tool
    } else {
	set newToolchain	$tcMigrate::mNewToolchain
    }

    puts stdout "\tCreate a new default project with toolchain\
		 $newToolchain ... "

    tcMigrateLib::execute [wtxPath] [tcMigrateLib::tmpDir] \
			  "wtxtcl [file nativename $tcMigrate::mMigrateScripts]\
			  prjCreate $prjFile $bspDir {\"\"} $cpu $newToolchain\
			  $type"

    # create a migration log file

    set logId	[open [file join [file dirname $prjFile] migration.log.txt] w]

    # open this newly created project

    cmpProjOpen			$prjFile
    set hProj			[cmpProjHandleGet]
    set hSpecRegexp		[join [prjBuildListGet $hProj] "|"]
    set wcc			[wccCurrentGet $hProj]
    set wccRegexp		[join [wccListGet $hProj] "|"]

    set nameRegexp [ \
	format "(BUILD_(%s)_MACRO_)|(BUILD_RULE_)|(WCC_(%s)_(PARAM_)*)" \
	       $hSpecRegexp $wccRegexp \
    ]

    # copy all the files which where in the old project, but not in the old
    # default project (assuming they were added, and meant to be added, by the
    # user)

    set parentDir		[file dirname $prjFile]

    foreach fileName [array names addedFiles] {

	set srcFile	[file join [file dirname $continueScript] $fileName]
	set destFile	[prjFullPath $hProj $addedFiles($fileName)]

	puts $logId "Copied source $srcFile to $addedFiles($fileName)"
	if {[catch {file copy -force $srcFile $destFile} status]} {
	    puts stderr "    Could not copy $srcFile to $destFile : $status"
	}
    }

    # take care of the added components and files

    foreach fileName [array names addedFiles] {

	puts $logId	"Adding file $fileName"
	cmpFileAdd	[prjFullPath $hProj $addedFiles($fileName)]
    }

    foreach fileName [array names removedFiles] {
	cmpFileRemove [prjFullPath $hProj $removedFiles($fileName)]
    }

    # remove components if any

    foreach comp $removedComps {
	puts $logId "Removing component $comp"
    }

    if {[llength $removedComps] != 0} {
	cmpRemove $removedComps
    }

    # add new components if any

    set addedComps	[tcMigrate::componentsFix $addedComps $logId]

    foreach comp $addedComps {
	puts $logId "Adding component $comp"
    }

    if {[llength $addedComps] != 0} {
	cmpAdd $addedComps
    }

    # remove all the tags in this project which the user removed in the original
    # project

    set newDefaultTags	[prjTagListGet $hProj]

    foreach tag [array names removedTags] {

	# regsub is just for the log to me clearer, tagName is not used

	regsub -all -- $nameRegexp $tag "" tagName
	puts $logId "Removing tag $tagName ..."
	prjTagRemove $hProj $tag
    }

    # modify the tags the user modified in the old project (hoping he did not
    # change some toolchain related lines wich would break the migration)

    foreach tag [array names modifiedTags] {

	regsub -all -- $nameRegexp $tag "" tagName

	# the values stored in the modifiedTagValueList are :
	#   - old value
	#   - old default tag value
	#
	# Lets just append the difference to the new project values

	set oldTagValue [prjFullPath $hProj [lindex $modifiedTags($tag) 0]]
	set oldDefaultTagValue [prjFullPath $hProj \
					    [lindex $modifiedTags($tag) 1]]

	set oldTagValueIxes		{}
	set oldDefaultTagValueIxes	{}

	# first check that this tag belongs to the new project

	if {([lsearch -exact $newDefaultTags $tag] != -1)} {

	    set newTagValue	[prjTagDataGet $hProj $tag]
	    set prevTagValue	$newTagValue

	    # create an indices array where the changing values are stored

	    foreach elt $oldTagValue {
		lappend oldTagValueIxes [lsearch -exact $oldDefaultTagValue \
							$elt]
	    }

	    foreach elt $oldDefaultTagValue {
		lappend oldDefaultTagValueIxes [lsearch -exact $oldTagValue \
							       $elt]
	    }

	    set oldTagMissingIxes [tcMigrate::gatherIndices $oldTagValueIxes]

	    set oldDefaultTagMissingIxes [ \
		tcMigrate::gatherIndices $oldDefaultTagValueIxes \
	    ]

	    set addedTagValue	{}

	    foreach jx $oldTagMissingIxes {

		if {[lsearch -exact $oldDefaultTagMissingIxes $jx] != -1} {

		    # seems like this element was changed between default and
		    # user project. If it appears in this tag value, let's
		    # replace it too

		    if {[lsearch -exact $newTagValue\
				 [lindex $oldDefaultTagValue $jx]] != -1} {

			# it does appear in the new default value for this tag,
			# replace it with the user defined value

			puts $logId "$tagName : replacing \"[lindex\
				     $newTagValue $jx]\" with \"[lindex\
				     $oldTagValue $jx]\""

			set newTagValue [lreplace $newTagValue $jx $jx \
						  [lindex $oldTagValue $jx]]
		    }

		} else {

		    # tag value element is present in old project, but not in
		    # default project, it should be added to the new tag value,
		    # but only after all replacements took place

		    lappend addedTagValue [list $jx [lindex $oldTagValue $jx]]
		}
	    }

	    # add the tag value elements

	    foreach elt $addedTagValue {

		set eltIx		[lindex $elt 0]
		set eltValue	[lindex $elt 1]

		if {[lsearch -exact $newTagValue $eltValue] == -1} {
		    puts $logId "$tagName : adding \"$eltValue\""
		    set newTagValue	[linsert $newTagValue $eltIx $eltValue]
		}
	    }

	    # now try to find elements to remove

	    foreach jx $oldDefaultTagMissingIxes {

		if {[lsearch -exact $oldTagMissingIxes $jx] == -1} {
		    set eltValue	[lindex $oldDefaultTagValue $jx]

		    if {[set eltIx [lsearch -exact $newTagValue $eltValue]] \
				   != -1} {

			puts $logId "$tagName : removing \"[lindex $newTagValue\
				     $eltIx]\""
			set newTagValue [lreplace $newTagValue $eltIx $eltIx]
		    }
		}
	    }

	    # ask the user if he really wants to replace the tag value. Do not
	    # ask for COMPONENTS ... if the user changed one of them, it is
	    # normal that the value for this tag changed ...

	    if {($prevTagValue != $newTagValue) && ($tagName != "COMPONENTS") &&
		($tcMigrate::mAlwaysYes == 0)} {

		# apply some regsub on tag name

		puts stdout "Would you like to replace $tagName :\n    -\
			     \"$prevTagValue\" \nwith\n    - \"$newTagValue\""
		puts -nonewline stdout "Replace value ? (y/\[n]) : "
		flush stdout
		set userReply [gets stdin]

		if {($userReply == "y") || ($userReply == "Y") ||
		    ($userReply == "yes") || ($userReply == "YES")} {

		    puts $logId "$tagName : replacing \"$prevTagValue\" with\
				 \"$newTagValue\""
		    prjTagDataSet $hProj $tag $newTagValue
		} else {

		    puts $logId "$tagName : Ignoring new value \"$newTagValue\"\
				 ... keep old one \"$prevTagValue\""
		}
	    }
	}
    }

    # add the tags added by the user in the old project, except the _dependDone
    # and _dependencies tags

    foreach tag [array names addedTags] {

	set tagValue $addedTags($tag)

	# some tags may be added when adding files and their types, do not add a
	# tag which is already there

	if {(! [regexp "_dependDone\[ \t]*$" $tag]) &&
	    (! [regexp "_dependencies\[ \t]*$" $tag]) &&
	    ([lsearch -exact $newDefaultTags $tag] == -1)} {

	    regsub -all -- $nameRegexp $tag "" tagName
	    puts $logId "Adding tag $tagName ..."
	    prjTagDataSet $hProj $tag $tagValue
	}
    }

    # now copy the dependencies if any

    foreach depFile [array names dependencies] {

	set srcDep	[file join [file dirname $continueScript] $depFile]
	set destDep	[file join $parentDir $dependencies($depFile)]

	puts $logId "Copied dependency $depFile to $dependencies($depFile)"
	catch {file copy -force $srcDep $destDep}
    }

    # SPR#104184 : check the build macros

     if {$type == "vxApp"} {

	# if toolchain changed, the build macros were named
	# "BUILD_<CPU><tool>_MACRO_XXX", but they should now be named
	# "BUILD_<CPU><newTool>_MACRO_XXX" to be listed in the build macros of
	# the current build spec

 	set oldBuild	[format "%s%s" $cpu $tool]
 	set newBuild	[format "%s%s" $cpu $newToolchain]
	set oldPattern	[format "BUILD_%s_MACRO" $oldBuild]
	set macroList	[prjTagListGet $hProj $oldPattern]

	foreach oldMacro $macroList {
	    set macroValue	[prjTagDataGet $hProj $oldMacro]
	    regsub -all -- $oldPattern $oldMacro \
		   [format "BUILD_%s_MACRO" $newBuild] newMacro
	}

	prjTagRemove $hProj $oldMacro
	prjTagDataSet $hProj $newMacro $macroValue

	puts $logId "Replaced macro named $oldMacro with $newMacro (value\
		     unchanged)"
     }

    catch {close $logId}

    cmpSave
    cmpProjClose

    # build the destination file path

    set destDir	[file join $tcMigrate::mOutDir $prjDir]

    if {[file exists $destDir]} {

	puts stdout "\tRemoving [file nativename $prjDir] from\
		     [file nativename $tcMigrate::mOutDir]"

	file delete -force $destDir
    }

    # create the dest directory if it does not exist yet

    if {! [file exist [file dirname $destDir]]} {
	catch {file mkdir [file dirname $destDir]}
    }

    file copy -force [file dirname $prjFile] $destDir

    puts stdout "\tProject has been created in [file nativename $destDir]"

    # unset the values set by the continue script sourcing

    unset prjFile bsp cpu tool type desc addedComps removedComps \
	  addedFiles removedFiles addedTags removedTags modifiedTags \
	  dependencies errors warnings
}

puts stdout "Toolchain migration finished"
