# tcMigrateLib.tcl - Tornado Project Migration library

# Copyright 2003-2004 Wind River Systems, Inc.

# modification history
# --------------------
# 01c,28sep04,fle  SPR#100083 : use wrenv when there is no wrVars nor torVars
# 01b,13apr04,fle  Adapted to new resources location
# 01a,21may03,fle  written

#
# DESCRIPTION
# This library implements some procedures used by the Toolchain migration tool.
#
# INITIALISATION PHASE
# When this library is sourced, it first gets the current values of WIND_BASE
# and PATH environment variables, and stores them in an array which will handle
# the several values of WIND_BASE and 'PATH' throughout the Tcl scripts.
#

namespace eval tcMigrateLib {

    namespace export execute tmpDir

    variable	mWindBase	""	;# WIND_BASE initial value
    variable	mPath		""	;# PATH environment var's initial value
    variable	mInitialBase	""	;# initial WIND_BASE value
    variable	mInitialPath	""	;# initial PATH value
    variable	mTmpDir		""	;# library temp directory

    variable	mFileId		""	;# log file id

    variable	mBase

    global env
    global tcl_platform

    # get a temp directory by default. Try to avoid dirs with spaces inside
    # their name (compiler would fail to create a project in such a directory)

    set mTmpDir	"/tmp"

    # do some itinialisation

    if {$tcl_platform(platform) == "windows"} {
	set mWindBase		[string tolower [file join [wtxPath]]]
	set mPath		[string tolower $env(PATH)]
	set mTmpDir		[file join c:/ temp]
    } else {
	set mWindBase		[file join [wtxPath]]
	set mPath		$env(PATH)
    }

    set mBase($mWindBase,WIND_BASE)	$mWindBase
    set mBase($mWindBase,PATH)		$mPath
    set mInitialBase			$mWindBase
    set mInitialPath			$mPath
}

# ---------------------------- public procedures ----------------------------- #

################################################################################
#
# tcMigrateLib::execute - execute <cmdLine> in the given environment
#
# SYNOPSIS
# \cs
#   envExecute <base> <tmpDir> <cmdLine>
# \ce

proc tcMigrateLib::execute {base tmpDir cmdLine} {

    global env

    # create the tmp directory if does not exist

    if {! [file exists $tmpDir]} {
	file mkdir $tmpDir
    }

    set binDir	[file join $base host [wtxHostType] bin]
    set doChmod	0

    if {[wtxHostType] == "x86-win32"} {
	set ext		"bat"
	set header	"@ECHO OFF\nCD [file nativename $binDir]"
	set footer	"CMD /C $cmdLine"
    } else {
	set ext		"sh"
	set header	"#!/bin/sh\ncd [file nativename $binDir]"
	set footer	$cmdLine
	set doChmod	1
    }

    set pid		[pid]
    set shScript	[file join $tmpDir [format "tcMigrate.%i.%s" $pid $ext]]

    while {[file exists $shScript]} {
	incr pid
	set shScript	[file join $tmpDir [format "tcMigrate.%i.%s" $pid $ext]]
    }

    set fileId [open $shScript w]

    # write the shell script

    puts $fileId $header

    # read the xxxVars.$ext file if it is there

    set fileToRead	""

    if {[file exists [file join $binDir [format "wpVars.%s" $ext]]]} {

	set fileToRead	[file join $binDir [format "wpVars.%s" $ext]]

    } elseif {[file exists [file join $binDir torVars.${ext}]]} {

	set fileToRead	[file join $binDir [format "torVars.%s" $ext]]

    } elseif {([info exists env(WIND_BASE)]) &&
	      ([file exists [file join $env(WIND_BASE) \
				        [format "wrVars.%s" $ext]]])} {

	set fileToRead	[file join $env(WIND_BASE) [format "wrVars.%s" $ext]]

    }

    # append the Vars content to the shell script if any

    if {$fileToRead != ""} {
	set fd [open $fileToRead r]
	set fileContent [read $fd]
	close $fd
	puts $fileId $fileContent
    }

    puts $fileId $footer
    close $fileId

    # check if a chmod has to be done

    if {$doChmod} {
	file attributes $shScript -permissions 00755
    }

    if {[catch {exec [file nativename $shScript] 2>@ stdout} status]} {

	# remove script file anyway

	catch {file delete -force $shScript}
	error $status
    }

    # remove script file anyway

    catch {file delete -force $shScript}

    return $status
}

################################################################################
#
# tcMigrateLib::tmpDir - get or set the toolchain migration tool temp directory.
#
# SYNOPSIS
# \ts
#   tcMigrateLib::tmpDir [dir]
# \te
#
# DESCRIPTION
# This procedure sets or gets the temp directory of the 'tcMigrateLib' library.
#
# It always returns the current temp directory of the 'tcMigrateLib' library.
#
# If the <dir> parameter is specified, then the the 'tcMigrateLib' temp
# directory is overwritten to use <dir> instead. If <dir> does not exist, it is
# created, but if the creation failed, old value is is used (and returned).
#
# The default values for the 'tcMigrateLib' library temp directory are :
# \ibs
# \ibi C:/temp on Windows hosts
# \ibi /tmp on Unixes hosts
# \ibe
#
# RETURNS: the temp directory used by the 'tcMigrateLib' library.
#
# ERRORS:
# \ibs
# \ibi 'Could not create directory'
# \ibe
#

proc tcMigrateLib::tmpDir {{dir ""}} {

    variable mTmpDir		;# temp directory

    set tmpDir [file join $dir]

    if {$dir != ""} {

	# a directory has been provided. Try to see if it can be used

	if {! [file isdirectory $tmpDir]} {

	    # if this temp dir exists, but is not a directory, just keep the
	    # old value ...

	    if {! [file exists $tmpDir]} {

		if {! [catch {file mkdir $tmpDir} status]} {
		    set mTmpDir $tmpDir
		}
	    }
	} else {
	    set mTmpDir $tmpDir
	}
    }

    return $mTmpDir
}

# ---------------------------- private procedures ---------------------------- #

################################################################################
#
# pathReplace - replace all occurences of <oldPath> in PATH with <newPath>
#
# SYNOPSIS
# \ts
#   pathReplace <oldPath> <newPath>
# \te
#
# PARAMETERS:
#   oldPath : the path fragment to be replace with <newPath>
#   newPath : path fragment to replace <newPath> with
#
# RETURNS: N/A
#
# EXAMPLE:
# \cs
#   pathReplace C:/view/myview/wind/river C:/view/anotherview/wpwr
# \ce
#

proc tcMigrateLib::pathReplace {oldPath newPath} {

    global tcl_platform

    variable mPath

    # if there is no PATH environment variable, just exit

    if {$oldPath == $newPath} {
	return
    }

    # now depending on the platform, we have to split the path on ";" (windows)
    # or ":" (Unixes)

    if {$tcl_platform(platform) == "windows"} {

	set sep 	";"
	set paths	[string tolower $mPath]
	set newPath	[string tolower [file join $newPath]]
	set oldPath	[string tolower [file join $oldPath]]

	# check again that neaPath is not equal to oldPath

	if {$newPath == $oldPath} {
	    return
	}

    } else {

	set sep 	":"
	set paths	$mPath
	set newPath	[file join $newPath]
	set oldPath	[file join $oldPath]
    }

    set splitted 	[split $paths $sep]
    set oldPathLen	[string length $oldPath]
    set newPathLen	[string length $newPath]
    set newPathList	{}

    # now look for <oldPath> in any of the PATH elements

    foreach path $splitted {

	set thisPath	[file join $path]
	set thisPathLen	[string length $thisPath]

	while {[set startIx [string first $oldPath $thisPath]] != -1} {

	    set start	[string range $thisPath 0 [expr {$startIx} - 1]]
	    set end	[string range $thisPath \
				      [expr {$startIx} + {$oldPathLen}] \
				      [expr {$thisPathLen}]]

	    set thisPath [format "%s%s%s" $start $newPath $end]
	}

	lappend newPathList [file nativename $thisPath]
    }

    set mPath		[join $newPathList $sep]
}
