# tcMigrateScripts.tcl - a collection of script commands to use by tcMigrate

# Copyright 2003-2005 Wind River Systems, Inc.

# modification history
# --------------------
# 01g,22jan05,fle  Fixed the way CPU and TOOL are read from a project
#                  + set logfile ID to -1 when log file is closed
# 01f,26nov04,fle  Fixed use of prjRelPath when using foundation scripts
#                  + fixed an error message
# 01e,06oct04,fle  Fixed resource dir path since foundation
# 01d,29jul04,fle  SPR#100083 : Fixed saved BSP dir may contain make macros
# 01c,14may04,fle  Fixed type in prjCreate when migrating applications
# 01b,13apr04,fle  Adapted to new resource file locations
# 01a,21may03,fle  written

namespace eval tcMigrateScripts {

    namespace export prjRead prjCreate

    variable	mLogFileId	-1
    variable	mLogFilePath	""

    global tcl_platform

    if {$tcl_platform(platform) == "windows"} {
	set mLogFilePath	[file join c:/ temp tcMigrateScripts.log]
    } else {
	set mLogFilePath	[file join ~ tmp tcMigrateScripts.log]
    }
}

# ---------------------------- public procedures ----------------------------- #

################################################################################
#
# tcMigrateScripts::prjRead - Get useful information for project file <prjFile>
#
# SYNOPSIS
# \ts
#   tcMigrateScripts::prjRead <prjFile> [<outFile>]
# \te
#
# DESCRIPTION
# This project migration facility allows to read a project file and to get some
# intersting information about it.
#
# If an <outFile> is provided, the results are also written to that output
# file. This allows to get rid of backslashes issues when getting result from
# a shell script call
#
# PARAMETERS:
#   prjFile : the project file this script should get information on
#   outFile : the output file where the values are saved
#
# RETURNS: A Tcl list made of :
# \ibs
# \ibi The project file path itself
# \ibi The BSP directory used by this project
# \ibi The CPU used by this project
# \ibi The toolchain used by this project
# \ibi The project type of this project
# \ibi The components list of this project
# \ibi The file list of this project (and related info).
# \ibe

proc tcMigrateScripts::prjRead {prjFile {outFile ""}} {

    global env

    set prjHandle	[::prjOpen $prjFile]
    set wcc		[::wccCurrentGet $prjHandle]

    set build		[::prjBuildCurrentGet $prjHandle]
    set toolchain	[::prjBuildTcGet $prjHandle $build]
    set bspDir		[::prjTagDataGet $prjHandle BSP_DIR]
    set coreInfoType	[::prjTagDataGet $prjHandle CORE_INFO_TYPE]
    set desc		[::prjTagDataGet $prjHandle userComments]
    set comps		[lsort [wccComponentListGet $prjHandle $wcc]]
    set files		[::prjFileListGet $prjHandle]

    if {[info command "Wind::tgtDir"] != "Wind::tgtDir"} {
	if {[info exists env(TGT_DIR)]} {
	    set tgtDir	[file join $env(TGT_DIR)]
	} else {
	    set tgtDir	[file join [Wind::base] target]
	}
    } else {
	set tgtDir	[file join [Wind::tgtDir]]
    }

    regsub -all -- {\$\(TGT_DIR\)} [file join $bspDir] $tgtDir bspDir

    array set fileDescs {}

    foreach fileName $files {

	set fileInfo	{}

	foreach infoType [::prjFileInfoListGet $prjHandle $fileName] {

	    if {$infoType != "dependencies"} {
		lappend fileInfo [ \
		    list $infoType [prjRelPath $prjHandle \
					       [::prjFileInfoGet $prjHandle \
								 $fileName \
								 $infoType]] \
		]
	    } else {
		lappend fileInfo [ \
		    list $infoType [::prjFileInfoGet $prjHandle $fileName \
						     $infoType] \
		]
	    }
	}

	set fileDescs([prjRelPath $prjHandle [file join $fileName]]) $fileInfo
    }

    # gather tags too

    array set tagsDescs	{}

    foreach tag [::prjTagListGet $prjHandle] {

	if {! [regexp {_dependencies$} $tag]} {
	    set tagsDescs([prjRelPath $prjHandle $tag]) \
		[prjRelPath $prjHandle [::prjTagDataGet $prjHandle $tag]]
	}
    }

    # get cpu and tool from toolchain namespace

    set cpu	[${toolchain}::cpu]
    set tool	[${toolchain}::tool]

    regsub -all -- {::prj_(.*)} $coreInfoType {\1} type

    # now we can close the old project

    ::prjClose $prjHandle

    set res [list [file join $prjFile] [file join $bspDir] $cpu $tool $type \
		  $desc $comps [array get fileDescs]]

    if {$outFile != ""} {
	catch {
	    set fileId	[open $outFile w]

	    set bspDir	[file join $bspDir]

	    puts $fileId "set prjFile \"[file join $prjFile]\""
	    puts $fileId "set bspDir \{$bspDir\}"
	    puts $fileId "set cpu \"$cpu\""
	    puts $fileId "set tool \"$tool\""
	    puts $fileId "set type \"$type\""
	    puts $fileId "set desc \"$desc\""
	    puts $fileId "set comps \[list $comps]"
	    puts $fileId "array set prjFiles \[list [array get fileDescs]]"
	    puts $fileId "array set tags \[list [array get tagsDescs]]"
	    close $fileId
	}
    }

    return $res
}

################################################################################
#
# tcMigrateScripts::prjDiff - do a diff between two projects
#

proc tcMigrateScripts::prjDiff {prjFile withPrjFile {outFile ""}} {

    global env

    set prjHandle	[::prjOpen $prjFile]
    set withPrjHandle	[::prjOpen $withPrjFile]

    set prjFiles	[prjRelPath $prjHandle [prjFileListGet $prjHandle]]
    set withPrjFiles	[prjRelPath $withPrjHandle \
				    [prjFileListGet $withPrjHandle]]

    # keep some project related values

    set bspDir	[prjTagDataGet $prjHandle BSP_DIR]
    set desc	[prjTagDataGet $prjHandle userComments]
    set tc	[prjBuildTcGet $prjHandle [prjBuildCurrentGet $prjHandle]]
    set cpu	[${tc}::cpu]
    set tool	[${tc}::tool]
    regsub -all -- {^::prj_} [prjTagDataGet $prjHandle CORE_INFO_TYPE] "" type

    # get the files which were added bu the user

    set depFiles	{}
    set errors		{}
    set warnings	{}
    set hostDir		[file join [wtxPath] host]

    if {$outFile != ""} {
	set outputDir	[file dirname $outFile]
    } else {
	set outputDir	[pwd]
    }

    array set addedFiles	{}
    array set dependencies	{}
    set ignoredTags		{}

    foreach fileName $prjFiles {

	if {[lsearch -exact $withPrjFiles $fileName] == -1} {

	    # get the full path of the file first

	    set fullPath	[prjFullPath $prjHandle $fileName]

	    # add all tags related to this added file to the tags to be ignored

	    foreach infoType [prjFileInfoListGet $prjHandle $fullPath] {
		lappend ignoredTags [format "FILE_%s_%s" $fileName $infoType]
	    }

	    # copy this file to the output directory

	    set destFile [uniqueFileNameGet [file join $outputDir \
						       [file tail $fileName]]]

	    if {[catch {file copy -force $fullPath $destFile} copyStatus] } {

		lappend errors "    *** ERROR *** : Could not copy\
				$fullPath to $destFile : $copyStatus"
	    }

	    set addedFiles([file tail $destFile]) $fileName

	    # now take care of its dependencies if any ...

	    set deps		[prjFileInfoGet $prjHandle $fullPath \
						dependencies]

	    foreach depFile $deps {

		set srcDepFile	[file join $depFile]

		# do not do anything for this file if it is already in the
		# project files

		if {([lsearch -exact [array names prjFiles] $depFile] == -1) &&
		    ([lsearch -exact $depFiles $srcDepFile] == -1)} {

		    if {[regsub -all -- "[file dirname $prjFile]/" $srcDepFile \
				"" depFile]} {

			# nothing to do here

		    } elseif {! [regexp -- $hostDir $srcDepFile]} {

			# ignore the host directory dependencies. They must
			# be some compiler dependencies ...
			#
			# issue a warning that this file will not be moved
			# to the destination directory

			set depFile	[file tail $srcDepFile]

			lappend warnings "*** Warning *** : dependency file\
					  [file nativename $srcDepFile] \
					  \n\t\t      copied to the migrated\
					  project directory"
		    } else {

			continue
		    }

		    set destDepFile	$depFile
		    set destFile [uniqueFileNameGet [file join $outputDir \
							       $destDepFile]]

		    set dependencies([file tail $destFile]) $depFile

		    if {[catch {file copy -force $srcDepFile $destFile}\
			       copyStatus] } {

			lappend errors "    *** ERROR *** : Could not copy\
					$srcDepFile to $destFile : $copyStatus"
		    } else {
			lappend depFiles	$srcDepFile
		    }
		}
	    }
	}
    }

    set removedFiles	{}

    foreach fileName $withPrjFiles {
	if {[lsearch -exact $prjFiles $fileName] == -1} {
	    lappend removedFiles $fileName
	}
    }

    # look at the components

    set prjWcc		[wccCurrentGet $prjHandle]
    set prjComps	[lsort [wccComponentListGet $prjHandle $prjWcc]]

    set withPrjWcc	[wccCurrentGet $withPrjHandle]
    set withPrjComps	[lsort [wccComponentListGet $withPrjHandle $withPrjWcc]]

    # get the added components

    set addedComps	{}

    foreach comp $prjComps {
	if {[lsearch -exact $withPrjComps $comp] == -1} {
	    lappend addedComps $comp
	}
    }

    # get the removed components

    set removedComps	{}

    foreach comp $withPrjComps {
	if {[lsearch -exact $prjComps $comp] == -1} {
	    lappend removedComps $comp
	}
    }

    # take care of the tags. Ignore the FILE ones, since they are already
    # taken into account by the files processing

    array set addedTags		{}
    array set removedTags	{}
    array set modifiedTags	{}

    set prjTags		[prjRelPath $prjHandle [::prjTagListGet $prjHandle]]
    set withPrjTags	[prjRelPath $withPrjHandle \
				    [::prjTagListGet $withPrjHandle]]

    foreach tag $prjTags {

	if {([lsearch -exact $withPrjTags $tag] == -1) &&
	    ([lsearch -exact $ignoredTags $tag] == -1)} {

	    set addedTags($tag) [prjRelPath $prjHandle \
					    [prjTagDataGet $prjHandle $tag]]
	} else {

	    set tagValue	[prjTagDataGet $prjHandle $tag]
	    set withTagValue	[prjTagDataGet $withPrjHandle $tag]

	    set relTagValue	[prjRelPath $prjHandle $tagValue]
	    set withRelTagValue	[prjRelPath $withPrjHandle $withTagValue]

	    # check if values differ

	    if {($tagValue != $withTagValue) && ($tag != "userComments") &&
		($tag != "PROJECT_FILES") &&
		(! [regexp "WCC_.*_COMPONENTS\$" $tag])} {

		set modifiedTags($tag) [list $relTagValue $withRelTagValue]
	    }
	}
    }

    foreach tag $withPrjTags {

	if {[lsearch -exact $prjTags $tag] == -1} {
	    set removedTags($tag) [ \
		prjRelPath $withPrjHandle [prjTagDataGet $withPrjHandle $tag] \
	    ]
	}
    }

    if {$outFile != ""} {
	catch {

	    set fileId	[open $outFile w]

	    set commonPrjFile	""
	    set splitted	[file split $prjFile]
	    set withSplitted	[file split $withPrjFile]
	    set ix		[expr {[llength $splitted]} - 1]
	    set withIx		[expr {[llength $withSplitted]} - 1]

	    while {[lindex $splitted $ix] == [lindex $withSplitted $withIx]} {

		if {$commonPrjFile == ""} {
		    set commonPrjFile [lindex $splitted $ix]
		} else {
		    set commonPrjFile [file join [lindex $splitted $ix] \
						 $commonPrjFile]
		}

		incr ix -1
		incr withIx -1
	    }

	    if {$commonPrjFile == ""} {
		set commonPrjFile	[file tail $prjFile]
	    }

	    puts $fileId "set prjFile \"$commonPrjFile\""
	    puts $fileId "set bsp \"[file tail $bspDir]\""
	    puts $fileId "set cpu \"$cpu\""
	    puts $fileId "set tool \"$tool\""
	    puts $fileId "set type \"$type\""
	    puts $fileId "set desc \"$desc\""

	    # components

	    puts $fileId	"set addedComps \[list $addedComps]"
	    puts $fileId	"set removedComps \[list $removedComps]"

	    # files

	    if {[llength [array names addedFiles]] != 0} {
		puts $fileId "array set addedFiles \[list [array get\
			      addedFiles]]"
	    } else {
		puts $fileId "array set addedFiles {}"
	    }

	    puts $fileId "set removedFiles \[list $removedFiles]"

	    if {[llength [array names dependencies]] != 0} {
		puts $fileId "array set dependencies \[list [array get\
			      dependencies]]"
	    } else {
		puts $fileId "array set dependencies {}"
	    }

	    # tags

	    if {[llength [array names addedTags]] != 0} {
		puts $fileId "array set addedTags \[list [array get addedTags]]"
	    } else {
		puts $fileId "array set addedTags {}"
	    }

	    if {[llength [array names modifiedTags]] != 0} {
		puts $fileId "array set modifiedTags \[list [array get \
			      modifiedTags]]"
	    } else {
		puts $fileId "array set modifiedTags {}"
	    }

	    if {[llength [array names removedTags]] != 0} {
		puts $fileId "array set removedTags \[list [array get \
			      removedTags]]"
	    } else {
		puts $fileId "array set removedTags {}"
	    }

	    # errors and warnings

	    if {[llength $errors] != 0} {
		puts $fileId	"set errors \[list $errors]"
	    } else {
		puts $fileId	"set errors {}"
	    }

	    if {[llength $warnings] != 0} {
		puts $fileId	"set warnings \[list $warnings]"
	    } else {
		puts $fileId	"set warnings {}"
	    }

	    close $fileId
	}
    }

    ::prjClose $withPrjHandle
    ::prjClose $prjHandle

    return

    set wcc		[::wccCurrentGet $prjHandle]

    set build		[::prjBuildCurrentGet $prjHandle]
    set toolchain	[::prjBuildTcGet $prjHandle $build]
    set bspDir		[::prjTagDataGet $prjHandle BSP_DIR]
    set coreInfoType	[::prjTagDataGet $prjHandle CORE_INFO_TYPE]
    set desc		[::prjTagDataGet $prjHandle userComments]
    set comps		[lsort [wccComponentListGet $prjHandle $wcc]]
    set files		[::prjFileListGet $prjHandle]

    if {[info command "Wind::tgtDir"] != "Wind::tgtDir"} {
	if {[info exists env(TGT_DIR)]} {
	    set tgtDir	[file join $env(TGT_DIR)]
	} else {
	    set tgtDir	[file join [Wind::base] target]
	}
    } else {
	set tgtDir	[file join [Wind::tgtDir]]
    }

    regsub -all -- {\$\(TGT_DIR\)} [file join $bspDir] $tgtDir bspDir

    array set fileDescs {}

    foreach fileName $files {

	set fileInfo	{}

	foreach infoType [::prjFileInfoListGet $prjHandle $fileName] {

	    if {$infoType != "dependencies"} {
		lappend fileInfo [ \
		    list $infoType [prjRelPath $prjHandle \
					       [::prjFileInfoGet $prjHandle \
								 $fileName \
								 $infoType]] \
		]
	    } else {
		lappend fileInfo [ \
		    list $infoType [::prjFileInfoGet $prjHandle $fileName \
						     $infoType] \
		]
	    }
	}

	set fileDescs([prjRelPath $prjHandle [file join $fileName]]) $fileInfo
    }

    # gather tags too

    array set tagsDescs	{}

    foreach tag [::prjTagListGet $prjHandle] {

	if {! [regexp {_dependencies$} $tag]} {
	    set tagsDescs([prjRelPath $prjHandle $tag]) \
		[prjRelPath $prjHandle [::prjTagDataGet $prjHandle $tag]]
	}
    }

    regsub -all -- {::tc_(.*)(gnu|diab)} $toolchain {\1} cpu
    regsub -all -- {::tc_(.*)(gnu|diab)} $toolchain {\2} tool
    regsub -all -- {::prj_(.*)} $coreInfoType {\1} type

    # now we can close the old project

    ::prjClose $prjHandle

    set res [list [file join $prjFile] [file join $bspDir] $cpu $tool $type \
		  $desc $comps [array get fileDescs]]

    if {$outFile != ""} {
	catch {
	    set fileId	[open $outFile w]

	    set bspDir	[file join $bspDir]

	    puts $fileId "set prjFile \"[file join $prjFile]\""
	    puts $fileId "set bspDir \{$bspDir\}"
	    puts $fileId "set cpu \"$cpu\""
	    puts $fileId "set tool \"$tool\""
	    puts $fileId "set type \"$type\""
	    puts $fileId "set desc \"$desc\""
	    puts $fileId "set comps \[list $comps]"
	    puts $fileId "array set prjFiles \[list [array get fileDescs]]"
	    puts $fileId "array set tags \[list [array get tagsDescs]]"
	    close $fileId
	}
    }

    return $res
}

################################################################################
#
# tcMigrateScripts::prjCreate - create a new project using given parameters
#
# SYNOPSIS
# \ts
#   tcMigrateScripts::prjCreate <prjFile> <bspDir> <description> <cpu> <tool>
#                                <type>
# \te
#
# DESCRIPTION
# This tools creates a new project in the parent directory of <prjFile>. If this
# parent directory does not exist, it is created.
#
# PARAMETERS:
#   prjFile : path of the new project file to create
#   bspDir : BSP directory to use to create the new project
#   description : short description for this new project
#   cpu : CPU of the new project to create
#   tool : toolchain to use to create the new project
#   type : either vxWorks, or vxApp
#
# RETURNS: N/A
#
# SEE ALSO
#
# EXAMPLES:
# \cs
# \ce
#

proc tcMigrateScripts::prjCreate {prjFile bspDir desc cpu tool type} {

    set prjDir	[file dirname $prjFile]

    # check that the new project parent directory exists

    if {! [file exists $prjDir]} {

	file mkdir $prjDir
    } elseif {! [file isdirectory $prjDir]} {

	# check that it is not a file

	error "File $prjDir is a regular file. Project file cannot be created\
	       in a file."
    }

    # now depending on the project type, rather call the vxWorks or vxApp
    # project creator

    if {$type == "vxWorks"} {

	# warning, depending on the Tornado version, the <cpu> and <tool>
	# arguments may not be available


	set versionStr	[wtxTsVersionGet]
	scan $versionStr "%f" version

	if {$version < 2.2} {
	    prj_vxWorks::create $prjFile $bspDir "" $desc
	} else {
	    prj_vxWorks::create $prjFile $bspDir "" $desc $cpu $tool
	}

    } elseif {$type == "vxApp"} {

	set tc	[format "::tc_%s%s" $cpu $tool]
	prj_vxApp::create $prjFile $tc
    } else {

	# wrong type

	error "Unknown project type : \"$type\". Should be \"vxWorks\" or\
	       \"vxApp\""
    }
}

################################################################################
#
# tcMigrateScripts::prjBuild - build all build specs and rules of a project
#
# SYNOPSIS
# \cs
#   tcMigrateScripts::prjBuild <prjFile>
# \ce
#
# DESCRIPTION
# This procedure opens the given <prjFile> project file. It then gets all its
# build specs, and build all rules for that build spec
#
# PARAMETERS:
#   prjFile : project file to build all build specs and rules for
#   outFile : output file where to save the project build failures
#
# RETURNS: A list of build specs/rules for which the build failed
#

proc tcMigrateScripts::prjBuild {prjFile {outFile ""}} {

    set buildFailures	{}

    # open the project

    set hProj	[::prjOpen $prjFile]

    # get the list of build specs

    foreach hSpec [::prjBuildListGet $hProj] {

	foreach rule [::prjBuildRuleListGet $hProj $hSpec] {

	    if {[catch {::prjBuild $hProj $rule $hSpec 1} status]} {

		lappend buildFailures [list $hSpec $rule $status]
	    }
	}

	# build clean to remove all what we created

	::prjBuild $hProj clean $hSpec
    }

    # close the project

    ::prjClose $hProj

    if {$outFile != ""} {

	set fileId	[open $outFile w]
	puts $fileId	"set buildFailures \[list $buildFailures]"
	close $fileId
    }

    return $buildFailures
}

# ---------------------------- private procedures ---------------------------- #

proc tcMigrateScripts::uniqueFileNameGet {filePath} {

    set fileName	[file tail $filePath]
    set dirName		[file dirname $filePath]
    set destFile	$fileName
    set ix		0

    while {[file exists [file join $dirName $destFile]]} {

	set destFile [format "%s_%i%s" [file root $fileName] $ix \
				       [file extension $fileName]]
	incr ix
    }

    return [file join $dirName $destFile]
}

proc tcMigrateScripts::log {msg} {

    variable mLogFileId
    variable mLogFilePath

    # check if the log file is opened or not

    if {$mLogFileId == -1} {

	set mLogFileId	[open $mLogFilePath "a"]
	set dateStr [clock format [clock seconds] -format \
			   "%a %b %d %Y at %H:%M:%S"]
	puts $mLogFileId "Opened at $dateStr"
    }

    puts $mLogFileId $msg
}

proc tcMigrateScripts::logClose {} {

    variable mLogFileId

    if {$mLogFileId != -1} {
	puts $mLogFileId "-----------------------------------------------------"
	catch {close $mLogFileId}
	set mLogFileId -1
    }
}

# ----------------------------- scripts wrapper ------------------------------ #

# check if we are in an old T2 install

set rcDir	[file join [wtxPath] host resource osconfig tcl]

if {! [file exists [file join $rcDir prjLib.tcl]]} {
    set rcDir [file join [wtxPath] host resource tcl app-config Project]
}

if {! [file exists [file join $rcDir prjLib.tcl]]} {

    # well, try with the founctaion path

    if {[info exist env(WIND_FOUNDATION_PATH)]} {
	set rcDir	[file join $env(WIND_FOUNDATION_PATH) resource \
				   osconfig tcl]
    }
}

source [file join $rcDir prjLib.tcl]
source [file join $rcDir prj_vxWorks.tcl]
source [file join $rcDir prj_vxApp.tcl]
source [file join $rcDir Utils.tcl]

# if prjRelPath does not exist, bound it to PrjUtils::relPath

if {[info procs prjRelPath] != "prjRelPath"} {

    proc prjRelPath {hProj files} {
	return [PrjUtils::relPath $files]
    }
}

if {$argc != 0} {

    # some arguments have been given to this library. Let's just try to execute
    # the given procedure
    #
    # first argument should be the procedure name, prepend it with the namespace
    # name

    set procName [format "tcMigrateScripts::%s" [lindex $argv 0]]
    set args [lrange $argv 1 [expr {[llength $argv]} - 1]]

    if {[catch {eval $procName $args} status]} {

	tcMigrateScripts::logClose
	error $status
    } else {

	puts stdout $status
    }
}

tcMigrateScripts::logClose
