# tc_cplus.tcl - project manager library
#
# Copyright 1998-2004 Wind River Systems, Inc.
#
# modification history
# --------------------
# 02q,23dec04,c_c  removed references to foundation.
# 02p,29sep04,fle  Generated tcInfo files when they are not there
# 02o,28sep04,fle  Consider that the tcInfo files are always up to date
# 02n,23sep04,fle  Do not cd to unexisting dir to generate tc info files
# 02m,01sep04,fle  Made it use Wind::foundation when needed
# 02l,11may04,fle  Regenerate tcInfo files if tcInfoGen.tcl is newer
# 02k,06may04,fle  Apigen related doc fixes
# 02j,01apr04,fle  Removed platform from tc file generation message
# 02i,25feb04,fle  Renamed gnuInfoGen.tcl in tcInfoGen.tcl
# 02h,16feb04,fle  Make it require PrjUtils package
#                  + removed most of the unused procedures
#                  + move toolchain namespace generation in tcInfoGen.tcl
#                  + moved archDirFind in Utils.tcl
# 02g,23jan04,fle  Rather use Wind::tgtDir when possible
# 02f,26nov03,fle  Added one tcInfo_ per host type
# 02e,19jun03,fle  Updated to new location
# 02d,23apr03,dbt  Added SIMLINUX support.
# 02c,28jan03,rbl  fix SPR 85176, tc_cplus.tcl contains logic error in error
#                  processing
# 02b,11jul02,rbl  fixing SPR 79510 - IDE can fail to build toolchain info
#                  (hence dead project facility) for no apparent reason...
# 02a,18jun02,rbl  fix problem where saved toolchain and bsp have wrong arch
#                  for current installation
# 01z,23may02,cjs  Fix spr 77715: put make diab toolchain default if
#                  installed, and remember last choice
# 01y,22may02,cjs  Moved filenameQuoteIfNecessary() to Utils.tcl
# 01x,22may02,cjs  add call to filenameQuoteIfNecessary() to protect
#                  temporary file containing spaces from eval()
# 01w,07may02,cjs  Continued support for spr 76601 -- expose -U (undefine)
#                  compiler option to mirror OPTION_DEFINE_MACRO
# 01v,14mar02,j_s  update cPrependChar for ARM,PENTIUM,STRONGARM and XSCALE
# 01u,17dec01,rbl  make tcInfo_* production robust against diab not being
#                  installed
# 01t,10dec01,rbl  dependency generation did not cope well with warnings
#                  commSymLink should only return symbol for the 68k arch.
# 01s,06nov01,rbl  diab now uses driver to invoke assembler, so it has
#                  preprocessor.
# 01r,15oct01,rbl  add functionality to map toolchains to arch's
#                  move archDirFind here from prj_vxWorks.tcl
#                  changes to deal with new "common" directory
#                  in target/h/tool
#                  get dependency generation working for diab
#                  ensure we are referring to env() array in root
#                  namespace ::
# 01q,17sep01,rbl  merge changes from T3: auto-generate tcInfo_<family>
#                  files whenever target/h/tool/<family> files change.
#                  in toolchains, rename "vendor" to "tool" for consistency
#                  with the makefile TOOL variable
#                  added .C to inputSuffixes for compiler
# 01r,09oct01,cjs  Fixing dependFiles() to propagate an error to the user
#                  also nuked -MG, which suppressed any errors about missing
#                  headers
# 01q,25sep01,dat  moved makefiles to h/tool/
# 01p,13jun01,agf  add logic to check host type on stdout/stderr syntax
# 01o,09may01,agf  undoing last change, @ breaks Win hosts
# 01n,16apr01,agf  fix syntax for redirecting output to stdout/stderr
# 01n,12mar01,sn   Ensure assembly files are correctly preprocessed
# 01m,10jan01,sn   Add support for OPTION_LANG_C
# 01m,12may00,zl   added Hitachi SuperH (SH) support
# 01l,16apr99,j_k  back-off from previous fix and make it right.
# 01k,13apr99,j_k  part of fix for SPR #26620.
# 01j,31mar99,cjs  Fixed bug in cppFlagsGet() that prevented proper -U undefs
#                  from being passed to CPP
# 01i,25jan99,ren  Fixing SPR 24302
# 01h,21jan99,cjs  Adding .cc to input suffixes for the C++ compiler
# 01g,26Oct98,ms  modified for DIAB toolchain, simplified interface.
# 01f,26aug98,ms  reworked so not GNU specific. Simpler interface to toolchain
#		  spec so people can change toolchains without writing TCL.
# 01e,12aug98,ren redefine simsparcsolaris and ppc as module level linking type.
# 01d,27jul98,ms  added nm, size, and ar utilities.
# 01d,22jul98,ms  added proc cpp.
# 01c,20jul98,ms  added cPrependChar and commSymLink methods to C++ toolchains.
# 01c,25jun98,ms  just include tcInfoGen.tcl, rather than copy it in.
# 01b,08apr98,ms  imported tcInfo for all CPUTOOLs (from tcInfoGen.tcl).
# 01a,26mar98,ms  written.
#
# DESCRIPTION
# This is the toolChain library C/C++ toolchains.
# It provides a TCL interface between the config tool
# and the specifics of each compiler.
#
# The file reads toolchain info is from any file named tcInfo_*.
# We provide a tcInfo_gnu file as an example. Our file is
# generated by tcInfoGen.tcl (see that file for details).
# Customers and 3rd party vendors can create their own
# toolchain interfaces as needed.
#
# Each toolchain (CPU-TOOL combo) is associated with a TOOL_FAMILY.
# The default TOOL_FAMILYs are 'gnu' and 'diab'.  A new TOOL_FAMILY
# is defined by creating a new directory in $TGT_DIR/h/tool/ with the
# name of the TOOL_FAMILY.
#
# Toolchains are defined by creating a makefile named make.$(CPU)$(TOOL)
# for each CPU and TOOL to be supported.
# The easiest way to start is by copying an existing file.
# Then one must create or update a tcInfo_${TOOL_FAMILY} file in this
# directory the info on all the toolchains supported by that tool family.
# That can be done either by running
# tcInfoGen, or by writing it by hand using an existing file
# as a template.
#
# There is a hard-coded assumption that the compiler flag "-c"
# produces a .o from a .c.
#

# This processes the tcInfo file for each tool family installed.
# It generates the tc database for all known toolchains.

#
# regenerate tcInfo_<family> (if they are writable)
# when any makefiles in target\h\tool\<family> have been changed
#

package require Wind

# get family list from dirs in target/h/tool

set tool_families ""

# tool families are all directories under target/h/tool (except for
# "common" that have make stubs in them

foreach file [glob -nocomplain [file join [Wind::tgtDir] h tool *]] {

    if {[file tail $file] == "common"} {
        continue
    }

    if {[file isdirectory $file]} {
        set makeStubs [glob -nocomplain $file/make.*]
	if {$makeStubs != ""} {
            lappend tool_families [file tail $file]
        }
    }
}

# read the toolchain information from the tcInfo_* files

set missingFiles	{}

foreach family $tool_families {

    set tcInfoFile	[tcInfoFileGet $family]

    if {![file exists $tcInfoFile]} {

	# generate it

	set genScript	[file join [Wind::base] host resource osconfig tcl \
				   tcInfoGen.tcl]

	if {[catch {exec [info nameofexecutable] $genScript $family >@ stdout} \
		   status]} {

	    puts [prjStderr] "Could not generated [file tail $tcInfoFile] :\
			      \"$status\""
	    lappend missingFiles	$tcInfoFile

	} else {
	    source $tcInfoFile
	}

        continue
    }

    source $tcInfoFile
}

# check if there was at least one tcInfo file

if {[llength $tool_families] == [llength $missingFiles]} {

    error "No toolchain info file was found"
}

################################################################################
#
# DependencyAnalyzer - The dependency analyser
#
# the dependency analyzer traverses through c or c++ code and returns
# a list of the headers that the c or c++ code depends on.  It caches 
# the data to avoid doing the same stuff twice.
#
# there are only two entry point into DependencyAnalyzer.
# they are DependencyAnalyzer::ResetCache and
# DependendencyAnalyzer::GenerateDependenciesFromFile.
#

namespace eval DependencyAnalyzer {

    variable _caches
    set _caches [list _sourceCache _badSourceCache _fileExistsCache]

    ############################################################################
    #
    # DependencyAnalyzer::ResetCache - empties the caches
    #
    # SYNOPSIS
    # \cs
    #   DependencyAnalyzer::ResetCache
    # \ce
    #
    # DESCRIPTION
    # Empties the caches, forcing the analyzer to re-examine files on disk next
    # time 'DependencyAnalyzer::GenerateDependenciesFromFile' is called.
    #

    proc ResetCache {} {
	variable _caches
	foreach cache $_caches {
	    variable $cache
	    catch {unset $cache}
	    array set $cache {}
	}
    }

    ResetCache

    variable _expIncludeOption

    # regular expression matches -I include directives on gcc command line

    set _expIncludeOption "-I\[ \t]*(\"\[^\"]+\"|'\[^']+'|\[^ \t\r\n]*)"

    # regular expression matches #include in C code

    set _expInclude "(^|\n|\r)\[ \t]*#include\[ \t]*"

    variable _expMacroInclude 

    # regular exp matches #include MACRO in c code.

    set _expMacroInclude \
	 "([set _expInclude]\[^<\" \t])|(#define\[^#\n]+#include)"
    variable _expLocalInclude 

    # regular exp matches $include "filename.h" in c code

    set _expLocalInclude "[set _expInclude]\"(\[^\"]+)\""
    variable _expFarInclude 

    # regular exp matches $include <filename.h> in c code
    set _expFarInclude "[set _expInclude]<(\[^>]+)>"
}


################################################################################
#
# DependencyAnalyzer::FileExists - check for file existency
#
# SYNOPSIS
# \cs
#   DependencyAnalyzer::FileExists <fullPath>
# \ce
#
# DESCRIPTION
# works just like tcl [file exists ...] and it memorizes (caches) the result
#

proc DependencyAnalyzer::FileExists { fullpath } {
    variable _fileExistsCache
    if {![info exists _fileExistsCache($fullpath)]} {
	set _fileExistsCache($fullpath) [file exists $fullpath]
    }
    set _fileExistsCache($fullpath)
}

################################################################################
#
# DependencyAnalyzer::CleanText - remove comment and line continuers in C code
#
# SYNOPSIS
# \cs
#   DependencyAnalyzer::CleanText <text>
# \ce
#
# DESCRIPTION
# Removes comments and \ line continuers in C or C++ code
#
# RETURNS: the cleaned code

proc DependencyAnalyzer::CleanText { text } {
    # remove all c++ comments
    regsub -all "(^|\r|\n)\[ \t]*//\[^\n]*(\n|\$)" $text {} text
    # remove all c comments
    # hex 81 is not a legal ascii character, so use it as an end
    # of comment marker
    regsub -all {\*/} $text \x81 text
    regsub -all "/\\*\[^\x81]*\x81" $text {} text
    regsub -all \x81 $text /* text
    # turn multiline preprocessor directives into singleline ones
    regsub -all "\\\\(\r?)\n" $text {} text
    set text
}

################################################################################
#
# DependencyAnalyzer::GenerateShallowDependenciesFromText - get shallowed deps
#
# SYNOPSIS
# \cs
#   DependencyAnalyzer::GenerateShallowDependenciesFromText <text>
# \ce
#
# DESCRIPTION
# Get a list of included files
#
# PARAMETERS:
#    text: the full text of a c or c++ file
#
# RETURNS: a list of the following structure, representing the includes of the
# file { <local|far> filename } {<local|far> filename} ... local means it was
# included with "", far means it was included with <>
#
# ERRORS:
# \is
# \i Text contains include macro
# if the text has a complex macro include
# Here's an example of what I mean by a complex macro include:
# \cs
# #define x "Jeffie.h"
# #include x
# \ce
# \ie
#

proc DependencyAnalyzer::GenerateShallowDependenciesFromText { text } {
    variable _expMacroInclude
    variable _expLocalInclude
    variable _expFarInclude

    set lResult [list]
    set text [CleanText $text]
    if {[regexp $_expMacroInclude $text]} {
	error "Text contains include macro."
    }

    # now remove all text except the includes
    # this keeps things O(n)
    # again, \x82 and \x83 are illegal ascii characters
    regsub -all "($_expLocalInclude)|($_expFarInclude)" \
	   $text \x83\\0\x82 text
    regsub -all "\x82\[^\x83]*\x83" "\x82$text\x83" "|" text
    # now split the remaining text into the list of include statements
    set listIncludes [split $text "|"]
    foreach include $listIncludes {
	if { [regexp $_expFarInclude $include x ws file] } {
	    lappend lResult [list far $file]
	} elseif { [regexp $_expLocalInclude $include x ws file] } {
	    lappend lResult [list local $file]
	}
    }
    set lResult
}

################################################################################
#
# DependencyAnalyzer::GenerateShallowDependenciesFromFile - generate deps
#
# SYNOPSIS
# \cs
#   DependencyAnalyzer::GenerateShallowDependenciesFromFile <fullPath>
# \ce
#
# DESCRIPTION
# Generate shallow dependencies from a file
#
# PARAMETERS:
#    fullpath: a filename naming a c or c++ source or header file
#
# RETURNS: a list in the form returned by GenerateShallowDependenciesFromText
#
# ERRORS:
# \is
# \i <fullpath> contains include macro
# \i cannot open file
# \ie
#

proc DependencyAnalyzer::GenerateShallowDependenciesFromFile { fullpath } {
    # look for dependencies in the cache
    set result [list ]
    variable _sourceCache
    if {[info exists _sourceCache($fullpath)]} {
	set result $_sourceCache($fullpath)
    } else {
	# it's not in the cache, so calculate
	if {[catch {
	    # use catch to prevent losing the file handle in event of error
	    set f [open $fullpath r]
	    set text [read $f]
	    close $f
	    } err]} {

	    catch {close $f}
	    error $err
	}

	if {[catch {set result [GenerateShallowDependenciesFromText $text]}]} {
	    variable _badSourceCache
	    set _badSourceCache($fullpath) {}
	    error "$fullpath contains include macro."
	} else {
	    # save result in cache
	    set _sourceCache($fullpath) $result
	}
    }

    set result
}


################################################################################
#
# DependencyAnalyzer::CombineDirectoriesAndFiles - combine dirs and files
#
# SYNOPSIS
# \cs
#   DependencyAnalyzer::CombineDirectoriesAndFiles <localDir> <farDirs> <files>
# \ce
#
# DESCRIPTION
# Combine directories and files
#
# PARAMETERS:
#    localDir: the directory in which the root c or c++ file resides
#    farDirs: a list of directories to search for other include files 
#    files: a list in the form returned by GenerateShallowDependenciesFromText
#
# RETURNS: A single list, with the full path of all the files.  If a file does
# not exist in any of the directories, then it is not in the return list.
#

proc DependencyAnalyzer::CombineDirectoriesAndFiles {localDir farDirs files} {

    set lFullPaths [list ]
    foreach fileinfo $files {
	set bIsLocal 0
	if {[lindex $fileinfo 0] == "local"} {
	    set filename [file join $localDir [lindex $fileinfo 1]]
	    if {[FileExists $filename]} {
		lappend lFullPaths $filename
		set bIsLocal 1
	    }
	}
	if {!$bIsLocal} {

	    # it's a far include so we need to look through all the far
	    # directories

	    catch { ;#to catch the break statement
		foreach dir $farDirs {
		    set filename [file join $dir [lindex $fileinfo 1]]
		    if {[FileExists $filename]} {
			lappend lFullPaths $filename
			break
		    }
		}
	    }
	}
    }

    set lFullPaths
}

################################################################################
#
# DependencyAnalyzer::GenerateDependenciesFromFile - generate deps from a file
#
# SYNOPSIS
# \cs
#   DependencyAnalyzer::GenerateDependenciesFromFile <flags> <fullPath>
# \ce
#
# DESCRIPTION
# Generate dependencies from a file
#
# PARAMETERS:
#    flags: a string containing the flags which will be passed to the 
#           c or c++ compiler
#    fullpath: an absolute path to the c or c++ source file for which
#              we want to generate dependencies
#
# RETURNS: A list of all existing header files that fullpath depends on.
#
# ERRORS:
# \is
# \i Could not open file
# \i <fullpath> contains include macro
# \ie
#
# NOTES: caches a lot of information so it never has to touch the file system
# more than once for any file.  To clear the cache, call ResetCache.
#

proc DependencyAnalyzer::GenerateDependenciesFromFile {flags fullpath} {
	  
    array set visited {}
	
    set farDirectories [ExtractDirectoriesFromFlags $flags]

    set localDirectory [file dirname $fullpath]
    if { $localDirectory == "" } {
	set localDirectory [pwd]
    }

    GenerateDependenciesFromFileRecursive $fullpath $localDirectory\
					  $farDirectories visited

    # remove the original .c file from visited because it's not
    # dependent on itself

    catch { unset visited($fullpath) }

    # remove the .c file from the cache, because it takes up a lot of
    # space and it is highly unlikely we'll see it again in this pass.

    variable _sourceCache
    catch { unset _sourceCache($fullpath) }
    array names visited
}

################################################################################
#
# DependencyAnalyzer::GenerateDependenciesFromFileRecursive - generate deps
#
# SYNOPSIS
# \cs
#   DependencyAnalyzer::GenerateDependenciesFromFileRecursive <fullpath> \
#           <localDirectory> <farDirectories> <varVisited>
# \ce
#
# DESCRIPTION
# Recursively generate dependencies from a file
#
# PARAMETERS:
#   fullpath: an absolute path to the c or c++ source or header file
#   localDirectory: the directory in which the source file resides
#   farDirectories: a list of directories in which to look for other header
#                   files
#   varVisited: the name the variable in the calling scope, which is an
#               array whos indices are files already visited.
#
# RETURNS: N/A
#
# ERRORS:
# \is
# \i Could not open file
# \i <fullpath> contains include macro
# \ie
#
# NOTES:   it adds fullpath to the visited array
#

proc DependencyAnalyzer::GenerateDependenciesFromFileRecursive \
    {fullpath localDirectory farDirectories varVisited } {

    # first check to see if this is a header with a #include MACRO
    variable _badSourceCache
    if {[info exists _badSourceCache($fullpath)]} {
	error "$fullpath contains include macro."
    }

    upvar $varVisited visited
    set visited($fullpath) {}

    set dependencies [GenerateShallowDependenciesFromFile $fullpath]

    set headers [CombineDirectoriesAndFiles $localDirectory \
					    $farDirectories $dependencies]

    # now recurse

    foreach header $headers {
	if {![info exists visited($header)]} {
	    GenerateDependenciesFromFileRecursive $header \
						  [file dirname $header] \
						  $farDirectories visited
	}
    }
}

################################################################################
#
# DependencyAnalyzer::ExtractDirectoriesFromFlags - extract dirs from flags
#
# SYNOPSIS
# \cs
#   DependencyAnalyzer::ExtractDirectoriesFromFlags <flags>
# \ce
#
# DESCRIPTION
# Extract directories from flags
#
# PARAMETERS:
#    flags: a string containing the compiler flags
#
# RETURNS: a list of all the include directories in the flags
#

proc DependencyAnalyzer::ExtractDirectoriesFromFlags {flags} {

    variable _expIncludeOption
    set lResult {}
	
    while { [regexp -- $_expIncludeOption $flags x includePath] } {
	lappend lResult [string trim $includePath {"'}] ;#"
	regsub -- $_expIncludeOption $flags {} flags
    }
    set lResult
}
