#!/bin/sh
# The next line restarts using tclsh \
exec tclsh "$0" "$@"

# xvxprj.tcl - UI for vxWorks project manipulations

# Copyright 2004 Wind River Systems, Inc.

# modification history
# --------------------
# 01a,23jun04,fle  written

#
# SYNOPSIS
#   tclsh cdfreader.tcl <wpjFile>
#
# DESCRIPTION
#
# This tcl script parses a wpj (Wind River Project) file given in argument and
# displays its components tree structure within a GUI tree.
#
# The GUI tree shows all the possible tree/node elements as returned by the
# '::reader::CdfReader' namespace.
#
# To start it, you will need a Tcl where the tcllib and BWidget packages
# are installed, and to copy the 'reader' directory in this tclsh's library
# directory. Library directory are either located in TCL_LIBRARY, or in
# <tclshpath>/../lib. After that, just run :
#
# tclsh cdfreader.tcl
#
# And it should work fine. Maybe using "wish" instead of "tclsh" should solve
# a non-starting application in some cases.
#

# get the Wind package, it should be in the ./lib directory

set scriptPath	[file dirname [info script]]
if {$scriptPath == "."} {
    set scriptPath	[pwd]
}

# add this script dir to the auto_path so that the local Tcl packages can be
# retrieved by the package require mechanism

lappend auto_path	[file join $scriptPath lib]

package require Wind

set libDir	lib
set libPrefix	lib
set libExt	.so

if {[Wind::hostType] == "x86-win32"} {
    set libDir		bin
    set libPrefix	""
    set libExt		.dll
}

# get all the tk*.(so|dll)

set libPath	[file join [file dirname [file dirname [info name]]] $libDir]
set libMatch	[format "%stk*%s" $libPrefix $libExt]

set tkLibList	[glob -nocomplain [file join $libPath $libMatch]]
if {[llength $tkLibList] == 0} {
    error "Could not find any Tk library in \"$libPath\""
}

load [lindex $tkLibList 0]

package require struct
package require reader
package require BWidget
package require OsConfig

namespace eval xvxprj {

    namespace export usage err
}

################################################################################
#
# initGuiTree - initialise the GUI tree <guiTree> with the <withTree>
#
# SYNOPSIS:
#   initGuiTree <guiTree> <withTree>
#
# DESCRIPTION
#
# Initialises guiTree (which is a BWidget::Tree) with structure tree <withTree>
# (which is a struct::tree as given by tcllib).
#

proc initGuiTree {guiTree withTree} {

    set rootNodes [$withTree children root]

    foreach rootNode $rootNodes {

	set type	[lindex [$withTree get $rootNode -key kind] 0]

	$guiTree insert end root $rootNode -drawcross auto \
		 -image [Bitmap::get $type] -open 0 \
		 -text [lindex [$withTree get $rootNode -key name] 0]

	# get the children from this root node

	set childNodes [$withTree children $rootNode]

	# now add those children nodes to the root node

	foreach childNode $childNodes {

	    addNode $childNode $rootNode $guiTree $withTree
	}
    }
}

################################################################################
#
# addNode - add given <node> in given <inNode> of <guiTree>
#
# This procedure adds the <node> node which belongs to <fromTree> ::struct::tree
# tree, into <inNode> node of the BWidget::Tree <guiTree>
#
# PARAMETERS
#   node : tree node to add to <guiTree>
#   inNode : node of <guiTree> to add <node> to
#   guiTree : the BWidget::Tree tree to add <node> to
#   fronTree : the struct::tree tree <node> belongs to
#
# RETURNS: N/A
#

proc addNode {node inNode guiTree fromTree} {

    set type	[lindex [$fromTree get $node -key kind] 0]

    if {[$fromTree isleaf $node]} {

	# draw this node as a leaf

	$guiTree insert end $inNode $node -drawcross auto \
		 -image [Bitmap::get $type] -open 0 \
		 -text [lindex [$fromTree get $node -key name] 0]
    } else {

	# draw this node a a folder, and insert child nodes in

	$guiTree insert end $inNode $node -drawcross auto \
		 -image [Bitmap::get $type] -open 0 \
		 -text [lindex [$fromTree get $node -key name] 0]

	# add all children of this node to the Gui Tree

	foreach childNode [$fromTree children $node] {

	    addNode $childNode $node $guiTree $fromTree
	}
    }

}

# ------------------------- xvxprj global procedures ------------------------- #

################################################################################
#
# xvxprj::usage - print this tool's help message in an info box and exit
#
# SYNOPSIS
# \cs
#   xvxprj::usage
# \ce
#
# DESCRIPTION
# This procedure prints out how to use this tool in an info tk message box, and
# exits with status '-1'
#
# PARAMETERS: N/A
#
# RETURNS: N/A
#
# ERRORS: N/A
#

proc xvxprj::usage {} {

    set msg "usage: tclsh xvxprj.tcl -help"
    append msg "\n                        <prjFile|cdfPath> \[<cdfPath>\[...]]"
    append msg "\nwhere:\n"
    append msg "\n    -help     : display this help message"
    append msg "\n    <cdfPath> : a directory path where to find CDF files. May\
		be specified"
    append msg "\n                more than once"
    append msg "\n    <prjFile> : path to the project file to get info from"

    puts stdout $msg

    exit -1
}

proc xvxprj::err {msg} {

    tk_messageBox -title "xvxprj error" -message $msg -type ok
}

# ------------------------- xvxprj main entry point -------------------------- #

# check arguments for the wpj file, and check that it exists

set prjFile	""
set cdfPath	{}

for {set ix 0} {$ix < $argc} {incr ix} {

    set arg	[lindex $argv $ix]

    switch -- $arg {

	-h	-
	--h	-
	-help	-
	--help	-
	help {
	    xvxprj::usage
	}

	-V {
	    xvxprj::verbose on
	}

	default {

	    if {([file exists $arg])} {
	    
		if {([file extension $arg] == ".wpj") && ($prjFile == "")} {
		    set prjFile		$arg
		} else {
		    lappend cdfPath	$arg
		}
	    } else {
		xvxprj::err "Unknown file : \"$arg\""
	    }
	}
    }
}

if {($prjFile == "") && ([llength $cdfPath] == 0)} {
    xvxprj::err "Please provide with a project file or cdf path. Exiting ..."
    xvxprj::usage
}

# some GUI initialisation

# create the tabbed notebook

set nb [NoteBook .nb -side bottom -bd 2]
set filesFrame	[$nb insert end vxWorksFiles -text "Files"]
set compsFrame	[$nb insert end vxWorksComponents -text "vxWorks"]
set buildsFrame	[$nb insert end vxWorksBuilds -text "Build"]

set sw [ScrolledWindow $compsFrame.sw -relief sunken -borderwidth 2]

set guiTree [Tree $sw.tree -relief flat -borderwidth 2 -width 50 -height 20 \
		  -highlightthickness 2 -redraw 0 -dropenabled 0 -dragenabled 0]

$sw setwidget $guiTree
pack $sw -side top -expand yes -fill both

if {$prjFile == ""} {
    set prjFile $cdfPath
}

# make the reader parse the wpj file, then get its struct tree

set reader [::reader::CdfReader]
$reader init $prjFile
set wpjTree [$reader tree]

# initialise the graphic tree with the Xml tree

$nb compute_size
pack $nb -fill both -expand yes -padx 4 -pady 4
$nb raise [$nb page 0]
initGuiTree $guiTree $wpjTree
$guiTree configure -redraw 1
pack $nb -fill both -expand yes -padx 4 -pady 4
$nb raise vxWorksComponents
