/* mmuV4eLib.h - mmu library for Coldfire V4e series chips */

/* 
Copyright (c) 2005, 2006 Wind River Systems, Inc. 

The right to copy, distribute, modify or otherwise make use 
of this software may be licensed only pursuant to the terms 
of an applicable Wind River license agreement. 
*/

/*
modification history
--------------------
01g,03may06,rec  SPR-120512 - use derived ASID in TLB_WRITE macros
01f,26apr06,h_k  changed MMU_REGS to volatile.
01e,28feb06,rec  defer support for page size optimization
01d,13feb06,rec  fix invalid state definition
01c,31jan06,rec  change args to mmuV4eTlbMissHandler
01b,28nov05,rec	 supervisor pools must be TLB miss-proof
01a,19sep05,rec	 written
*/

/*
DESCRIPTION

This header contains the public interface to the Coldfire V4e Core Memory
Management Library. It also defines private (internal) structures and
macros used to manipulate the MMU.

Theory: the vmBaseLib and vmLib provide an application programmer
interface to the memory management and protection features of the V4e
MMU. With the help of the aimMmuLib and mmuV4eLib layers, a data
structure is maintained which stores per-page attributes: validity,
protection, cacheability, and address translation. This data structure
represents each page as a page table entry (PTE).

The V4e MMU hardware maintains a cache of 32 entry ITLB and 32 entry
DTLB Translation Lookaside Buffer (TLB) Entry registers.  When the processor 
accesses a memory location and fails to find matching TLB entry, it calls an
interrupt handler to establish a mapping. That miss handler walks the
data structure and copies a PTE into a TLB Entry register.

The V4e MMU cannot be disabled, but several TLB Entry registers are
reserved at boot time to establish a static memory mapping that supports
low-level interrupt, exception, and MMU page management activities. This
is configured in applications via the sysStaticTlbDesc[] memory array.
The remainder of the TLB Entry registers are available to the
vmLib/vmBaseLib mapping, established via sysPhysMemDesc[].

This file provides data needed to coordinate the mmuV4eLib layer with
the aimMmuLib and vmLib/vmBaseLib layers, as well as definitions needed
to configure the sysStaticTlbDesc[] array.

See also:
    VxWorks Coldfire Architecture Supplement, WindRiver
    MCF5485 Preliminary Reference Manual, Motorola

*/

#ifndef __INCmmuV4eLibh
#define __INCmmuV4eLibh

#ifdef __cplusplus
extern "C" {
#endif

#ifndef	 _ASMLANGUAGE
#include <aimMmuLib.h>
#include <vmLib.h>
#endif	/* _ASMLANGUAGE */

/* defines */

#define LOCK_API	/* include the TLB lock functions */

#define P_MMUREGS ((volatile UINT *) &MMU_REGS) /* pointer to MMUBAR */

/* paragraph references are to the MCFV4e Preliminary Reference Manual */

/* 
 * MMU Fault Status Encoding definitions
 * (ref Par 5.4.3)
 */
 
#define TLB_MISS_DATA_BIT	0x08000000
#define TLB_MISS_EXT_BIT	0x00020000
#define TLB_MISS_FORMAT_MASK	0x0c030000
#define TLB_MISS_INST_FORMAT	0x04010000
#define TLB_MISS_EXTEND_FORMAT	0x04020000
#define TLB_MISS_DAT_WRT_FORMAT 0x08020000
#define TLB_MISS_DATA_FORMAT	0x0c020000


/* 
 * MMU Base Address Register bit definitions
 * (ref Par 5.5.3.1)
 */
 
#define MMUBAR_V    0x00000001	/* MMU contents valid */
#define MMUBAR_MASK 0xFFFF0000	/* MMU base address mask */

/* 
 * MMU Register Offset definitions
 * The following register offsets are relative to MMUBAR
 * (ref Par 5.5.3.2)
 */
 
#define MMUCR_OFFSET	   0x00000000  /* MMU control register */
#define MMUOR_OFFSET	   0x00000004  /* MMU operations register */
#define MMUSR_OFFSET	   0x00000008  /* MMU status register */
#define MMUAR_OFFSET	   0x00000010  /* MMU fault, test, or TLB address register */
#define MMUTR_OFFSET	   0x00000014  /* MMU read/write TLB tag register */
#define MMUDR_OFFSET	   0x00000018  /* MMU read/write TLB data register */

/* indexes for accessing the MMU registers */

#define MMUCR	    0x00000000	/* MMU control register */
#define MMUOR	    0x00000001	/* MMU operations register */
#define MMUSR	    0x00000002	/* MMU status register */
#define MMUAR	    0x00000004	/* MMU fault, test, or TLB address register */
#define MMUTR	    0x00000005	/* MMU read/write TLB tag register */
#define MMUDR	    0x00000006	/* MMU read/write TLB data register */
/* 
 * MMU Control Register bit definitions
 * (ref Par 5.5.3.3)
 */
 
#define MMUCR_EN    0x00000001	/* Virtual Mode enable */
#define MMUCR_ASM   0x00000002	/* Address Space Mode, 1 = ASID tags enabled */

/*
 * MMU Operations Register 
 * (ref Par 5.5.3.4)
 */

#define MMUOR_UAA   0x00000001	/* update allocations address */
#define MMUOR_ACC   0x00000002	/* MMU TLB access */
#define MMUOR_RW    0x00000004	/* read/write (1 = read) */
#define MMUOR_ADR   0x00000008	/* TLB Address select (0= TLB alloc, 1=MUAR) */
#define MMUOR_ITLB  0x00000010	/* ITLB operation (0=DTLB, 1=ITLB) */
#define MMUOR_DTLB  0x00000000 
#define MMUOR_CAS   0x00000020	/* clear all non-locked TLBs with ASID match */
#define MMUOR_CNL   0x00000040	/* clear all non-locked TLB entries */
#define MMUOR_CA    0x00000080	/* Clear all TLB entries */
#define MMUOR_STLB  0x00000100	/* search TLB */
#define MMUOR_AAMSK 0xFFFF0000	/* TLB allocation address mask */

/*
 * MMU Status Register
 * (ref Par 5.5.3.5) 
 */
#define MMUSR_HIT   0x00000002	/* search succeeded. (1=hit) */
#define MMUSR_WF    0x00000008	/* write access fault (1=write protect fault) */
#define MMUSR_RF    0x00000010	/* read access fault (1=read protect fault) */
#define MMUSR_SPF   0x00000020	/* supervisor protect fault (1=supervisor) */
 
/*
 * MMU Fault, Test, or TLB Address Register
 * There are no bit fields in this register.
 * (ref Par 5.5.3.6) 
 */


/*
 * MMU Read/Write Tag Register
 * (ref Par 5.5.3.7) 
 */
#define MMUTR_V	    0x00000001	/* valid */
#define MMUTR_SG    0x00000002	/* shared global for all ASIDs (1=shared) */
#define MMUTR_ASID  0x0000037C	/* ASID mask */
#define MMUTR_VAMSK 0xFFFFFC00	/* virtual address mask */
#define MMUTR_ASIDSHFT	     2	/* ASID shift left 2 bits */
#define MMUTR_VASHF	    10	/* virtual address shift left 10 bits */
#define MMUTR_AASHF	    16	/* allocation address shift left 16 bits */

/*
 * MMU Read/Write Data Entry Register
 * (ref Par 5.5.3.7) 
 */
#define MMUDR_RV    0x00000001	/* reserved (must be zero) */ 
#define MMUDR_LK    0x00000002	/* locked entry (1=locked) */
#define MMUDR_X	    0x00000004	/* execute enable */
#define MMUDR_W	    0x00000008	/* write enable */
#define MMUDR_R	    0x00000010	/* read enable */
#define MMUDR_SP    0x00000020	/* supervisor protect (no user access) */
#define MMUDR_CMWT  0x00000000	/* cache mode write-through */
#define MMUDR_CMCB  0x00000040	/* cache mode copyback*/
#define MMUDR_CMPR  0x00000080	/* cache mode non-cacheable precise */
#define MMUDR_CMIM  0x000000C0	/* cache mode non-cacheable imprecise */
#define MMUDR_1M    0x00000000	/* page size 1M */
#define MMUDR_4K    0x00000100	/* page size 4K */
#define MMUDR_8K    0x00000200	/* page size 8K */
#define MMUDR_1K    0x00000300	/* page size 1K */
#define MMUDR_SZMSK 0x00000300	/* page size mask */
#define MMUDR_PSMSK 0xFFFFFC00	/* physical address mask */
#define MMUTR_PASHF	    10	/* physical address shift left 10 bits */

/* MMU_PAGE_SIZES_ALLOWED are page sizes supported by the Coldfire MMU */

#ifdef MMUV4E_PAGE_OPTIMIZATION
#define MMU_PAGE_SIZES_ALLOWED	(MMU_PAGE_MASK_4K | \
				 MMU_PAGE_MASK_8K | \
				 MMU_PAGE_MASK_1M )


#define MMU_MAX_PAGE_SIZES	3
#else
#define MMU_PAGE_SIZES_ALLOWED	(MMU_PAGE_MASK_4K)


#define MMU_MAX_PAGE_SIZES	1
#endif
#define MMU_TLB_WORDS		2 /* words in TLB */
#define MMU_PTE_WORDS		2 /* words in PTE */


#define MMU_ITLB_ENTRIES       32 /* number of ITLB entries  */
#define MMU_DTLB_ENTRIES       32 /* number of DTLB entries  */


#define MMU_NUM_ASID	      256
#define MMU_NUM_CONTEXT	      MMU_NUM_ASID

/* Page table sizes */

#define MMU_REGION_TBL_SIZE	1024	/* number of region table entries */
#define MMU_BYTES_PER_ENTRY	8	/* Num bytes per page table entry */

#define TLB_4K_VADDR_MASK	0x003FF000
#define TLB_8K_VADDR_MASK	0x003FE000
#define TLB_1M_VADDR_MASK	0x00300000

#define TLB_4K_PAGE_SIZE_MASK	0x00000000
#define TLB_8K_PAGE_SIZE_MASK	0x00002000
#define TLB_1M_PAGE_SIZE_MASK	0x001FE000

#define PTE_ADR_MASK		0xFFFFF000  /* to get address from PTE */
#define PTE_ASID_MASK		0x000003FC  /* to get address from PTE */

/* defines for operating on pte.field.sz fields */

#define PTE_1M_PAGE_SZ		0
#define PTE_4K_PAGE_SZ		1
#define PTE_8K_PAGE_SZ		2
#define PTE_1K_PAGE_SZ		3



#define TLB_1M_TABLE_SIZE	4
#define TLB_8K_TABLE_SIZE	(128 * TLB_1M_TABLE_SIZE)
#define TLB_4K_TABLE_SIZE	(2 * TLB_8K_TABLE_SIZE)

#define TLB_4K_PAGE_SIZE	0x00001000
#define TLB_8K_PAGE_SIZE	0x00002000
#define TLB_1M_PAGE_SIZE	0x00100000

#define MMU_PTE_SIZE_SHIFT	3   /* 8 bytes */
#define TLB_1M_VADDR_SHIFT	(20 - MMU_PTE_SIZE_SHIFT)
#define TLB_8K_VADDR_SHIFT	(13 - MMU_PTE_SIZE_SHIFT)
#define TLB_4K_VADDR_SHIFT	(12 - MMU_PTE_SIZE_SHIFT)


/* Cache Table Indexes */

#define MMU_INVALID	0
#define MMU_CACHEOFF	1
#define MMU_CPYBACK	2
#define MMU_WRTHRU	3
#define MMU_COHRNT	4

/* 
 * arch dependent descriptions
 * These macros operate on the MMU_STATE structure.  (see below)
 */

#define MMU_STATE_VALID_STATE	    (MMUTR_V << 16)
#define MMU_STATE_USER_ACCESS_NOT   MMUDR_SP
#define MMU_STATE_READABLE	    MMUDR_R
#define MMU_STATE_WRITABLE	    MMUDR_W
#define MMU_STATE_EXECUTABLE	    (MMUDR_R | MMUDR_X)
#define MMU_STATE_USER_ACCESS	    (0)
#define MMU_STATE_LOCKED	    MMUDR_LK
#define MMU_STATE_INVALID_STATE	    (~MMU_STATE_VALID_STATE) /* must not be 0 */

/* arch dependent state used for supervisor stack locking */

#define MMU_STATE_SUPV_STACK_LK	   MMU_STATE_USER_ACCESS_NOT | \
			MMU_STATE_READABLE | MMU_STATE_WRITABLE


#define MMU_STATE_VALID_NOT	    0
#define MMU_STATE_VALID		    MMU_STATE_VALID_STATE

#define MMU_STATE_MASK_PROTECTION   (MMUDR_SP | MMUDR_R | MMUDR_W | MMUDR_X)
#define MMU_STATE_MASK_VALID	    (MMUTR_V << 16)
#define MMU_STATE_MASK_CACHEABLE    (MMUDR_CMCB | MMUDR_CMPR | MMUDR_CMIM)

/* Reserved bit used for ISR Callable */

#define MMU_STATE_MASK_ISR_CALLABLE MMUDR_RV
#define MMU_STATE_ISR_CALLABLE	    MMUDR_RV

/* max size (in bytes) of the lock table used to lock down task stacks */

#define MMU_STACK_LOCK_TABLE_SIZE   MMU_DTLB_ENTRIES * MMU_TLB_WORDS * 4

#ifndef _ASMLANGUAGE

#define MMUV4E_DISABLE func_mmuV4eCrWrite (0)
#define MMUV4E_ENABLE  func_mmuV4eCrWrite (MMUCR_EN | MMUCR_ASM)
#define MMUV4E_ENABLED ((P_MMUREGS [MMUCR] & MMUCR_EN) != 0)

/* the following assumes 4k pages */

#define ADDRESS_TO_PAGETABLE_INDEX(arg) ((arg >> 22) & 0x3ff)
#define ADDRESS_TO_PTE_INDEX(arg)   ((arg >> 12) & 0x3ff)

#define _WRS_NO_BITS_PHYS_ADDR	  32

/* clear ALL TLB entries */

#define MMU_TLB_CLEAR P_MMUREGS [MMUOR] = MMUOR_CA;  \
		      P_MMUREGS [MMUOR] = MMUOR_CA | MMUOR_ITLB 

#define MMU_TLB_CLEAR_UNLOCKED P_MMUREGS [MMUOR] = MMUOR_CNL;  \
		      P_MMUREGS [MMUOR] = MMUOR_CNL | MMUOR_ITLB 

/* write the TLB entry using the allocation address */

#define MMU_PTE_WRITE_D P_MMUREGS [MMUOR] = \
    MMUOR_ACC | MMUOR_UAA

#define MMU_PTE_WRITE_I P_MMUREGS [MMUOR] = \
    MMUOR_ACC | MMUOR_UAA | MMUOR_ITLB
     
#define TLB_WRITE_D(arg,asid) \
		    P_MMUREGS [MMUTR] = (arg)->words.word0 | asid; \
		    P_MMUREGS [MMUDR] = (arg)->words.word1; \
		    MMU_PTE_WRITE_D

#define TLB_WRITE_I(arg,asid) \
		    P_MMUREGS [MMUTR] = (arg)->words.word0 | asid; \
		    P_MMUREGS [MMUDR] = (arg)->words.word1; \
		    MMU_PTE_WRITE_I

#define TLB_SEARCH_D(arg) \
		    P_MMUREGS [MMUAR] = (arg)->words.word0 & MMUOR_AAMSK; \
		    P_MMUREGS [MMUOR] = MMUOR_ADR | MMUOR_STLB
		    
#define TLB_SEARCH_I(arg) TLB_SEARCH_D(arg) | MMUOR_ITLB

#define TLB_FOUND ((P_MMUREGS [MMUSR] & MMUSR_HIT) != 0)

#define TLB_SET_INVALID(aa) P_MMUREGS [MMUTR] = ~MMUTR_V; \
		    P_MMUREGS [MMUAR] = aa; \
		    P_MMUREGS [MMUOR] = MMUOR_ACC | MMUOR_ADR

#define TLB_SET_INVALID_D P_MMUREGS [MMUTR] = P_MMUREGS [MMUTR] & ~MMUTR_V; \
		    P_MMUREGS [MMUOR] = MMUOR_ACC | MMUOR_ADR

#define TLB_SET_INVALID_I TLB_SET_INVALID_D | MMUOR_ITLB
		   
/* 
 * Translation Buffer structure
 * MMU Read/Write Tag Register (ref Par 5.5.3.7) 
 * MMU Read/Write Data Entry Register(ref Par 5.5.3.7) 
 *
 * This structure is also used as the Page Table Entry
 *
 * The last bit, "isrc" is used as the ISR Callable bit by
 * the PTE and is not used by the TLB entry.
 */

typedef union mmuV4e_tlb	/* Page Table Entry format */
    {
    struct			/* Bit field description */
	{
	/* word 0 */
	u_int	va:22;		/* virtual address */
	u_int	id:8;		/* address space ID (ASID) */
	u_int	sg:1;		/* shared global */
	u_int	 v:1;		/* valid */
	/* word 1 */
	u_int	pa:22;		/* physical address */
	u_int	sz:2;		/* page size */
	u_int	cm:2;		/* cache mode */
	u_int	sp:1;		/* supervisor protect */
	u_int	 r:1;		/* read access enable */
	u_int	 w:1;		/* write access enable */
	u_int	 x:1;		/* execute access enable */
	u_int	lk:1;		/* lock */
	u_int isrc:1;		/* isr callable flag (must be zero in TLB) */
	} field;

    struct 
	{
	UINT32 word0;		/* word 0 */
	UINT32 word1;		/* word 1 */
	} words;

    } MMUV4E_TLB;
    
/* the PTE matches the hardware TLB */
    
typedef MMUV4E_TLB MMUV4E_PTE;

/* 
 * mmu state structure.	 This is a compressed version of the PTE.
 * It repacks the bit fields so that attributes fit in one word.
 * The last bit, "isrc" is not used by the TLB entry is is used
 * as the ISR callable bit.
 * This is used by the AIM when accessing functions such as mmuV4ePteSet.
 * The AIM isn't aware of the contents of this structure, so it passes it
 * as an integer.
 */

typedef struct mmuV4e_state
    {
	u_int  sp1:6;		/* unused */
	u_int  sp2:8;		/* unused */
	u_int	sg:1;		/* shared global */
	u_int    v:1;		/* valid */
	u_int  sp3:6;		/* unused */
	u_int	sz:2;		/* page size */
	u_int	cm:2;		/* cache mode */
	u_int	sp:1;		/* supervisor protect */
	u_int	 r:1;		/* read access enable */
	u_int	 w:1;		/* write access enable */
	u_int	 x:1;		/* execute access enable */
	u_int	lk:1;		/* lock */
	u_int isrc:1;		/* isr callable */
    } MMUV4E_STATE;

/*
 * This is the structure used to configure the mmu library
 */

typedef struct mmuColdfireConfigStruct
    {
    int	 nTlbEntries;	/* Number of Entries in TLB */
    
    UINT minPageSize;	/* minimum size of pages, 4k or greater */
    
    } MMU_COLDFIRE_CONFIG;

IMPORT MMU_COLDFIRE_CONFIG mmuV4eConfig;
IMPORT char * kernelTaskStackPartAdr;
IMPORT PART_ID kernelTaskStackPartId;

IMPORT volatile UINT    *MMU_REGS;	   /* MMUBAR */

/* mmuV4eLib.c */

IMPORT STATUS	mmuV4eLibInit (void);
IMPORT void	mmuV4eMmubarSet (char *addr);  /* set MMUBAR */
IMPORT void	mmuV4eTaskMmuCtxSet (void);
IMPORT void	mmuV4eContextSet (MMU_CONTEXT_TBL * pTransTbl,
				int context); /* set ASID */
IMPORT UINT	mmuV4eContextGet (void);       /* get ASID of running task */
IMPORT void	mmuV4eCrWrite (int value);     /* write to MMUCR */
IMPORT void	mmuV4eTlbShow (void);
IMPORT STATUS	sysSuperStackPartCreate (void);
IMPORT void	mmuV4eAsidSet (int asid);  /* used to set mmu context */

/* mmuV4eALib.s */

IMPORT STATUS	mmuV4eTlbMissHandler (int dummy1, int dummy2, int dummy3,
                    int dummy4, int fmtStatus, char * esfAddress); 

#endif	/* _ASMLANGUAGE */

#ifdef __cplusplus
}
#endif

#endif /* __INCmmuV4eLibh */
