/**
   \file        dio5200.h
   \author      Charles Crain
   \date        04/10/2006

   COPYRIGHT 2006. NATIONAL INSTRUMENTS. ALL RIGHTS RESERVED.
*/

#ifndef ___ni_arch_ppc_dio5200_h___
#define ___ni_arch_ppc_dio5200_h___

#define DIO_5200_INPUT            0x1 /* input pin */
#define DIO_5200_OUTPUT           0x2 /* output pin */
#define DIO_5200_PWM              0x4 /* pulse width modulation capable */
#define DIO_5200_INPUT_CAPTURE    0x8 /* supports input capture mode */

#define DIO_5200_IC_TYPE_MASK     0xF /* Mask for input capture mode types */
#define DIO_5200_IC_RISING_EDGE   0x1 /* Time between rising edges */
#define DIO_5200_IC_FALLING_EDGE  0x2 /* Time between falling edges */
#define DIO_5200_IC_PULSE         0x3 /* Time between any 2 edges */

#define DIO_5200_IC_REARM_MASK    0xF0 /* Mask for IC rearm type */
#define DIO_5200_IC_RESET         0x10 /* Reset timer on each event */
#define DIO_5200_IC_ACCUMULATE    0x20 /* Accumulate timer */

#ifdef __cplusplus
extern "C" {
#endif

   /**
      Initialize the library.

      Configures the board's hardware accordingly as well.
   */
   void dio5200Init();

   /**
      Query the number of digital inputs and outputs.

      \return the total number of pins on the device, including
      digital inputs, outputs, and input/output pins.
   */
   unsigned int dio5200GetPinCount();

   /**
      Get the mode(s) supported by the specified pin.

      \param pin [in] The index of the pin of interest.  Should be
      between 0 and 1 less than the result of dio5200GetPinCount().

      \return Can be DIO_5200_INPUT, DIO_5200_OUTPUT, DIO_5200_PWM, or
      DIO_5200_INPUT_CAPTURE, or a bitwise
      combination.  If the pin supports one mode, it
      is in that mode by default (of course).  If it supports
      two modes, it must be configured with dio5200SetPinMode()

      \pre 0 <= pin < dio5200GetPinCount()
   */
   unsigned int dio5200GetPinCapabilities(unsigned int pin);

   /**
      Set the pin for general-purpose input or output.

      \param pin [in] The index of the pin of interest.  Should be
      between 0 and 1 less than the result of dio5200GetPinCount().
      \param mode [in] One of DIO_5200_INPUT or DIO_5200_OUTPUT.  Must be
      a mode supported by the pin from dio5200GetSupportedModes() otherwise
      this function has no effect.

      \pre 0 <= pin < dio5200GetPinCount()
   */
   void dio5200SetPinMode(unsigned int pin, unsigned int mode);

   /**
      Gets the current operating mode of a pin

      \param pin [in] The index of the pin of interest.  Should be
      between 0 and 1 less than the result of dio5200GetPinCount().

      \return One of DIO_5200_INPUT, DIO_5200_OUTPUT, DIO_5200_PWM, or
      DIO_5200_INPUT_CAPTURE.

      \pre 0 <= pin < dio5200GetPinCount()
   */
   unsigned int dio5200GetPinMode(unsigned int pin);
   
   /**
      Write to a general purpose output pin.

      If the pin is not in DIO_5200_OUTPUT mode, this function can
      have undefined results.

      \param pin [in] The index of the pin of interest.  Should be
      between 0 and 1 less than the result of dio5200GetPinCount().
      \param on [in] If nonzero, the pin is turned on.
      
      \pre 0 <= pin < dio5200GetPinCount()
   */
   void dio5200WritePin(unsigned int pin, int on);

   /**
      Read from a general purpose input pin.

      \param pin [in] The index of the pin of interest.  Should be
      between 0 and 1 less than the result of dio5200GetPinCount().
      
      \return 0 if off, 1 if on.
      
      \pre 0 <= pin < dio5200GetPinCount()
   */
   int dio5200ReadPin(unsigned int pin);

   /**
      Start pulse-width modulation on the specified pin.

      \param pin [in] The index of the pin of interest.  Should be
      between 0 and 1 less than the result of dio5200GetPinCount().
      \param ticksOn [in] Number of prescaled clock ticks to keep the pin on
      during the cycle.
      \param ticksOff [in] Number of prescaled clock ticks to keep the pin off
      during the cycle.
      \param prescale [in] Number of counts to prescale by.  The ticks are
      multiplied by this value effectively.  On some hardware, zero is
      a synonym for 2 to the power of the presacaler bit width.  On
      pins with no prescaler, this is ignored.

      \pre 0 <= pin < dio5200GetPinCount() 
      \pre (ticksOn + ticksOff) less than 2 raised to the power of the
      timer bit width from dio5200GetClockParams()
      \pre prescaler less than 2 raised to the power of the
      prescaler bit width from dio5200GetClockParams()
   */
   void dio5200PWM(unsigned int pin,
                   unsigned short ticksOn,
                   unsigned short ticksOff,
                   unsigned short prescale);

   /**
      Start input capture on the specified pin.

      \param pin [in] The index of the pin of interest.  Should be
      between 0 and 1 less than the result of dio5200GetPinCount().
      \param prescale [in] Number of counts to prescale by.  The ticks are
      multiplied by this value effectively.  On some hardware, zero is
      a synonym for 2 to the power of the presacaler bit width.  On
      pins with no prescaler, this is ignored.
      \param mode [in] One of DIO_5200_IC_RISING_EDGE, DIO_5200_IC_FALLING_EDGE,
      or DIO_5200_IC_PULSE, combined bitwise with one of DIO_5200_IC_ONE_SHOT or
      DIO_5200_IC_RECURRING.

      \pre 0 <= pin < dio5200GetPinCount()
      \pre prescaler less than 2 raised to the power of the
      prescaler bit width from dio5200GetClockParams()
   */
   void dio5200InputCaptureStart(unsigned int pin, 
                                 unsigned short prescale,
                                 int mode);

   /**
      Returns non-zero if an input capture event has occurred.

      \param pin [in] The index of the pin of interest.  Should be
      between 0 and 1 less than the result of dio5200GetPinCount().
      \param overflow [out] If non-NULL, on exit this will be non-zero
      if the counter overflowed.

      \pre 0 <= pin < dio5200GetPinCount()  
   */
   int dio5200PollInputCapture(unsigned int pin, int* overflow);

   /**
      Return the input capture value for the pin.

      \param pin [in] The index of the pin of interest.  Should be
      between 0 and 1 less than the result of dio5200GetPinCount().

      \return the number of prescaled counts 

      \pre 0 <= pin < dio5200GetPinCount()
      \pre dio5200PollInputCapture() has returned non-zero since you called
      dio5200InputCaptureStart().
   */
   unsigned int dio5200GetInputCapture(unsigned int pin);

   /**
      Convert the specified number of ticks from the IPB clock domain
      to the system clock domain.

      The IPB clock is what clocks PWM and input capture.  This function
      converts that into ticks in the system clock domain, which
      drives the time base register.
    */
   unsigned int dio5200IPBToSysClk(unsigned int ticks);

   /**
      Convert the specified number of ticks from the system clock domain
      to the IP Bus clock domain.

      The IPB clock is what clocks PWM and input capture.  This function
      converts into that from ticks in the system clock domain, which
      drives the time base register.
    */
   unsigned int dio5200SysToIPBClk(unsigned int ticks);

#ifdef __cplusplus
}
#endif

#endif /* ___ni_arch_ppc_dio5200_h___ */
